//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <process.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include "..\..\..\include\driver.h"
#include "..\..\..\include\os.h"

#include "Unit1.h"
#include "Unit2.h"

//---------------------------------------------------------------------------

typedef struct _PT_ThreadArg
{
	LONG	m_lDriverHandle;
	HWND	m_hWnd;
//	PT_CheckEvent	ptCheckEvent;
} PT_ThreadArg, * LPT_ThreadArg;

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;

__fastcall TIntThread::TIntThread() :
    TThread(True), P833TimerINT(False)
{
}

__fastcall TIntThread::TIntThread(bool Status, bool TimerINT) :
    TThread(Status), P833TimerINT(TimerINT)
{
    PT_EnableEvent	EventSetting;

    // Free itself when it is terminated
    FreeOnTerminate = True;

    if (Form1->RadioButton7->Checked)
        Priority = tpNormal;
    else
        Priority = tpTimeCritical;

    P833DriverHandle = Form1->GetDriverHandle();
    P833BaseAddress = Form1->GetDeviceAddress();

    if (TimerINT) {
        // Setup Interrupt by Timer Control
        P833WritePortByte.port = (USHORT) (P833BaseAddress + 9);
        P833WritePortByte.ByteData = (USHORT) (Form1->GetTimeBase() | 0x08);
        if ((ErrCde=DRV_WritePortByte(P833DriverHandle, &P833WritePortByte))!= 0)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
        }
    }

    // ICW1	: Starts the initialization,
    // D0 & D1 = 1
    // IC4(D0) = 1 ==> ICW4 needed
    // SINGL = 1 ==> Not in CASCADE ==> skip ICW3
    P833WritePortByte.port = (USHORT) (P833BaseAddress + 12);
    P833WritePortByte.ByteData = 0x13;
    if ((ErrCde=DRV_WritePortByte(P833DriverHandle, &P833WritePortByte))!= 0)
    {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
    }

    // ICW2 : 3 ==> CH1 ZIN  (ref. p.10 On PCL-833 User's Manual)
    // 6 ==> for reference when read from BASE+15
    P833WritePortByte.port = (USHORT) (P833BaseAddress + 13);
    P833WritePortByte.ByteData = 0x63;
    if ((ErrCde=DRV_WritePortByte(P833DriverHandle, &P833WritePortByte))!= 0)
    {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
    }

    // ICW4 : 0b Hex = 0000 1011 Bin
    // AEOI  = 1 ==> Auto EOI
    // BUF/MS = 10 ==> Buffered mode / Slave
    P833WritePortByte.port = (USHORT) (P833BaseAddress + 13);
    P833WritePortByte.ByteData = 0x0b;
    if ((ErrCde=DRV_WritePortByte(P833DriverHandle, &P833WritePortByte))!= 0)
    {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
    }
    // OCW1 : 0xf7 (1111 0111) ==> enable CH1 ZIN interrupt
    // 0xfe (1111 1110) ==> enable CH1 overflow interrupt
    // 0xfd (1111 1101) ==> enable CH2 overflow interrupt
    // 0xfb (1111 1011) ==> enable CH3 overflow interrupt
    // 0xf7 (1111 0111) ==> enable CH1 index in interrupt
    // 0xef (1110 1111) ==> enable CH2 index in interrupt
    // 0xdf (1101 1111) ==> enable CH3 index in interrupt
    // 0xbf (1011 1111) ==> enable DI0 input    interrupt
    // 0x7f (0111 1111) ==> enable DI1 input/Timer Pulse interrupt
    // Interrupt Source is shown on p.10 of User's Manual
    P833WritePortByte.port = (USHORT) (P833BaseAddress + 13);
    // Enable all interrupt
    P833WritePortByte.ByteData = 0x00;
    if ((ErrCde=DRV_WritePortByte(P833DriverHandle, &P833WritePortByte))!= 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
    }

	EventSetting.EventType = ADS_EVT_INTERRUPT;
	EventSetting.Enabled = True;
    // number of interrupt count will be send event
	EventSetting.Count = 1;
	if ((ErrCde=DRV_EnableEvent(P833DriverHandle, &EventSetting))!= 0) {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
    }

    // Read Base+15 as sending out 8259 acknowledge
    // It should be 0x63 ~ 0x6A for the InterruptID
    P833ReadPortByte.port = (USHORT) (P833BaseAddress + 15);
    P833ReadPortByte.ByteData = &InterruptID;
}

//---------------------------------------------------------------------------

void __fastcall TIntThread::Execute(void)
{
    const int       BaseID = 0x63;

	DWORD	        dwStartTime;
	DWORD	        dwCurrentTime;
	DWORD	        dwTime;
    DWORD           dwTotalCount = 0;
	DWORD	        dwEventCount = 0;
	float	        ratio;
	char	        szBuf[20];
	USHORT	        usEventType;

    dwStartTime = GetTickCount();
 	ptCheckEvent.EventType = &usEventType;

    while (!Terminated) {
        if (DRV_CheckEvent(P833DriverHandle, &ptCheckEvent) == SUCCESS) {
            dwTotalCount++;
			dwEventCount++;
        }

		dwCurrentTime = GetTickCount();
		if ((dwTime = dwCurrentTime - dwStartTime) > 1000) {
            sprintf(szBuf, "%8d", dwTotalCount);
            Form1->Edit4->Text = szBuf;

			ratio = (float)(((float)dwEventCount / (float)dwTime) * 1000.0);
			sprintf(szBuf, "%.3f", ratio );
            dwStartTime = GetTickCount();
			dwEventCount = 0;

            Form1->Edit5->Text = szBuf;
		}
        // The parameters' value of P833ReadPortByte are set in TIntThread::TIntThread() function
        if ((ErrCde=DRV_ReadPortByte(P833DriverHandle, &P833ReadPortByte))!= 0)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
        }

        // Clear the interrupt of 8259 on PCL-833 card
        P833WritePortByte.port = (USHORT) (P833BaseAddress + 12);
        P833WritePortByte.ByteData = 0x20;
        if ((ErrCde=DRV_WritePortByte(P833DriverHandle, &P833WritePortByte))!= 0)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
        }

        // InterruptID = 0x63 ~ 0x6A
        switch (InterruptID) {
        case (BaseID+0):
            Form1->Edit6->Text = "CH1 Overflow";
            break;
        case (BaseID+1):
            Form1->Edit6->Text = "CH2 Overflow";
            break;
        case (BaseID+2):
            Form1->Edit6->Text = "CH3 Overflow";
            break;
        case (BaseID+3):
            Form1->Edit6->Text = "CH1 ZIN";
            break;
        case (BaseID+4):
            Form1->Edit6->Text = "CH2 ZIN";
            break;
        case (BaseID+5):
            Form1->Edit6->Text = "CH3 ZIN";
            break;
        case (BaseID+6):
            Form1->Edit6->Text = "DI0 Input";
            break;
        case (BaseID+7):
            if (!P833TimerINT)
                Form1->Edit6->Text = "DI1 Input";
            else
                Form1->Edit6->Text = "Timer Pulse";
            break;
        default:
            Form1->Edit6->Text = "Error Interrupt ID!";
    }
    }
}

//---------------------------------------------------------------------------

USHORT TIntThread::GetInterruptID(void)
{
    return (10);
}
//---------------------------------------------------------------------------

/*************************************************************************\
*  FUNCTION: GetKeyValue()
*  PURPOSE: gets individual entry from registry
*  COMMENTS:
*   Inputs :hkKey   - open registry key
*			pszName - name of key value to set
*           pItem   - Item to get
*           usNum   - indicate read byte number
*   Outputs:return() -FALSE - if no problem , TRUE -failed
*
\*************************************************************************/
BOOL TForm1::GetKeyValue(HKEY  hkReg, PCHAR pszName, LPVOID pItem, USHORT usNum)
{
    DWORD retCode;
    DWORD ValueLength;
    DWORD Type;
    DWORD Value;
    DWORD * Value_DWORD;
    USHORT * Value_USHORT;

    ValueLength = sizeof(REG_DWORD);
    Type = REG_DWORD;

    retCode = RegQueryValueEx(hkReg, pszName, 0,  // Put it in Registry
        &Type, (LPBYTE)(&Value), &ValueLength);

    if (retCode)
    {
        return(TRUE);
    }

    if (usNum == 2)
    {
        Value_USHORT = (USHORT *) pItem;
        *Value_USHORT = (USHORT)(Value & 0xffff);
    }
    else
    {
        Value_DWORD = (DWORD *) pItem;
        *Value_DWORD = (DWORD) Value;
    }

    return(FALSE);
}

//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
    bRun = FALSE;
    DriverHandle = (LONG) NULL;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::AddDevice1Click(TObject *Sender)
{
    char arg[30];
    SECURITY_ATTRIBUTES saProcess, saThread;
    STARTUPINFO si;
    PROCESS_INFORMATION piProcess;

    if (bRun) {
        Application->MessageBox("Please close device before run Device Installation utility",
             "Warning!", MB_OK);
        return;
    }

    saProcess.nLength = sizeof(saProcess);
    saProcess.lpSecurityDescriptor = NULL;
    saProcess.bInheritHandle = TRUE;

    saThread.nLength = sizeof(saThread);
    saThread.lpSecurityDescriptor = NULL;
    saThread.bInheritHandle = FALSE;

    ZeroMemory(&si, sizeof(si));
    si.cb = sizeof(si);

    if (CreateProcess("C:\\Program Files\\Advantech\\ADSAPI\\DEVINST.EXE",
                  NULL,
                  &saProcess,
                  &saThread,
                  False,
                  CREATE_DEFAULT_ERROR_MODE,
                  NULL,
                  NULL,
                  &si,
                  &piProcess)== 0) {
        sprintf(arg, "Error Code = %d.", GetLastError());
        Application->MessageBox(arg, "Warning!", MB_OK);
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Open1Click(TObject *Sender)
{
    ULONG       dwDeviceNum;
    char        szErrMsg[80];               // Use for MESSAGEBOX function
    LRESULT     ErrCde;                     // Return error code
    HKEY        hkReg;                      // For Reg key
    CHAR        szBuf[30];                  // temp buffer
    CHAR        szReg[256];                 // Reg name buffer

    ErrCde = DRV_SelectDevice(Handle, FALSE, &dwDeviceNum, szBuffer);
    if (ErrCde != SUCCESS) {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
    }

    // Step 2: Open Device
    ErrCde = DRV_DeviceOpen(dwDeviceNum, (LONG far *) &DriverHandle);
    if (ErrCde != SUCCESS) {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
    }
    else {
        // Flag for Device Open
        // ==> If Ture, you should perform DRV_DeviceClose() before leave the program
        bRun = TRUE;

        // Open ADSIO registry key
        strcpy(szReg, "SYSTEM\\CurrentControlSet\\Services\\ADSIO");
        strcat(szReg, "\\Device");
        wsprintf(szBuf, "%03d", dwDeviceNum);  // ulDeviceNum from addemo.c
        strcat(szReg, szBuf);
        if ((ErrCde = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szReg, 0,
            KEY_ENUMERATE_SUB_KEYS | KEY_EXECUTE | KEY_QUERY_VALUE, &hkReg))!= SUCCESS)
        {
            Application->MessageBox("Registry Query Error!", "Driver Message", MB_OK);
        }

        // Get Base Address of selected PCL-833
        if ((ErrCde = GetKeyValue(hkReg, "BaseAddress", &DeviceAddress, 2))
            != SUCCESS)
            Application->MessageBox("Registry Query Error (Base Address)!", "Driver Message", MB_OK);
        // Time Base setting of selected PCL-833
        if ((ErrCde = GetKeyValue(hkReg, "TimeBase", &TimeBase, 2))
            != SUCCESS)
            Application->MessageBox("Registry Query Error (Time Base)!", "Driver Message", MB_OK);
 
        // Enable Function and Initialize function since DeviceOpen successfully. 
        Function1->Enabled = TRUE;
        Initialize8331->Enabled = TRUE;

        Open1->Enabled = False;
        Close1->Enabled = TRUE;
        Initialize8331->Enabled = TRUE;
    }
    PageControl1->ActivePage = (TTabSheet *) TabSheet1;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Exit1Click(TObject *Sender)
{
    Timer1->Enabled = False;

    if (bRun) {
        // If user doesn't close the Device Driver, run DRV_DeviceClose() function
        DRV_DeviceClose(&DriverHandle);
        bRun = False;
    }

    AdvanceThread->Suspended = True;
    Application->Terminate();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
    if (bRun)
        // If user doesn't close the Device Driver, run DRV_DeviceClose() function
        DRV_DeviceClose(&DriverHandle);
}
//---------------------------------------------------------------------------


void __fastcall TForm1::About1Click(TObject *Sender)
{
    About->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::OneChannel1Click(TObject *Sender)
{
    OneChannel1->Checked = True;
    TwoChannel1->Checked = False;
    ThreeChannel1->Checked = False;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::TwoChannel1Click(TObject *Sender)
{
    OneChannel1->Checked = False;
    TwoChannel1->Checked = True;
    ThreeChannel1->Checked = False;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ThreeChannel1Click(TObject *Sender)
{
    OneChannel1->Checked = False;
    TwoChannel1->Checked = False;
    ThreeChannel1->Checked = True;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Base1Click(TObject *Sender)
{
    Timer1->Enabled = False;
    QCounterConfig(0);
    QCounterConfig(1);
    QCounterConfig(2);

    QCounterStart(0);
    QCounterStart(1);
    QCounterStart(2);

    Button4->Caption = "Disable";
    Timer1->Enabled = True;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Advance1Click(TObject *Sender)
{
    Timer1->Enabled = False;

    // Create the thread to wait for Interrupt Event from PCL-833
    if (bRun)
        AdvanceThread = new TIntThread(False, True);

    Button5->Caption = "Disable";

    PageControl1->ActivePage = (TTabSheet *) TabSheet2;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::WithPCL8391Click(TObject *Sender)
{
    Timer1->Enabled = False;
    PageControl1->ActivePage = (TTabSheet *) TabSheet3;
}
//---------------------------------------------------------------------------


void __fastcall TForm1::FormCreate(TObject *Sender)
{
    usOverflow  = 0;     // counter over 32 bit flag
    ulLoCount   = 0;     // low byte of counter
    ulHiCount   = 0;     // high byte of counter

    ptQCounterRead.overflow = (USHORT far *) &usOverflow;
    ptQCounterRead.LoCount  = (ULONG  far *) &ulLoCount;
    ptQCounterRead.HiCount  = (ULONG  far *) &ulHiCount;

    gwValue = 0;        // Digital input value (DI1, DI0, CH3 ZIN, CH2 ZIN, CH3 ZIN)
    ptDioReadPortByte.port = 0;
    ptDioReadPortByte.value = (USHORT far *)&gwValue;

    // FormHelp is assigned to the OnHelp event handler of Application in
    // the OnCreate event of Form1.
    Application->OnHelp = FormHelp;
    Application->HelpFile = "C:\\Program Files\\Advantech\\ADSAPI\\AD833.HLP";

    // Add Input Mode to ComboBox1, ComboBox3 and ComboBox5
    // DISABLE             0       ==> disable mode
    // ABPHASEX1           1       ==> Quadrature input X1
    // ABPHASEX2           2       ==> Quadrature input X2
    // ABPHASEX4           3       ==> Quadrature input X4
    // TWOPULSEIN          4       ==> 2 pulse input
    // ONEPULSEIN          5       ==> 1 pulse input

    ComboBox1->Items->Add("DISABLE");
    ComboBox1->Items->Add("AB PHASE X1");
    ComboBox1->Items->Add("AB PHASE X2");
    ComboBox1->Items->Add("AB PHASE X4");
    ComboBox1->Items->Add("2-PULSE IN");
    ComboBox1->Items->Add("1-PULSE IN");
    // Display first item in the Combo Box
    ComboBox1->ItemIndex = 0;

    ComboBox3->Items->Add("DISABLE");
    ComboBox3->Items->Add("AB PHASE X1");
    ComboBox3->Items->Add("AB PHASE X2");
    ComboBox3->Items->Add("AB PHASE X4");
    ComboBox3->Items->Add("2-PULSE IN");
    ComboBox3->Items->Add("1-PULSE IN");
    // Display first item in the Combo Box
    ComboBox3->ItemIndex = 0;

    ComboBox5->Items->Add("DISABLE");
    ComboBox5->Items->Add("AB PHASE X1");
    ComboBox5->Items->Add("AB PHASE X2");
    ComboBox5->Items->Add("AB PHASE X4");
    ComboBox5->Items->Add("2-PULSE IN");
    ComboBox5->Items->Add("1-PULSE IN");
    // Display first item in the Combo Box
    ComboBox5->ItemIndex = 0;

    // Add Latch Source to ComboBox2, ComboBox4 and ComboBox6
    // SWLATCH             0       ==> S/W read latch data
    // INDEXINLATCH        1       ==> Index-in latch data
    // DI0LATCH            2       ==> DI0 latch data
    // DI1LATCH            3       ==> DI1 latch data
    // TIMERLATCH          4       ==> Timer latch data

    ComboBox2->Items->Add("S/W LATCH");
    ComboBox2->Items->Add("ZIN LATCH");
    ComboBox2->Items->Add("DI0 LATCH");
    ComboBox2->Items->Add("DI1 LATCH");
    ComboBox2->Items->Add("TIMER LATCH");
    // Display first item in the Combo Box
    ComboBox2->ItemIndex = 0;

    ComboBox4->Items->Add("S/W LATCH");
    ComboBox4->Items->Add("ZIN LATCH");
    ComboBox4->Items->Add("DI0 LATCH");
    ComboBox4->Items->Add("DI1 LATCH");
    ComboBox4->Items->Add("TIMER LATCH");
    // Display first item in the Combo Box
    ComboBox4->ItemIndex = 0;

    ComboBox6->Items->Add("S/W LATCH");
    ComboBox6->Items->Add("ZIN LATCH");
    ComboBox6->Items->Add("DI0 LATCH");
    ComboBox6->Items->Add("DI1 LATCH");
    ComboBox6->Items->Add("TIMER LATCH");
    // Display first item in the Combo Box
    ComboBox6->ItemIndex = 0;

    Shape1->ShowHint = true;
    Shape2->ShowHint = true;
    Shape3->ShowHint = true;
    Shape4->ShowHint = true;
    Shape5->ShowHint = true;

    Button4->Caption = "Enable";

    AdvanceThread = new TIntThread();
}
//---------------------------------------------------------------------------

bool __fastcall TForm1::FormHelp(WORD Command, int Data, bool &CallHelp)
{
    return True;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Conetent1Click(TObject *Sender)
{
    Application->HelpCommand(HELP_CONTENTS, 0);
}
//---------------------------------------------------------------------------

void    TForm1::QCounterConfig(USHORT wChannel)
{
    PT_QCounterConfig  ptQCounterConfig;

    switch (wChannel) {
        case 0:  // Setup channel 1
            // Configure Counter #1, Counter #2 and Counter #3 depending on the
            // Basic Tag's setup
            ptQCounterConfig.counter       = 0;
            ptQCounterConfig.LatchSrc      = (USHORT) ComboBox2->ItemIndex;
            ptQCounterConfig.LatchOverflow = (USHORT) CheckBox1->State;
            ptQCounterConfig.ResetOnLatch  = (USHORT) CheckBox2->State;
            if (RadioButton1->Checked)
                ptQCounterConfig.ResetValue = 0;
            else
                ptQCounterConfig.ResetValue = 1;
            break;

        case 1:  // Setup channel 2
            ptQCounterConfig.counter       = 1;
            ptQCounterConfig.LatchSrc      = (USHORT) ComboBox4->ItemIndex;
            ptQCounterConfig.LatchOverflow = (USHORT) CheckBox3->State;
            ptQCounterConfig.ResetOnLatch  = (USHORT) CheckBox4->State;
            if (RadioButton3->Checked)
                ptQCounterConfig.ResetValue = 0;
            else
                ptQCounterConfig.ResetValue = 1;
            break;

        case 2:  // Setup channel 3
            ptQCounterConfig.counter       = 2;
            ptQCounterConfig.LatchSrc      = (USHORT) ComboBox6->ItemIndex;
            ptQCounterConfig.LatchOverflow = CheckBox5->State;
            ptQCounterConfig.ResetOnLatch  = CheckBox6->State;
            if (RadioButton5->Checked)
                ptQCounterConfig.ResetValue = 0;
            else
                ptQCounterConfig.ResetValue = 1;
            break;
            
        default:
            Application->MessageBox("Please type in correct channel!", "Warning Message", MB_OK);
    }
    // Execute counter config
    if((ErrCde = DRV_QCounterConfig(DriverHandle,
        (LPT_QCounterConfig)&ptQCounterConfig)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
        return;
    }

    return;
}

//---------------------------------------------------------------------------

void    TForm1::QCounterStart(USHORT wChannel)
{
    PT_QCounterStart    ptQCounterStart;
    USHORT              usInputMode;

    switch (wChannel) {
        case 0:  // Setup channel 1
            usInputMode = (USHORT) ComboBox1->ItemIndex;
            break;
        case 1:  // Setup channel 2
            usInputMode = (USHORT) ComboBox3->ItemIndex;
            break;
        case 2:  // Setup channel 3
            usInputMode = (USHORT) ComboBox5->ItemIndex;
            break;
        default:
            Application->MessageBox("Please setup proper InputMode in Basic page!", "Warning Message", MB_OK);
            break;
    }
    ptQCounterStart.counter   = wChannel;
    ptQCounterStart.InputMode = usInputMode;

    // Execute counter start
    if((ErrCde = DRV_QCounterStart(DriverHandle,
        (LPT_QCounterStart)&ptQCounterStart)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
        return;
    }
}
//---------------------------------------------------------------------------

LONG TForm1::GetDriverHandle(void)
{
    return DriverHandle;
}

//---------------------------------------------------------------------------

USHORT TForm1::GetDeviceAddress(void)
{
    return DeviceAddress;
}

//---------------------------------------------------------------------------

USHORT TForm1::GetTimeBase(void)
{
    return TimeBase;
}
//---------------------------------------------------------------------------

void TForm1::CounterReset(USHORT wChannel)
{
    if( (ErrCde = DRV_CounterReset(DriverHandle, wChannel)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
        return;
    }
    return;
}

//---------------------------------------------------------------------------

void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
    if ((ErrCde = DRV_DioReadPortByte(DriverHandle,
        (LPT_DioReadPortByte)&ptDioReadPortByte)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
        Timer1->Enabled = False;
        return;
    }

    if (gwValue & 0x10) // DI 1
        Shape1->Brush->Color = clRed;
    else
        Shape1->Brush->Color = clWhite;

    if (gwValue & 0x08) // DI 0
        Shape2->Brush->Color = clRed;
    else
        Shape2->Brush->Color = clWhite;

    if (gwValue & 0x04) // CH3 ZIN
        Shape3->Brush->Color = clRed;
    else
        Shape3->Brush->Color = clWhite;

    if (gwValue & 0x02) // CH2 ZIN
        Shape4->Brush->Color = clRed;
    else
        Shape4->Brush->Color = clWhite;

    if (gwValue & 0x01) // CH1 ZIN
        Shape5->Brush->Color = clRed;
    else
        Shape5->Brush->Color = clWhite;

    for (int i=0; i<3; i++)
    {
        ptQCounterRead.counter = (USHORT) i;
        // Execute Quadrature Counter reading
        if((ErrCde = DRV_QCounterRead(DriverHandle,
            (LPT_QCounterRead) &ptQCounterRead)) != 0)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
            Timer1->Enabled = False;
            return;
        }

        ultoa((unsigned long)ulLoCount,(char *)szBuf1,  10);
        ultoa((unsigned long)ulHiCount,(char *)szBuf2, 10);
        sprintf(szBuffer, "%18s : %10s", szBuf1, szBuf2, usOverflow);

        switch (i) {
            case 0:  // value of Counter Channel 1
                Edit1->Text = szBuffer;                       // low byte of counter
                break;
            case 1:  // value of Counter Channel 2
                Edit2->Text = szBuffer;
                break;
            case 2:  // value of Counter Channel 3
                Edit3->Text = szBuffer;
                break;
            default:
                Application->MessageBox("Please setup proper InputMode in Basic page!", "Warning Message", MB_OK);
                break;
        }
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button1Click(TObject *Sender)
{
    // counter reset
    CounterReset(0);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button2Click(TObject *Sender)
{
    // counter reset
    CounterReset(1);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button3Click(TObject *Sender)
{
    // counter reset
    CounterReset(2);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Close1Click(TObject *Sender)
{
    Timer1->Enabled = False;
    Function1->Enabled = False;
    if (bRun)
        // If user doesn't close the Device Driver, run DRV_DeviceClose() function
        DRV_DeviceClose(&DriverHandle);
    bRun = False;

    Open1->Enabled = True;
    Close1->Enabled = False;
    Initialize8331->Enabled = False;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Initialize8331Click(TObject *Sender)
{
    CounterReset(0);
    QCounterConfig(0);
    QCounterStart(0);

    CounterReset(1);
    QCounterConfig(1);
    QCounterStart(1);

    CounterReset(2);
    QCounterConfig(2);
    QCounterStart(2);
    return;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button4Click(TObject *Sender)
{
    if (!bRun)     // No Device open 
        return;

    if (Timer1->Enabled) {
        Button4->Caption = "Enable";
        Timer1->Enabled = False;
    }
    else {
        Button4->Caption = "Disable";
        Timer1->Enabled = True;

        // counter reset
        CounterReset(0);
        CounterReset(1);
        CounterReset(2);
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button5Click(TObject *Sender)
{
    PT_EnableEvent	EventSetting;

    EventSetting.EventType = ADS_EVT_INTERRUPT;

    if (!bRun)     // No Device open
        return;

    if (AdvanceThread->Suspended) {
        Button5->Caption = "Disable";
        if (RadioButton7->Checked)
            AdvanceThread->Priority = tpNormal;
        else
            AdvanceThread->Priority = tpTimeCritical;

        EventSetting.Enabled = True;
        // number of interrupt count will be send event
	    EventSetting.Count = 1;
	    if ((ErrCde=DRV_EnableEvent(DriverHandle, &EventSetting))!= 0) {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
        }
        AdvanceThread->Resume();
    }
    else {
        Button5->Caption = "Enable";
        EventSetting.Enabled = False;
        // number of interrupt count will be send event
	    EventSetting.Count = 1;
	    if ((ErrCde=DRV_EnableEvent(DriverHandle, &EventSetting))!= 0) {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            Application->MessageBox((char *)szErrMsg, "Driver Message", MB_OK);
        }
        AdvanceThread->Suspend();
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormDestroy(TObject *Sender)
{
    AdvanceThread->Terminate();    
}
//---------------------------------------------------------------------------


