//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include <math.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TFrmMain *FrmMain;

int ErrCde;                     // Return error code
char szErrMsg[80];              // Use for MESSAGEBOX function
LONG DriverHandle = (LONG)NULL; // driver handle
ULONG dwDeviceNum;              // Device number
char szBuffer[40];              // Temperatory buffer
BOOL bRunTimer = FALSE;
PT_FAOScale ptFAOScale;         // FAOScale table
USHORT far * lpBuf[4] = {NULL,NULL,NULL,NULL};
FLOAT far * lpVoltageBuf[4] = {NULL,NULL,NULL,NULL};
HGLOBAL hBuf[4], hVoltageBuf[4];
USHORT far * lpCommonBuf;
HGLOBAL	hCommonBuf;
HANDLE hThreadHandle;
DWORD dwThreadID;
BOOL bThreadloop = FALSE;
BOOL bThreadflag = FALSE;
char szDescript[100];           // Description device
USHORT gwSelectDevice = FALSE;  // have never selected "Select Device" button
DWORD gdwPacerRate = 10000;     // pacer rate
ULONG gulConvNum = 2048;        // conversion number
ULONG gwWaveFormCount = 30;     // waveform count
USHORT gwExtTrig = 0;           // external or internal trigger
USHORT gwEvtFlag = 1;           // event enable(0)

void SetRealBuffer(float far *lpBuf, long num, LPSWAVE lpWave)
{
    int i, r, half;
    float slope;

    switch (lpWave->wWaveform)
    {
        case WAVE_SINE:
            for (i = 0; i < num; i ++)
            {
                *(lpBuf+i) = (float)(lpWave->fMagnitude
                           * sin(6.2831853*(double)i/(double)lpWave->wPeriod)
                           + lpWave->fOffset);
            }
            break;
        case WAVE_TRIANGLE:
            slope = lpWave->fMagnitude * 4.0f / (float)lpWave->wPeriod;
            for (i = 0; i < num; i ++)
            {
                r = i % lpWave->wPeriod;
                half = (int)((float)lpWave->wPeriod / 2.0f);
                if (r <= half)
                    *(lpBuf + i) = slope * r - lpWave->fMagnitude
                                 + lpWave->fOffset;
                else
                    *(lpBuf + i) = slope * (lpWave->wPeriod - r)
                                 - lpWave->fMagnitude + lpWave->fOffset;
            }
            break;
        case WAVE_SQUARE:
            for (i = 0; i < num; i ++)
            {
                r = i % lpWave->wPeriod;
                half = (int)((float)lpWave->wPeriod / 2.0f);
                if (r <= half)
                    *(lpBuf + i) = lpWave->fOffset - lpWave->fMagnitude;
                else
                    *(lpBuf + i) = lpWave->fOffset + lpWave->fMagnitude;
            }
            break;
        case SINE_TRIANGLE:
        {
            ULONG x;
	    double y;

            slope = lpWave->fMagnitude * 4.0f / (float)lpWave->wPeriod;
            for (i = 0; i < num; i ++)
	    {
		if ( i%2)
		{
		    modf(((double)i/2.0),(double *)&y);
		    y = y + 1;
                    r = ((USHORT)y) % lpWave->wPeriod;
                    half = (int)((float)lpWave->wPeriod / 2.0f);
                    if (r <= half)
                        *(lpBuf + i) = slope * r - lpWave->fMagnitude
                                     + lpWave->fOffset;
                    else
                        *(lpBuf + i) = slope * (lpWave->wPeriod - r)
                                     - lpWave->fMagnitude + lpWave->fOffset;
                }
		else
		{
		    x = i/2;
		    *(lpBuf+i) = (float)(lpWave->fMagnitude
                               * sin(6.28318*(double)x/(double)lpWave->wPeriod)
                               + lpWave->fOffset);
                }
            }
            break;
        }
    }
}

void SetMultiToOneBuffer(USHORT usEnabledChannel, int count)
{
    int i,TotalCount;

    TotalCount=0;

    for (i = 0; i < count; i ++)
    {
    	if(usEnabledChannel & ADV_CHANNEL0)
    	{
            ((USHORT far *)lpCommonBuf)[TotalCount] =
	    	((USHORT)((USHORT far *) lpBuf[0])[i]) & 0x0fff;
	    TotalCount++;
        }
	if(usEnabledChannel & ADV_CHANNEL1)
        {
	    ((USHORT far *)lpCommonBuf)[TotalCount] =
	    	((USHORT)((USHORT far *) lpBuf[1])[i] | ( 0x01 << 12)) & 0x3fff;
	    TotalCount++;
	}
	if(usEnabledChannel & ADV_CHANNEL2)
	{
	    ((USHORT far *)lpCommonBuf)[TotalCount] =
	    	((USHORT)((USHORT far *) lpBuf[2])[i] | ( 0x02 << 12)) & 0x3fff;
	    TotalCount++;
	}
	if(usEnabledChannel & ADV_CHANNEL3)
	{
	    ((USHORT far *)lpCommonBuf)[TotalCount] =
	        ((USHORT)((USHORT far *) lpBuf[3])[i] | ( 0x03 << 12)) & 0x3fff;
            TotalCount++;
	}
    }
}

void MyFreeBuffer()
{
    int i;

    for(i=0; i<4; i++)
    {
        if(hBuf[i])
       	{
	    GlobalFree(hBuf[i]);
	    hBuf[i] = NULL;
	}
	if(hVoltageBuf[i])
	{
	    GlobalFree(hVoltageBuf[i]);
	    hVoltageBuf[i] = NULL;
	}
    }
}

void adInterruptEvent()
{
    return;
}

void adBufChangeEvent()
{
    return;
}

void adOverrunEvent()
{
    return;
}

void adTerminateEvent()
{
    bThreadloop = FALSE;

    // Disable stop button
    FrmMain->Stop->Enabled = FALSE;

    return;
}

void UserThread()
{
    USHORT usEventType;
    PT_CheckEvent ptCheckEvent;   // Check event

    while(bThreadloop)
    {
        // Check message
        ptCheckEvent.EventType = &usEventType;
        ptCheckEvent.Milliseconds = 100;

       	bThreadflag = TRUE;
        DRV_CheckEvent(DriverHandle, (LPT_CheckEvent)&ptCheckEvent);

        // Process interrupt event
        if (usEventType & ADS_EVT_INTERRUPT)
            adInterruptEvent();

        // Process buffer change event
        if (usEventType & ADS_EVT_BUFCHANGE)
            adBufChangeEvent();

        // Process overrun event
        if (usEventType & ADS_EVT_OVERRUN)
            adOverrunEvent();

        // Process terminate event
        if (usEventType & ADS_EVT_TERMINATED)
            adTerminateEvent();

        bThreadflag = FALSE;
    }
}

//---------------------------------------------------------------------------

__fastcall TFrmMain::TFrmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::WaveformClick(TObject *Sender)
{
    Application->CreateForm(__classid(TFrmWave), &FrmWave);
    FrmWave->Show();        
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::FormClose(TObject *Sender, TCloseAction &Action)
{
    MyFreeBuffer();
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::RunClick(TObject *Sender)
{
    int i;
    PT_EnableEvent ptEnableEvent;             // Enable event
    PT_FAOWaveFormStart ptFAOWaveFormStart;   // FAODMAStart table
    
    // Step 1: Device open
    ErrCde = DRV_DeviceOpen(dwDeviceNum, (LONG far *)&DriverHandle);
    if (ErrCde != SUCCESS)
    {
        DRV_GetErrorMessage(ErrCde, szErrMsg);
        MessageBox(Handle, szErrMsg, "Device Open", MB_OK);
        return;
    }

    // Step 2: Allocate buffer and set data to buffer
    // Process 4 channel
    usChannelCount = 0;    // reset channel count
    usEnabledChannel = 0;
    for(i=0; i<4; i++)
    {
    	// Allocate memory used by driver
	if((hBuf[i]=(USHORT far *)GlobalAlloc(GPTR,
	    sizeof(USHORT) * gulConvNum)) == 0)
	{
	    MessageBox(Handle, "Not enough memory for buffer", "High Speed", MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
            return;
        }

	// Allocate memory for real voltage
	if((hVoltageBuf[i]=(FLOAT far *)GlobalAlloc(GPTR,
            sizeof(FLOAT) * gulConvNum)) == 0)
	{
	    MessageBox(Handle, "Not enough memory for buffer", "High Speed", MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
	    return;
        }

	lpBuf[i] = (USHORT far *)hBuf[i];
	lpVoltageBuf[i] = (FLOAT far  *)hVoltageBuf[i];

	switch(i)
	{
	    case 0 :
	        if(sWaveCh0.wWaveform != 4)  // 4 --> No Waveform
		{
		    usEnabledChannel |= ADV_CHANNEL0;
		    // set real voltage to hVoltageBuf
		    SetRealBuffer(lpVoltageBuf[0], gulConvNum, &sWaveCh0);
		    usChannelCount++;
		}
		else
		    continue;

                break;
	    case 1 :
		if(sWaveCh1.wWaveform != 4)
		{
		    usEnabledChannel |= ADV_CHANNEL1;
		    SetRealBuffer(lpVoltageBuf[1], gulConvNum, &sWaveCh1);
		    usChannelCount++;
                }
		else
		    continue;

                break;
	    case 2 :
		if(sWaveCh2.wWaveform != 4)
		{
		    usEnabledChannel |= ADV_CHANNEL2;
		    SetRealBuffer(lpVoltageBuf[2], gulConvNum, &sWaveCh2);
		    usChannelCount++;
		}
		else
		    continue;

                break;
	    case 3 :
		if(sWaveCh3.wWaveform != 4)
		{
		    usEnabledChannel |= ADV_CHANNEL3;
		    SetRealBuffer(lpVoltageBuf[3], gulConvNum, &sWaveCh3);
		    usChannelCount++;
		}
		else
		    continue;

		break;
        }

	// call FAOScale for transfer voltage to binary data
	ptFAOScale.VoltArray = lpVoltageBuf[i];
	ptFAOScale.BinArray  = lpBuf[i];
	ptFAOScale.chan      = i;
	ptFAOScale.count     = gulConvNum;
	if ((ErrCde = DRV_FAOScale(DriverHandle, (LPT_FAOScale)&ptFAOScale)) != 0)
        {
	    DRV_GetErrorMessage(ErrCde, szErrMsg);
	    MessageBox(Handle, szErrMsg, "Driver Message", MB_OK);
            MyFreeBuffer();
	    DRV_DeviceClose((LONG far *)&DriverHandle);
	    return;
	}
    }

    // Allocate memory for common buffer of 4 channel
    if((hCommonBuf=(USHORT far *)GlobalAlloc(GHND, 
        sizeof(USHORT) * gulConvNum * usChannelCount)) == 0)
    {
	MessageBox(Handle, "Not enough memory for buffer", "High Speed", MB_OK);
        MyFreeBuffer();
	DRV_DeviceClose((LONG far *)&DriverHandle);
	return;
    }

    // Lock down buffer
    if((lpCommonBuf = (USHORT far *)GlobalLock(hCommonBuf)) == 0)
    {
    	MessageBox(Handle, "Not enough memory for buffer ", "High Speed", MB_OK);
        MyFreeBuffer();
	GlobalFree(hCommonBuf);
	DRV_DeviceClose((LONG far *)&DriverHandle);
	return;
    }

    // set 4 buffer to 1 common buffer
    SetMultiToOneBuffer(usEnabledChannel,gulConvNum);

    // Step 3: Enable event feature
    ptEnableEvent.EventType = ADS_EVT_INTERRUPT  |
                              ADS_EVT_BUFCHANGE  |
                              ADS_EVT_TERMINATED |
                              ADS_EVT_OVERRUN;
    ptEnableEvent.Enabled = gwEvtFlag;
    ptEnableEvent.Count   = 1;
    if ((ErrCde = DRV_EnableEvent(DriverHandle,
        (LPT_EnableEvent)&ptEnableEvent)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(Handle, szErrMsg, "Driver Message", MB_OK);
        MyFreeBuffer(); 
	DRV_DeviceClose((LONG far *)&DriverHandle);
        return;
    }

    // Step 4: call FAOWaveFormStart for start action
    // Default setting - 
    // conversion number = 2048 * 4
    // Enabled Channel = 0,1,2,3
    // pacer rate = 1000 Hz
    // output waveform number = 30

    ptFAOWaveFormStart.TrigSrc    = gwExtTrig;			// triggering
    ptFAOWaveFormStart.SampleRate = gdwPacerRate;		// pacer rate = 1K
    ptFAOWaveFormStart.Count      = gulConvNum * usChannelCount;// DA conversion number
    ptFAOWaveFormStart.WaveCount  = gwWaveFormCount;		// Waveform number
    ptFAOWaveFormStart.Buffer     = (USHORT far *)lpCommonBuf;  // analog output data
    ptFAOWaveFormStart.EnabledChannel     = usEnabledChannel;   // which channel

    if ((ErrCde = DRV_FAOWaveFormStart(DriverHandle,
        (LPT_FAOWaveFormStart)&ptFAOWaveFormStart)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(Handle, szErrMsg, "Driver Message", MB_OK);
        MyFreeBuffer();
	GlobalUnlock(hCommonBuf);
	GlobalFree(hCommonBuf);
	DRV_DeviceClose((LONG far *)&DriverHandle);
        return;
    }

    // if event feature is enabled , then create one thread.
    if (gwEvtFlag)
    {
        hThreadHandle = CreateThread(0, 0, (LPTHREAD_START_ROUTINE) UserThread,
            0, 0, (LPDWORD)&dwThreadID);
	bThreadloop = TRUE;
    }

    Run->Enabled = FALSE;
    Stop->Enabled = TRUE;     // Enable stop buttons

    // if event feature is enabled 
    // Start up timer
    //
    if (gwEvtFlag)
    {
        Tmr->Enabled = TRUE;
	bRunTimer = TRUE;
    }        
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::StopClick(TObject *Sender)
{
    BOOL bflag = TRUE;
    
    bThreadloop = FALSE;	// disable thread loop
    Sleep(500);

    if(!gwExtTrig)
    {
    	while(bflag)
	{
	    if(bThreadflag==TRUE)
	    	Sleep(10);
	    else
	    	bflag = FALSE;
	}
    }

    // Stop Timer
    if(bRunTimer)
    {
	Tmr->Enabled = FALSE;
	bRunTimer = FALSE;
    }

    Stop->Enabled = FALSE;     // Disable stop buttons
    CloseHandle(hThreadHandle);     // close thread handle
    Run->Enabled = TRUE;

    // Step 1: Stop D/A conversion 
    if ((ErrCde = DRV_FAOStop(DriverHandle)) != 0)
    {
        DRV_GetErrorMessage(ErrCde, szErrMsg);
        MessageBox(Handle, szErrMsg, "Driver Message", MB_OK);
        MyFreeBuffer();
	GlobalUnlock(hCommonBuf);
	GlobalFree(hCommonBuf);
	DRV_DeviceClose((LONG far *)&DriverHandle);
        return;
    }

    // Step 2: Free buffer
    MyFreeBuffer();
    GlobalUnlock(hCommonBuf);
    GlobalFree(hCommonBuf);

    // Step 3: Close driver
    DRV_DeviceClose((LONG far *)&DriverHandle);
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::TmrTimer(TObject *Sender)
{
    PT_FAOCheck ptFAOCheck;
    USHORT gwActiveBuf = 0;     // return by FAOCheck
    USHORT gwStopped   = 0;     // return by FAOCheck
    ULONG  ulCurrentCount = 0;  // return by FAOCheck
    USHORT gwOverrun   = 0;     // return by FAOCheck
    USHORT gwHalfReady = 0;     // return by FAOCheck

    // Get transfer status
    ptFAOCheck.ActiveBuf    = &gwActiveBuf;
    ptFAOCheck.stopped      = &gwStopped;
    ptFAOCheck.CurrentCount = &ulCurrentCount;
    ptFAOCheck.overrun      = &gwOverrun;
    ptFAOCheck.HalfReady    = &gwHalfReady;

    if ((ErrCde = DRV_FAOCheck(DriverHandle,(LPT_FAOCheck)&ptFAOCheck)) != 0)
    {
    	DRV_GetErrorMessage(ErrCde, szErrMsg);
	MessageBox(Handle, szErrMsg, "Driver Message", MB_OK);
	MyFreeBuffer();
	GlobalUnlock(hCommonBuf);
	GlobalFree(hCommonBuf);
	DRV_DeviceClose((LONG far *)&DriverHandle);
	return;
    }

    // Display Data
    WaveCountText->Text = ulCurrentCount;

    // if thread finish run stop instruction
    if(!bThreadloop)
    {
	// if A/O process finish --> call DRV_FAOStop
	if(gwStopped)
	{
            // Step 1: Stop D/A conversion for high speed
	    if ((ErrCde = DRV_FAOStop(DriverHandle)) != 0)
	    {
	    	DRV_GetErrorMessage(ErrCde, szErrMsg);
		MessageBox(Handle, szErrMsg, "Thread Message", MB_OK);
	    }

            // Step 2: Free buffer
	    MyFreeBuffer();
	    GlobalUnlock(hCommonBuf);
	    GlobalFree(hCommonBuf);

	    // Step 3: Close driver
	    DRV_DeviceClose((LONG far *)&DriverHandle);

	    // Stop Timer
	    if(bRunTimer)
	    {
	    	Tmr->Enabled = FALSE;
                bRunTimer = FALSE;
            }

            Run->Enabled = TRUE;
            CloseHandle(hThreadHandle);
        }
    }
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::FormCreate(TObject *Sender)
{
    if (!gwSelectDevice)
    {
        // Select Device
        ErrCde = DRV_SelectDevice(Handle, FALSE, &dwDeviceNum, szDescript);
        if (ErrCde != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde, szErrMsg);
            MessageBox(Handle, szErrMsg, "Driver Message", MB_OK);
            return;
        }

        gwSelectDevice = TRUE;
        FrmMain->Run->Enabled = TRUE;
    }

    // Show device description to Button text
    DevSel->Caption = szDescript;

    // Initialize Conversion Number
    ltoa(gulConvNum, szBuffer, 10);
    Conv->Text = szBuffer;

    // Initialize Pacer Rate
    ltoa(gdwPacerRate, szBuffer, 10);
    Pacer->Text = szBuffer;

    // Initialize wave form count
    if(gwWaveFormCount == INFINITE) 
        strcpy(szBuffer,"INFINITE");
    else
    	ltoa(gwWaveFormCount, szBuffer, 10);
    WaveCount->Text = szBuffer;

    // Initialize Triggering
    if(gwExtTrig)
        ExtTrig->Checked = True;
    else
        IntTrig->Checked = True;

    // Initialize Message Flag
    if(gwEvtFlag)
        EvtEnable->Checked = True;
    else
        EvtDisable->Checked = True;        
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::DevSelClick(TObject *Sender)
{
    // Select Device
    ErrCde = DRV_SelectDevice(Handle, FALSE, &dwDeviceNum, szDescript);
    if (ErrCde != SUCCESS)
    {
        DRV_GetErrorMessage(ErrCde,szErrMsg);
        MessageBox(Handle, szErrMsg, "Driver Message", MB_OK);
        return;
    }

    // Show device description to Button text
    DevSel->Caption = szDescript;        
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::PacerChange(TObject *Sender)
{
    strcpy(szBuffer, Pacer->Text.c_str());
    gdwPacerRate = atol(szBuffer);        
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::ConvChange(TObject *Sender)
{
    strcpy(szBuffer, Conv->Text.c_str());
    gulConvNum = atol(szBuffer);
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::WaveCountChange(TObject *Sender)
{
    strcpy(szBuffer, WaveCount->Text.c_str());
    if(!strcmp(szBuffer,"INFINITE"))
        gwWaveFormCount = INFINITE;
    else
        gwWaveFormCount = atol(szBuffer);        
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::IntTrigClick(TObject *Sender)
{
    gwExtTrig = 0;        
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::ExtTrigClick(TObject *Sender)
{
    gwExtTrig = 1;        
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::EvtEnableClick(TObject *Sender)
{
    gwEvtFlag = 1;        
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::EvtDisableClick(TObject *Sender)
{
    gwEvtFlag = 0;        
}
//---------------------------------------------------------------------------

