/*
 ****************************************************************************************
 * Program        : FREQ.CPP                                                            *
 * Description    : Demo program for frequency measurement function                     *
 * Boards Supp.   :                                                                     *
 * APIs used      : DRV_DeviceOpen,DRV_DeviceClose, DRV_GetErrorMessage,                *
 *                  DRV_CounterReset, DRV_CounterFreqStart, DRV_CounterFreqRead         *
 * Revision       : 1.00                                                                *
 * Date           : 7/8/1999                   Advantech Co., Ltd.                      *
 ****************************************************************************************
 */
#include <windows.h>
#include <windef.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

#define WIN_CONSOLE
#include "..\..\..\..\include\driver.h"

/******************************
 * Local function declaration *
 ******************************/
void ErrorHandler(DWORD dwErrCde);
void ErrorStop(long*, DWORD);

void main()
{
    DWORD  dwErrCde;
    ULONG  lDevNum;
    long   lDriverHandle;

    USHORT usChan;									// input channel          
	float  fFreq = 0.0; 
	PT_CounterFreqStart tCounterFreqStart;
	PT_CounterFreqRead  tCounterFreqRead;

    //Step 1: Display hardware and software settings for running this example
    printf("Before running this example, please\n");
    printf("use the device installation utility to add the device.\n");

    //Step 2: Input parameters
    printf("\nPlease input parameters:");
    printf("\nDevice Number (check the device installation utility): ");
    scanf("%d", &lDevNum);
    printf("\nInput Channel: ");
    scanf("\n%d", &usChan);

    //Step 3: Open device
    dwErrCde = DRV_DeviceOpen(lDevNum, &lDriverHandle);   
    if (dwErrCde != SUCCESS)
    {
        ErrorHandler(dwErrCde);
        printf("Program terminated!\n");

        printf("Press any key to exit....");
        getch();
        exit(1);
    }

    // Step 4: Reset counter by DRV_CounterReset
	dwErrCde = DRV_CounterReset(lDriverHandle, usChan);
	if (dwErrCde != SUCCESS)
    {
        ErrorHandler(dwErrCde);
        printf("Program terminated!\n");

        printf("Press any key to exit....");
        getch();
        exit(1);
    }

    // Step 5: Start frequency measurement operation by DRV_CounterFreqStart
    tCounterFreqStart.counter = usChan;
    tCounterFreqStart.GatePeriod = 100;
    tCounterFreqStart.GateMode = 0;

    dwErrCde = DRV_CounterFreqStart(lDriverHandle, &tCounterFreqStart);
	if (dwErrCde != SUCCESS)
    {
        ErrorHandler(dwErrCde);
        printf("Program terminated!\n");

        printf("Press any key to exit....");
        getch();
        exit(1);
    }

    // Step 6: Read counter value by DRV_CounterFreqRead in while loop
    //         and display counter value, exit when pressing any key
    tCounterFreqRead.counter = usChan;
    tCounterFreqRead.freq = &fFreq;         
    while(!kbhit())
    {
        dwErrCde = DRV_CounterFreqRead(lDriverHandle, &tCounterFreqRead);
        if (dwErrCde != SUCCESS)
        {
            ErrorStop(&lDriverHandle, dwErrCde);
            return;
        }
        printf("\nFrequcney = %f Hz", fFreq);
        Sleep((DWORD)500);
		dwErrCde = DRV_CounterFreqStart(lDriverHandle, &tCounterFreqStart);
		if (dwErrCde != SUCCESS)
		{
			ErrorHandler(dwErrCde);
			printf("Program terminated!\n");
			
			printf("Press any key to exit....");
			getch();
			exit(1);
		}
    }
    if (kbhit()) getch();

    // Step 7: Stop counter by DRV_CounterReset
    dwErrCde = DRV_CounterReset(lDriverHandle, usChan);
    if (dwErrCde != SUCCESS)
    {
        ErrorStop(&lDriverHandle, dwErrCde);
        return;
    }

    // Step 8: Close device
    dwErrCde = DRV_DeviceClose(&lDriverHandle);

    printf("\nPress any key to exit....");
    getch();
}//main

/**********************************************************************
 * Function: ErrorHandler
 *           Show the error message for the corresponding error code
 * input:    dwErrCde, IN, Error code
 * return:   none
 **********************************************************************/
void ErrorHandler(DWORD dwErrCde)
{
    char szErrMsg[180];

    DRV_GetErrorMessage(dwErrCde, szErrMsg);
    printf("\nError(%d): %s\n", dwErrCde & 0xffff, szErrMsg);
}//ErrorHandler

/**********************************************************************
 * Function:   ErrorStop
 *             Release all resource and terminate program if error occurs 
 * Paramaters: pDrvHandle, IN/OUT, pointer to Driver handle
 *             dwErrCde, IN, Error code.
 * return:     none             
 **********************************************************************/
void ErrorStop(long *pDrvHandle, DWORD dwErrCde)
{
    //Error message 
    ErrorHandler(dwErrCde);
    printf("Program terminated!\n");
    
    //Close device
    DRV_DeviceClose(pDrvHandle);

    printf("Press any key to exit....");
    getch();
    exit(1);
}//ErrorStop
