unit Formstar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Driver, Global;

type
  Tfrmstart = class(TForm)
    labSelDev: TLabel;
    lstDevice: TListBox;
    labModule: TLabel;
    lstModule: TListBox;
    cmdRun: TButton;
    labChannel: TLabel;
    lstChannel: TListBox;
    cmdExit: TButton;
    procedure cmdExitClick(Sender: TObject);
    procedure cmdRunClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lstDeviceClick(Sender: TObject);
    procedure lstModuleClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmstart: Tfrmstart;

implementation

uses formrun;

var
   lpDevFeatures      : DEVFEATURES;
   gnNumOfSubdevices  : Smallint;
   Response           : Integer;

{$R *.DFM}
procedure Tfrmstart.cmdExitClick(Sender: TObject);
begin
     if bRun then
        DRV_DeviceClose(DeviceHandle);
     close;
end;

procedure Tfrmstart.cmdRunClick(Sender: TObject);
begin
  ptAOConfig.chan := lstChannel.ItemIndex;
  if gnNumOfSubdevices = 0 then
  begin
       { for DAS and DIO card series }
       ptAOConfig.MaxValue := 5.0;
       ptAOConfig.MinValue := 0.0;
  end
  else
  begin
       { for ADAM series module }
       ptAOConfig.MaxValue := 10.0;
       ptAOConfig.MinValue := 0.0;
  end;

  if (gnNumOfSubdevices = 0) then
  // ADAAM don't support AOConfig function
  begin
       ErrCde := DRV_AOConfig(DeviceHandle, ptAOConfig);
       If (ErrCde <> 0) Then
       begin
            DRV_GetErrorMessage(ErrCde, pszErrMsg);
            Response := Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
            Exit;
       end;
  end;

  { The Output Voltage Range must match to what you select on board }
  FormRun.frmRun.labVolt.Caption := 'Chan# ' + IntToStr(ptAOConfig.chan) + ' Output Voltage:';
  FormRun.frmRun.labHiVolt.Caption := FloatToStrF(ptAOConfig.MaxValue, ffFixed, 5, 0) + 'V';
  FormRun.frmRun.labLoVolt.Caption := FloatToStrF(ptAOConfig.MinValue, ffFixed, 5, 0) + 'V';
  FormRun.frmRun.Show;
end;

procedure Tfrmstart.FormCreate(Sender: TObject);
var
  MaxEntries, OutEntries : Smallint;
  i, j                   : Integer;
  tempStr                : String;
begin
  bRun := False;

  { Add type of PC Laboratory Card }
  ErrCde := DRV_DeviceGetList(DeviceList[0], MaxEntries, OutEntries);
  If (ErrCde <> 0) Then
  begin
       DRV_GetErrorMessage(ErrCde, pszErrMsg);
       Response := Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
       Exit;
  end;

  { Here MaxEntries = OutEntries }
  ErrCde := DRV_DeviceGetNumOfList(MaxEntries);
  If (ErrCde <> 0) Then
  begin
       DRV_GetErrorMessage(ErrCde, pszErrMsg);
       Response := Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
       Exit;
  end;

  For i := 0 To (MaxEntries - 1) do
  begin
    tempStr := '';
    For j := 0 To MaxDevNameLen do
        tempStr := tempStr + DeviceList[i].szDeviceName[j];
    lstDevice.Items.Add(tempStr);
  end;

  { Disable module in this example }
  lstModule.Enabled := False;
  labModule.Enabled := False;

  labChannel.Enabled := False;
  lstChannel.Enabled := False;
  cmdRun.Enabled := False;
end;

procedure Tfrmstart.lstDeviceClick(Sender: TObject);
var
  tempNum, i, ii         : Integer;
  nOutEntries            : Smallint;
  TestRes                : Boolean;
  TempStr                : String;
  dwDeviceNum            : Longint;
begin
  lstModule.Items.Clear;
  lstChannel.Items.Clear;

  tempNum := Pos('DEMO', lstDevice.Items[lstDevice.ItemIndex]);
  if (tempNum <> 0) then
     TestRes := True
  else
     TestRes := False;

  { Avoid to open Advantech Demo Card }
  If (TestRes) Then
  begin
    labModule.Enabled := False;
    lstModule.Enabled := False;
    labChannel.Enabled := False;
    lstChannel.Enabled := False;
    lstChannel.Items.Add('No Use');
    cmdRun.Enabled := False;
  end;

  If (Not TestRes) Then
  begin
    { Check if there is any device attached on this COM or CAN }
    gnNumOfSubdevices := DeviceList[lstDevice.ItemIndex].nNumOfSubdevices;
    if (gnNumOfSubdevices > MaxDev) then
       gnNumOfSubdevices := MaxDev;

    dwDeviceNum := DeviceList[lstDevice.ItemIndex].dwDeviceNum;
    { COM and CAN bus }
    if (gnNumOfSubdevices <> 0) then
    begin
       ErrCde := DRV_DeviceGetSubList(dwDeviceNum, SubDeviceList[0], gnNumOfSubdevices, nOutEntries);
       If (ErrCde <> 0) Then
       begin
            DRV_GetErrorMessage(ErrCde, pszErrMsg);
            Response := Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
            Exit;
       end;
       For i := 0 To (gnNumOfSubdevices - 1) do
       begin
           tempStr := '';
           For ii := 0 To MaxDevNameLen do
               tempStr := tempStr + SubDevicelist[i].szDeviceName[ii];
           lstModule.Items.Add(tempStr);
       end;
       lstModule.Enabled := True;
       labModule.Enabled := True;
    end;

    { PCL DAS & DIO card }
    if (gnNumOfSubdevices = 0) then
    begin
         ErrCde := DRV_DeviceOpen(dwDeviceNum, DeviceHandle);
         If (ErrCde <> 0) Then
         begin
              DRV_GetErrorMessage(ErrCde, pszErrMsg);
              Response := Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
              Exit;
         end
         else
              bRun := True;

         ptDevGetFeatures.buffer := @lpDevFeatures;
         ErrCde := DRV_DeviceGetFeatures(DeviceHandle, ptDevGetFeatures);

         { Add analog output channel items }
         tempNum := lpDevFeatures.usMaxAOChl;
         if (tempNum > 0) then
         begin
              For i := 0 To (tempNum - 1) do
              begin
                   tempStr := 'Chan#' + IntToStr(i);
                   lstChannel.Items.Add(tempStr);
              end;
              labChannel.Enabled := True;
              lstChannel.Enabled := True;
              lstChannel.ItemIndex := 0;
         end;

         { Since you have selected a PC-Lab Card, you can run the application}
         cmdRun.Enabled := True;
    end;
  end;
end;

procedure Tfrmstart.lstModuleClick(Sender: TObject);
var
    tempNum, i             : Integer;
    dwDeviceNum            : Longint;
    TempStr                : String;
begin
     lstChannel.Items.Clear;

     { open COM device or CAN device }
     dwDeviceNum := SubDevicelist[lstModule.ItemIndex].dwDeviceNum;
     ErrCde := DRV_DeviceOpen(dwDeviceNum, DeviceHandle);
     If (ErrCde <> 0) Then
     begin
          DRV_GetErrorMessage(ErrCde, pszErrMsg);
          Response := Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
          Exit;
     end
     Else
       bRun := True;

     ptDevGetFeatures.buffer := @lpDevFeatures;
     ErrCde := DRV_DeviceGetFeatures(DeviceHandle, ptDevGetFeatures);

     { Add analog output channel items }
     tempNum := lpDevFeatures.usMaxAOChl;
     if (tempNum > 0) then
     begin
          For i := 0 To (tempNum - 1) do
          begin
               tempStr := 'Chan#' + IntToStr(i);
               lstChannel.Items.Add(tempStr);
          end;
          labChannel.Enabled := True;
          lstChannel.Enabled := True;
          lstChannel.ItemIndex := 0;
          { Since you have selected a PC-Lab Card, you can run the application}
          cmdRun.Enabled := True;
     end;
end;

end.
