unit Init;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, Mask, Global, Driver;

type
  TfrmInit = class(TForm)
    butExit: TButton;
    lstDevice: TListBox;
    labDevice: TLabel;
    butConvert: TButton;
    butGainList: TButton;
    grpScanChl: TGroupBox;
    labStartChl: TLabel;
    cmbStartChl: TComboBox;
    labChlCnt: TLabel;
    cmbChlCount: TComboBox;
    grpTrigSrc: TGroupBox;
    radExtTrig: TRadioButton;
    radInterTrig: TRadioButton;
    grpChecks: TGroupBox;
    chkCyclic: TCheckBox;
    chkEventEnable: TCheckBox;
    chkFloatData: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    GroupBox1: TGroupBox;
    editSpeed: TMaskEdit;
    labSpeed: TLabel;
    labHz: TLabel;
    labDataCount: TLabel;
    editDataCounts: TMaskEdit;
    GroupBox2: TGroupBox;
    labFifo: TLabel;
    editFifo: TMaskEdit;
    chkFifo: TCheckBox;
    GroupBox3: TGroupBox;
    cmbOverallGain: TComboBox;
    radGainList: TRadioButton;
    radOverall: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure butExitClick(Sender: TObject);
    procedure cmbStartChlChange(Sender: TObject);
    procedure butConvertClick(Sender: TObject);
    procedure lstDeviceClick(Sender: TObject);
    procedure butGainListClick(Sender: TObject);
    procedure cmbOverallGainClick(Sender: TObject);
    procedure radExtTrigClick(Sender: TObject);
    procedure radInterTrigClick(Sender: TObject);
    procedure chkFifoClick(Sender: TObject);
    procedure radOverallClick(Sender: TObject);
    procedure radGainListClick(Sender: TObject);
    procedure cmbOverallGainChange(Sender: TObject);

  private
    { Private declarations }
    ptDevFeatures : PT_DeviceGetFeatures;
    iMaxChl       : Smallint;
    wGainCode  : array [0..15] of Word;
    DeviceList     : array [0..MaxDev] of PT_DEVLIST;
    ptAiStart      : PT_FAIIntScanStart;

    procedure ThreadDone(Sender: TObject);

  public
    { Public declarations }
    iGainIndex : array [0..15] of Smallint;
    ghDev       : Longint;
    giConvCount : LongInt;
    hBuf        : HGLOBAL;
    hUserbuf    : HGLOBAL;     { for voltage data }

  end;

var
  frmInit: TfrmInit;


implementation

uses Convert, DataShow, GainListForm, GetEvent;

{$R *.DFM}

{Internal used procedure}
procedure IsNumbericKey(var Key: Char);
begin
     {Check the pressed key}
     if ((Key < '0') or (Key > '9')) and (Key <> Chr(8))then
        Key := Chr(0);
end;

{*************************************************************
 * Function: Handle the error code. If the input error code > 0,
 *           it means some error apperent.  This function can
 *           show the error message to a message box and stop
 *           this application.
 * Input:    The error code.
 * return:   none
 ************************************************************* }
Function DoesErr(var lErrCode: LongInt): integer;
var
    szErrMsg   : string[100];
    pszErrMsg  : PChar;
begin
    {Check the pressed error code}
    If (lErrCode <> 0) Then
    begin
        pszErrMsg := @szErrMsg;
        DRV_GetErrorMessage(lErrCode, pszErrMsg);
        Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
        DoesErr := 1;
    end
    else
        DoesErr := 0;
end;


procedure TfrmInit.FormCreate(Sender: TObject);
var
    iNumOfDevices : Smallint;
    lErrCde : Longint;
    i : Smallint;

begin
    { Initialize controls}

  { Make Device list }
  lErrCde := DRV_DeviceGetNumOfList(iNumOfDevices);
  if( DoesErr(lErrCde) = 1 ) then
    Exit;

  lErrCde := DRV_DeviceGetList(DeviceList[0], iNumOfDevices, iNumOfDevices);
  if DoesErr(lErrCde) = 1 then
    Exit;

  For i := 0 To (iNumOfDevices - 1) do
  begin
    lstDevice.Items.Add(DeviceList[i].szDeviceName);
  end;
  lstDevice.ItemIndex := 0;
  lstDeviceClick(Sender);


  {Give the initialize value}

end;

procedure TfrmInit.butExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmInit.cmbStartChlChange(Sender: TObject);
var
  i, iIndex : Smallint;
  szTmp     : string;

begin
  {We suggest the stop channel# should large then stop channel}

  {1. Make channel count selection}
  iIndex := cmbChlCount.itemIndex;
  cmbChlCount.Clear;
  for i := 1 to iMaxChl - cmbStartChl.itemIndex do
  begin
    szTmp := IntToStr(i);
    cmbChlCount.items.Add(szTmp)
  end;

  {2. Selection current count}
  if iIndex >= cmbChlCount.items.Count then
    iIndex := cmbChlCount.items.Count - 1;

  cmbChlCount.itemIndex := iIndex;
end;

procedure TfrmInit.butConvertClick(Sender: TObject);
var
  i, j          : Smallint;
  lErrCde       : Longint;
  ptEnableEvent : PT_EnableEvent;
  phiChan       : integer;
begin
  {Start converting action}

  {1. Open device }
  lErrCde := DRV_DeviceOpen( DeviceList[lstDevice.ItemIndex].dwDeviceNum,
                             ghDev);
  if ( DoesErr(lErrCde) = 1 ) then
    Exit;

  {Allocate memory for INT conversion}
  giConvCount := StrToInt(editDataCounts.Text);
  hBuf := GlobalAlloc(GPTR, SizeOf(Word) * (giConvCount+1));
  if hBuf = 0 then
  begin
    Application.MessageBox('Allocate memory for INT transfering failure.',
                           'Error!!', MB_OK);
    DRV_DeviceClose(ghDev);
    exit;
  end;
  if(chkFloatData.Checked ) then
    hUserbuf := GlobalAlloc(GPTR, SizeOf(Single) * (giConvCount+1))
  else
    hUserbuf := GlobalAlloc(GPTR, SizeOf(Word) * (giConvCount+1));
  if hUserbuf = 0 then
  begin
    Application.MessageBox('Allocate memory for INT transfering failure.',
                           'Error!!', MB_OK);
    DRV_DeviceClose(ghDev);
    exit;
  end;

  {2. Config event feature}
  ptEnableEvent.EventType := ADS_EVT_INTERRUPT or
                             ADS_EVT_BUFCHANGE or
                             ADS_EVT_TERMINATED or
                             ADS_EVT_OVERRUN;
  ptEnableEvent.Enabled := Word(chkEventEnable.Checked);
{ 4/23/1999 Eric lin}
  if chkFifo.Checked And (StrToInt(editFifo.Text) > 0) then
     ptEnableEvent.Count   := StrToInt(editFifo.Text)
  else
     ptEnableEvent.Count   := 1;
{Done}

  lErrCde := DRV_EnableEvent(ghDev,ptEnableEvent);
  if DoesErr(lErrCde) = 1  then
  begin
     GlobalUnlock(hBuf);
    GlobalFree(hBuf);
    GlobalUnlock(hUserbuf);
    GlobalFree(hUserbuf);

    DRV_DeviceClose(ghDev);
    Exit;
  end;

  {3. Set gain list}
   phiChan := usStartChan;
   for i:=0 to usChanNum-1 do
   begin
      usGainCode[i] :=
         lpDevFeatures.glGainList[usGainIndx[usStartChan+phiChan]].usGainCde;
      if( (lpDevConfig_AI.ulChanConfig and (1 shl phiChan)) <> 0 )  then
        phiChan:=phiChan+1;
    end;

  {4. Start AI interrupt funciton}
  {4.1 Fill element of PT_FAIIntScanStart}
  with ptAiStart do
  begin
    TrigSrc     := Word(radExtTrig.Checked);
    SampleRate  := StrToInt(editSpeed.Text);
    NumChans    := cmbChlCount.itemIndex + 1;
    StartChan   := cmbStartChl.itemIndex;
    GainList    := @usGainCode[0];
    count       := giConvCount;
    cyclic      := Word(chkCyclic.Checked);
    if chkFifo.Checked And (StrToInt(editFifo.Text) > 0) then
      IntrCount   := StrToInt(editFifo.Text)
    else
      IntrCount   := 1;
    buffer      := Pointer(GlobalLock(hBuf));
  end;

  {4.2 Call advantech API function DRV_FAIIntScanStart}
  lErrCde := DRV_FAIIntScanStart( ghDev, ptAiStart);
  if DoesErr(lErrCde) = 1 Then
  begin
    GlobalUnlock(hBuf);
    GlobalFree(hBuf);
    GlobalUnlock(hUserbuf);
    GlobalFree(hUserbuf);

    lErrCde := DRV_DeviceClose(ghDev);
    Exit;
  end;

  {5. Enable Thread for getting message.}
  if chkEventEnable.Checked = True then
  begin
    with TGetEvent.Create( DeviceList[lstDevice.ItemIndex].dwDeviceNum,
                           ptAiStart.count * 1000 div ptAiStart.SampleRate + 100,
                           chkCyclic.Checked) do
        frmConverting.ghThreadGetEvent := Handle;
  end;
  Sleep(0);

  {6. Enter converting form}
  frmConverting.gbCyclic := chkCyclic.Checked;
  frmConverting.gbConverting := True;
  frmConverting.txtInt.Text :=IntToStr(0);
  frmConverting.txtBuffChange.text := IntToStr(0);

  frmConverting.prgsDataCounts.Max := giConvCount;
  frmConverting.prgsDataCounts.Position := 0;
  frmConverting.prgsDataCounts.Min := 0;

  frmInit.Enabled := False;
  frmConverting.Show;
end;

procedure TfrmInit.lstDeviceClick(Sender: TObject);
var
    lErrCde          : Longint;
    i                : integer;
    szTmp            : string;
    ptAIConfig       : PT_AIGetConfig;
begin
    butConvert.Enabled := FALSE;

    {1. Reset all controls because of changing different device}
    {Clear the previous selection list & combo boxes}
    cmbStartChl.Clear;            {clear start channel}
    cmbChlCount.Clear;            {Clear stop channel}
    cmbOverallGain.Clear;                {Clear the voltage range list}

    {2. Open sellected device for reading its informations}
    lErrCde := DRV_DeviceOpen( DeviceList[lstDevice.ItemIndex].dwDeviceNum,
                               ghDev);
    if( DoesErr(lErrCde) = 1 ) then
        Exit;

    {3. Get device's features}
    ptDevFeatures.buffer := @lpDevFeatures;
    ptDevFeatures.size := SizeOf(DEVFEATURES);
    lErrCde := DRV_DeviceGetFeatures(ghDev, ptDevFeatures);
    if( DoesErr(lErrCde) = 1 ) then
    begin
        DRV_DeviceClose(ghDev);
        Exit;
    end;

    {4. Get device's AI configuration}
    ptAIConfig.buffer := @lpDevConfig_AI;
    ptAIConfig.size := SizeOf(DEVCONFIG_AI);
    lErrCde := DRV_AIGetConfig(ghDev, ptAIConfig);
    if ( DoesErr(lErrCde) = 1 ) then
    begin
        DRV_DeviceClose(ghDev);
        Exit;
    end;

    {5. Setting channel number combo box}

    iMaxChl := lpDevFeatures.usMaxAISiglChl;


    {done}
    for i := 0 to (iMaxChl - 1) do
    begin
        szTmp := IntToStr(i);
        cmbStartChl.items.Add(szTmp);
        szTmp := IntToStr(i+1);
        cmbChlCount.items.Add(szTmp)
    end;
    cmbStartChl.ItemIndex := 0;
    cmbChlCount.ItemIndex := 0;

    {6. Setting gain code selection}
    if (lpDevFeatures.usNumGain > 0) then
        For i := 0 To (lpDevFeatures.usNumGain - 1) do
            cmbOverallGain.Items.Add(lpDevFeatures.glGainList[i].szGainStr);
    cmbOverallGain.ItemIndex := 0;

    {7. Give the initialize Gain code list value}
    for i := 0 to iMaxChl - 1 do
        wGainCode[i] := lpDevFeatures.glGainList[0].usGainCde;
    for i := 0 to 15 do
        iGainIndex[i] := 0;

    {8. Close the device}
    lErrCde := DRV_DeviceClose(ghDev);
    if DoesErr(lErrCde) = 1 then
        Exit;

    butConvert.Enabled := TRUE;
end;

procedure TfrmInit.butGainListClick(Sender: TObject);
begin
    usStartChan := cmbStartChl.ItemIndex;
    usChanNum := StrToInt(cmbChlCount.Text);

    frmGainList.SetGainList();
    frmGainList.ShowModal();

end;

procedure TfrmInit.cmbOverallGainClick(Sender: TObject);
var
    i : Smallint;
begin
    for i := 0 to 15 do
        iGainIndex[i] := cmbOverallGain.ItemIndex;
end;

procedure TfrmInit.radExtTrigClick(Sender: TObject);
begin
    labSpeed.Enabled  := False;
    labHz.Enabled     := False;
    editSpeed.Enabled := False;
end;

procedure TfrmInit.radInterTrigClick(Sender: TObject);
begin
    labSpeed.Enabled  := True;
    labHz.Enabled     := True;
    editSpeed.Enabled := True;
end;

procedure TfrmInit.ThreadDone(Sender: TObject);
begin
    frmConverting.close;
end;

procedure TfrmInit.chkFifoClick(Sender: TObject);
var
    lFifoSize : Longint;
    lErrCde : Longint;
begin
    if (chkFifo.Checked) then
    begin
        { Step 1: Open device }
        lErrCde := DRV_DeviceOpen(DeviceList[lstDevice.ItemIndex].dwDeviceNum, ghDev);
        if DoesErr(lErrCde) = 1 then
            Exit;

        { Step 2: Get FIFO size }
        lErrCde := DRV_GetFIFOSize(ghDev, lFifoSize);
        if DoesErr(lErrCde) = 1 then
            Exit;

        { Step 3: Close device }
        DRV_DeviceClose(ghDev);

        { divide by 2 for conversion from byte to word }
        editFifo.Text := IntToStr(lFifoSize div 2);
        editDataCounts.Text := IntToStr(lFifoSize );
        editFifo.Enabled := true;
    end
    else
        editFifo.Enabled := false;
end;

procedure TfrmInit.radOverallClick(Sender: TObject);
var
   i  : integer;
begin
    for i := 0 to 63 do
          usGainIndx[i] := cmbOverallGain.ItemIndex;
      cmbOverallGain.Enabled := True;
      cmbOverallGainClick(Sender);
      butGainList.Enabled := False



end;

procedure TfrmInit.radGainListClick(Sender: TObject);
begin
        cmbOverallGain.Enabled := False;
        butGainList.Enabled := True
end;

procedure TfrmInit.cmbOverallGainChange(Sender: TObject);
var
   i : integer;
begin
    for i := 0 to 63 do
          usGainIndx[i] := cmbOverallGain.ItemIndex;
end;

end.

