unit FormMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Driver;

type
  TfrmMain = class(TForm)
    GroupBox1: TGroupBox;
    btnSelectDevice: TButton;
    labDeviceName: TLabel;
    grpPWMConfiguration: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    txtPeriod0: TEdit;
    txtHiPeriod0: TEdit;
    btnRun: TButton;
    btnStop: TButton;
    btnExit: TButton;
    txtPluseCount0: TEdit;
    chkPluseEnable0: TCheckBox;
    Label3: TLabel;
    cmbChannel: TComboBox;
    Label4: TLabel;
    Label5: TLabel;

    procedure btnSelectDeviceClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnExitClick(Sender: TObject);
    procedure btnRunClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure chkPluseEnable0Click(Sender: TObject);

  private
    { Private declarations }

  public
    { Public declarations }
    lDeviceNum    :	Longint;
    lDeviceHandle :	Longint;
    szDescription :	array[0..81] of Char;
    ErrorNum:	Longint;

    ptDevFeatures:			DevFeatures;	{ structure for device features }
    ptDevGetFeatures:		PT_DeviceGetFeatures;
    ptCounterPWMSetting:	PT_CounterPWMSetting;

  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}


{*************************************************************
 * Function: Handle the error code. If the input error code > 0,
 *           it means some error apperent.  This function can
 *           show the error message to a message box and stop
 *           this application.
 * Input:    The error code.
 * return:   none
 ************************************************************* }
Function DoesErr(var lErrCode: LongInt): integer;
var
	szErrMsg   : string[100];
	pszErrMsg  : PChar;
begin
     {Check the pressed error code}
	If (lErrCode <> 0) Then
     Begin
		pszErrMsg := @szErrMsg;
		DRV_GetErrorMessage(lErrCode, pszErrMsg);
		Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
		DoesErr := 1;
	  End
	Else
		DoesErr := 0;
end;


procedure TfrmMain.btnSelectDeviceClick(Sender: TObject);
begin
	DRV_SelectDevice(Handle, True, lDeviceNum, szDescription);
   labDeviceName.Caption := szDescription;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
	DRV_SelectDevice(Handle, True, lDeviceNum, szDescription);
   labDeviceName.Caption := szDescription;
end;

procedure TfrmMain.btnExitClick(Sender: TObject);
begin
	Close;
end;

procedure TfrmMain.btnRunClick(Sender: TObject);
var
	code:	Integer;
begin
   ErrorNum := DRV_DeviceOpen(lDeviceNum, lDeviceHandle);
	If ( DoesErr(ErrorNum) = 1 ) Then
		Exit;

   ptDevGetFeatures.buffer := @ptDevFeatures;
   ptDevGetFeatures.size := sizeof(ptDevFeatures);

   ErrorNum := DRV_DeviceGetFeatures(lDeviceHandle, ptDevGetFeatures);
	If ( DoesErr(ErrorNum) = 1 ) Then
     Begin
      DRV_DeviceClose(lDeviceHandle);
		Exit;
     End;

   { check number of counter channels }
   If ptDevFeatures.usMaxTimerChl = 0 Then
     Begin
      MessageBox(Handle, 'No Counter Channel', 'Driver Message', IDOK);
      DRV_DeviceClose(lDeviceHandle);
      Exit;
     End;
   { Enable channel  }

      ptCounterPWMSetting.Port := StrToInt(cmbChannel.Text);
      Val(txtPeriod0.Text,   ptCounterPWMSetting.Period,   code);
      Val(txtHiPeriod0.Text, ptCounterPWMSetting.HiPeriod, code);
      ptCounterPWMSetting.GateMode := 0;
      If chkPluseEnable0.Checked Then
         Val(txtPluseCount0.Text, ptCounterPWMSetting.OutCount, code)
      Else
         ptCounterPWMSetting.OutCount := 0;

      ErrorNum := DRV_CounterPWMSetting(lDeviceHandle, ptCounterPWMSetting);
		If ( DoesErr(ErrorNum) = 1 ) Then
        Begin
         DRV_DeviceClose(lDeviceHandle);
         Exit
        End;

   { Start Channel PWM }
    ErrorNum := DRV_CounterPWMEnable(lDeviceHandle, StrToInt(cmbChannel.Text));
		If ( DoesErr(ErrorNum) = 1 ) Then
        Begin
         DRV_DeviceClose(lDeviceHandle);
         Exit;
		  End;

   grpPWMConfiguration.Enabled := False;
   btnRun.Enabled  := False;
   btnStop.Enabled := True;
   btnExit.Enabled := False;
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
begin
   { Stop ChannelPWM }
   ErrorNum := DRV_CounterReset(lDeviceHandle, StrToInt(cmbChannel.Text));
   { Stop Channel 1 PWM }
   
   DRV_DeviceClose(lDeviceHandle);

   grpPWMConfiguration.Enabled := True;
   btnRun.Enabled  := True;
   btnStop.Enabled := False;
   btnExit.Enabled := True;
end;

procedure TfrmMain.chkPluseEnable0Click(Sender: TObject);
begin
   If chkPluseEnable0.Checked Then
   	txtPluseCount0.Enabled := True
   Else
   	txtPluseCount0.Enabled := False;
end;

end.
