/*
 ************************************************************************
 *  Program     : DADMA.C                                               *
 *  Description : Demo program for Windows with DMA data transfer       *
 *  Revision    : 1.00                                                  *
 *  Date        : 12/10/1997                  Advantech Co., Ltd.       *
 ************************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance() - Saves instance handle and creates main window
 *
 *   MainWndProc() - Process main window messages
 *
 *   ConfigDlgProc() - Process the dialog box messages for configuration
 *
 *   WaveDlgProc() - Process the dialog box messages for waveform setting
 */


#include <windows.h>              /* required for all Windows applications */
#include <stdlib.h>
#include <windowsx.h>
#include <commdlg.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "..\..\..\include\driver.h"
#include "..\..\..\include\os.h"
#include "resource.h"
#include "dadma.h"

typedef struct {
    int wWaveform;
    float fMagnitude;
    float fOffset;
    int wPeriod;
} SWAVE, FAR *LPSWAVE;

// ------------------------------------------------------------------------
// GLOBAL VARIABLES
// ------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hWnd;                       // main window handle

static      DEVFEATURES        DevFeatures;       // structure for device features
static      PT_DeviceGetFeatures  ptDevFeatures;  // Devfeatures table
static      PT_FAODmaStart     ptFAODmaStart;     // FAODMAStart table
static      PT_FAOLoad         ptFAOLoad;         // FAOLoad table
static      PT_FAOScale        ptFAOScale;        // FAOScale table
static      PT_FAOCheck        ptFAOCheck;        // FAOCheck table
static      PT_EnableEvent     ptEnableEvent;     // Enable event
static      PT_CheckEvent      ptCheckEvent;      // Check event
static      PT_AllocateDMABuffer  ptAllocateDMABuffer;  // buffer table

HANDLE      hEvent;
HANDLE      hThreadHandle;
DWORD       dwThreadID;

char        szErrMsg[80];               // Use for MESSAGEBOX function
char        szBuffer[40];               // Temperatory buffer
char        szDescript[100];            // Description device
LRESULT     ErrCde;                     // Return error code

static  char far szWaveform[3][10] = {"SINE","TRIANGLE","SQUARE"};

static  LONG   DriverHandle = (LONG)NULL;         // driver handle
static  ULONG  dwDeviceNum;                       // Device number
static  USHORT gwSelectDevice = FALSE;  // have never selected "Select Device" button
static  USHORT gwDevice = 0;            // device index
static  DWORD  gdwPacerRate = 10000;    // pacer rate
static  ULONG  gulConvNum = 10000;      // conversion number
static  USHORT gwStartChl = 0, gwStopChl = 0;  // scan channels
static  USHORT gwExtTrig = 0;           // external or internal trigger
static  USHORT gwCyclicMode = 1;        // cyclic or non-cylic mode
static  USHORT gwBufferMode = 0;        // buffer: single or double
static  USHORT gwEvtFlag = 1;           // event enable(0)

static  USHORT gwActiveBuf = 0;     // return by FAOCheck
static  USHORT gwOverrun   = 0;     // return by FAOCheck
static  USHORT gwStopped   = 0;     // return by FAOCheck
static  ULONG gulRetrieved = 0;     // return by FAOCheck
static  USHORT gwHalfReady = 0;     // return by FAOCheck

static  ULONG  gwActualBufSize = 0; // actual allocate buffer size

HGLOBAL hBuf, hVoltageBuf, hBinaryBuf;
FLOAT  far * lpVoltageBuf;
USHORT far * lpBinaryBuf;
USHORT far * lpBuf;

static  SWAVE sWaveA = {0,          // output waveform A index
                        2.00f,      // the magnitude of output waveform A
                        2.00f,      // the offset of output waveform A
                        500},       // the points of one period
              sWaveB = {1,          // output waveform B index
                        1.00f,      // the magnitude of output waveform B
                        3.00f,      // the offset of output waveform B
                        500},       // the points of one period
              sWaveC = {2,          // output waveform C index
                        2.00f,      // the magnitude of output waveform C
                        2.00f,      // the offset of output waveform C
                        500};       // the points of one period

static  USHORT gwRepeatCount = 0;  // number of dma terminal count
static  HWND hwndStopButton, hwndStatusButton;
static  HANDLE  hInstance ;
HMENU hMenu;
LONG pBuffer;

BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, int nCmdShow);
long FTYPE MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
int  FTYPE WinMain(HANDLE hInstance, HANDLE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow);
static  FARPROC lpfnConfigDlgProc, lpfnWaveDlgProc;
void SetRealBuffer(float far *lpBuf, long num, LPSWAVE lpWave);

void UserThread();
void adInterruptEvent();
void adBufChangeEvent();
void adOverrunEvent();
void adTerminateEvent();

/***************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
****************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                          /* current instance             */
HANDLE hPrevInstance;                      /* previous instance            */
LPSTR lpCmdLine;                           /* command line                 */
int nCmdShow;                              /* show-window type (open/icon) */
{
    MSG msg;                               /* message                      */

    if (!hPrevInstance)                /* Other instances of app running?  */
      if (!InitApplication(hInstance)) /* Initialize shared things         */
          return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,      /* message structure                      */
        (HWND)NULL,                    /* handle of window receiving the message */
        (UINT)NULL,                    /* lowest message to examine              */
        (UINT)NULL))                   /* highest message to examine             */
    {
        TranslateMessage(&msg);  /* Translates virtual key codes           */
        DispatchMessage(&msg);   /* Dispatches message to window           */
    }

    return (msg.wParam);         /* Returns the value from PostQuitMessage */
}

/****************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                            /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW; /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;  /* window process procedure    */
    wc.cbClsExtra = 0;                  /* No per-class extra data.        */
    wc.cbWndExtra = 0;                  /* No per-window extra data.       */
    wc.hInstance = hInstance;           /* Application that owns the class.*/
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";   /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";  /* Name used in call to CreateWindow. */

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));

}

/***************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
****************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
HANDLE          hInstance;          /* Current instance identifier.          */
int             nCmdShow;           /* Param for first ShowWindow() call.    */
{

    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hWnd = CreateWindow(
        "MyClass",                 /* See RegisterClass() call.           */
        "Advantech Driver Demo : Analog Output with DMA Data Transfer" , /* Window title bar  */
        WS_OVERLAPPEDWINDOW,        /* Window style.                       */
        CW_USEDEFAULT,              /* Default horizontal position.        */
        CW_USEDEFAULT,              /* Default vertical position.          */
        CW_USEDEFAULT,              /* Default width.                      */
        CW_USEDEFAULT,              /* Default height.                     */
        NULL,                       /* Overlapped windows have no parent.  */
        NULL,                       /* Use the window class menu.          */
        hInstance,                  /* This instance owns this window.     */
        NULL                        /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hWnd, nCmdShow);   /* Show the window                       */
    UpdateWindow(hWnd);           /* Sends WM_PAINT message                */
    return (TRUE);                /* Returns the value from PostQuitMessage*/
}

/***************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
****************************************************************************/

BOOL FTYPE ConfigDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                          /* additional information          */
LPARAM lParam;                          /* additional information          */
{

    switch (message)
    {
        case WM_INITDIALOG :

            if (!gwSelectDevice)
            {
                // Step 1: Select Device
                ErrCde = DRV_SelectDevice(hDlg, FALSE, &dwDeviceNum, szDescript);
                if (ErrCde != SUCCESS)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                    return 0;
                }

                gwSelectDevice = TRUE;
            }

			// Show device description to Button text
            SendDlgItemMessage(hDlg, IDC_DeviceSelect, WM_SETTEXT,
               50, (LPARAM)(LPSTR)szDescript);



            // Initialize Conversion Number
            ltoa(gulConvNum, szBuffer, 10);
            SendDlgItemMessage(hDlg, IDC_CONVERSION, EM_REPLACESEL, 0,
                            (LPARAM)((LPSTR)szBuffer));

            // Initialize Pacer Rate
            ltoa(gdwPacerRate, szBuffer, 10);
            SendDlgItemMessage(hDlg, IDC_PACER, EM_REPLACESEL, 0,
                            (LPARAM)((LPSTR)szBuffer));

            if (gwExtTrig)
               EnableWindow(GetDlgItem(hDlg, IDC_PACER), FALSE);
            else
               EnableWindow(GetDlgItem(hDlg, IDC_PACER), TRUE);

            // Initialize Start Channel
            itoa(gwStartChl, szBuffer, 10);
            SendDlgItemMessage(hDlg, IDC_ESTART, EM_REPLACESEL, 0,
                            (LPARAM)((LPSTR)szBuffer));

            // Initialize Triggering
            CheckRadioButton(hDlg, IDC_INTTRIG, IDC_EXTTRIG,
                                   (gwExtTrig)?IDC_EXTTRIG:IDC_INTTRIG);

            // Initialize Repeated Mode
            CheckRadioButton(hDlg, IDC_CYCLIC, IDC_NONCYCLIC,
                                   (gwCyclicMode)?IDC_CYCLIC:IDC_NONCYCLIC);

            // Initialize Message Flag
            CheckRadioButton(hDlg, IDC_EVTENABLE, IDC_EVTDISABLE,
                (gwEvtFlag)?IDC_EVTENABLE:IDC_EVTDISABLE);

            return TRUE;

        case WM_COMMAND :
             switch (LOWORD(wParam))
             {
                case IDOK :
                case IDCANCEL :
                     EndDialog(hDlg, 0);
                     return TRUE;

                case IDC_DeviceSelect:

                    // Step 1: Select Device
                    ErrCde = DRV_SelectDevice(hDlg, FALSE, &dwDeviceNum, szDescript);
                    if (ErrCde != SUCCESS)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                        return 0;
                    }

                    // Show device description to Button text
                    SendDlgItemMessage(hDlg, IDC_DeviceSelect, WM_SETTEXT,
                        50, (LPARAM)(LPSTR)szDescript);

                    gwSelectDevice = TRUE;

                    return TRUE;

                case IDC_CONVERSION :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_CONVERSION,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_CONVERSION,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           gulConvNum = atol(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_CONVERSION,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_PACER :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_PACER,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_PACER,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           gdwPacerRate = atol(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_PACER,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_ESTART :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_ESTART,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_ESTART,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           gwStartChl = atoi(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_ESTART,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_ESTOP :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_ESTOP,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_ESTOP,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           gwStopChl = atoi(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_ESTOP,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_INTTRIG :
                     EnableWindow(GetDlgItem(hDlg, IDC_PACER), TRUE);
                     gwExtTrig = 0;
                     break;;
                case IDC_EXTTRIG :
                     EnableWindow(GetDlgItem(hDlg, IDC_PACER), FALSE);
                     gwExtTrig = 1;
                     break;
                case IDC_CYCLIC :
                     gwCyclicMode = 1;
                     break;
                case IDC_NONCYCLIC :
                     gwCyclicMode = 0;
                     break;
                case IDC_EVTENABLE :
                    gwEvtFlag = 1;
                    break;
                case IDC_EVTDISABLE :
                    gwEvtFlag = 0;
                    break;
             }
             break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: WaveDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for waveform setting
****************************************************************************/

BOOL FTYPE WaveDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                          /* additional information          */
LPARAM lParam;                          /* additional information          */
{
    int  i;

    switch (message)
    {
        case WM_INITDIALOG :

             if (gwBufferMode)
             {
                EnableWindow(GetDlgItem(hDlg, IDC_WAVE_B), TRUE);
                EnableWindow(GetDlgItem(hDlg, IDC_MAGNITUDE_B), TRUE);
                EnableWindow(GetDlgItem(hDlg, IDC_OFFSET_B), TRUE);
                EnableWindow(GetDlgItem(hDlg, IDC_PERIOD_B), TRUE);
             }
             else
             {
                EnableWindow(GetDlgItem(hDlg, IDC_WAVE_B), FALSE);
                EnableWindow(GetDlgItem(hDlg, IDC_MAGNITUDE_B), FALSE);
                EnableWindow(GetDlgItem(hDlg, IDC_OFFSET_B), FALSE);
                EnableWindow(GetDlgItem(hDlg, IDC_PERIOD_B), FALSE);
             }

             // Initialize Output Waveform's Selection
             for (i = 0; i < 3; i ++)
             {
                 SendDlgItemMessage(hDlg, IDC_WAVE_A, CB_ADDSTRING, 0,
                       (LPARAM)((LPSTR)szWaveform[i]));
                 SendDlgItemMessage(hDlg, IDC_WAVE_B, CB_ADDSTRING, 0,
                       (LPARAM)((LPSTR)szWaveform[i]));
             }

             SendDlgItemMessage(hDlg, IDC_WAVE_A, CB_SELECTSTRING, (WPARAM)(-1),
                    (LPARAM)((LPSTR)szWaveform[sWaveA.wWaveform]));
             SendDlgItemMessage(hDlg, IDC_WAVE_B, CB_SELECTSTRING,
                    (WPARAM)(-1), (LPARAM)((LPSTR)szWaveform[sWaveB.wWaveform]));

             // Initialize Magnitude
             sprintf(szBuffer, "%6.2f", sWaveA.fMagnitude);
             SendDlgItemMessage(hDlg, IDC_MAGNITUDE_A, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             sprintf(szBuffer, "%6.2f", sWaveB.fMagnitude);
             SendDlgItemMessage(hDlg, IDC_MAGNITUDE_B, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));

             // Initialize Offset
             sprintf(szBuffer, "%6.2f", sWaveA.fOffset);
             SendDlgItemMessage(hDlg, IDC_OFFSET_A, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             sprintf(szBuffer, "%6.2f", sWaveB.fOffset);
             SendDlgItemMessage(hDlg, IDC_OFFSET_B, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));

             // Initialize Period
             itoa(sWaveA.wPeriod, szBuffer, 10);
             SendDlgItemMessage(hDlg, IDC_PERIOD_A, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             itoa(sWaveB.wPeriod, szBuffer, 10);
             SendDlgItemMessage(hDlg, IDC_PERIOD_B, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));

             return TRUE;

        case WM_COMMAND :
             switch (LOWORD(wParam))
             {
                case IDOK :
                case IDCANCEL :
                     EndDialog(hDlg, 0);
                     return TRUE;

                case IDC_WAVE_A :
                     if (HIWORD(wParam) == CBN_KILLFOCUS)
                     {
                        i = (int)SendDlgItemMessage(hDlg, IDC_WAVE_A,
                                  CB_GETCURSEL, 0, 0) ;
                        if (i != CB_ERR)
                           sWaveA.wWaveform = i;
                     }
                     return TRUE;

                case IDC_WAVE_B :
                     if (HIWORD(wParam) == CBN_KILLFOCUS)
                     {
                        i = (int)SendDlgItemMessage(hDlg, IDC_WAVE_B,
                                  CB_GETCURSEL, 0, 0) ;
                        if (i != CB_ERR)
                           sWaveB.wWaveform = i;
                     }
                     return TRUE;

                case IDC_MAGNITUDE_A :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_MAGNITUDE_A,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_A,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveA.fMagnitude = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_A,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_MAGNITUDE_B :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_MAGNITUDE_B,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_B,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveB.fMagnitude = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_B,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_OFFSET_A :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_OFFSET_A,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_OFFSET_A,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveA.fOffset = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_OFFSET_A,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_OFFSET_B :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_OFFSET_B,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_OFFSET_B,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveB.fOffset = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_OFFSET_B,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_PERIOD_A :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_PERIOD_A,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_PERIOD_A,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveA.wPeriod = atoi(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_PERIOD_A,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_PERIOD_B:
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_PERIOD_B,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_PERIOD_B,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveB.wPeriod = atoi(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_PERIOD_B,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;
             }
             break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
****************************************************************************/

long FTYPE MainWndProc(hWnd, message, wParam, lParam)
HWND hWnd;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                            /* additional information          */
LPARAM lParam;                            /* additional information          */
{
    int     dec, sign;
    TEXTMETRIC tm;
    HDC     hdc;
    static  int cxChar, cyChar;
    char    *szStr, szDest[100], szTemp[50];
    double  ratio;
    DWORD   dwExitCode;
    BOOL    bThreadStatus;

    switch (message)
      {
       case WM_CREATE:

            hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

            lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

            lpfnWaveDlgProc = MakeProcInstance (WaveDlgProc, hInstance) ;

            hdc = GetDC(hWnd);
            SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));
            GetTextMetrics(hdc, &tm);
            cxChar = tm.tmAveCharWidth;
            cyChar = tm.tmHeight + tm.tmExternalLeading;
            ReleaseDC(hWnd, hdc);

            //Create stop button and status button
			        //Create stop button and status button
            hwndStopButton = CreateWindow("button", "Stop",
                (DWORD)(WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON),
                0, 0, 8*cxChar, 3*cyChar/2, hWnd,
                (HMENU)IDC_STOPBUTTON,
                (HANDLE)hInstance, (LPVOID)NULL);

            hwndStatusButton = CreateWindow("button", "Status",
                (DWORD)(WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON),
                8*cxChar, 0, 8*cxChar, 3*cyChar/2, hWnd,
                (HMENU)IDC_STATUSBUTTON,
                (HANDLE)hInstance, (LPVOID)NULL);

            EnableWindow(hwndStopButton, FALSE);
            EnableWindow(hwndStatusButton, FALSE);

            return 0 ;

       case WM_COMMAND:     /* message: from application menu */

         hMenu = GetMenu(hWnd);

         switch(LOWORD(wParam))
          {
            // Show the dialog box of the input parameter
            case IDM_SETTING :

                DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                         hWnd, lpfnConfigDlgProc) ;
                return 0;

            case IDM_WAVEFORM :

                DialogBox (hInstance, MAKEINTRESOURCE(IDD_WAVEFORM),
                         hWnd, lpfnWaveDlgProc) ;
                return 0;

           case IDM_RUN :

                // Step 1: Device open
                ErrCde = DRV_DeviceOpen(dwDeviceNum,
                    (LONG far *)&DriverHandle);
                if (ErrCde != SUCCESS)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                    return 0;
                }

                // Step 2: Allocate memory used by driver
                if((hBuf=(USHORT far *)GlobalAlloc(GHND,
                    sizeof(USHORT) * gulConvNum)) == 0)
                {
                    MessageBox(hWnd,"Not enough memory for buffer ",
                        "High Speed",MB_OK);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                // Step 3: Allocate memory for real voltage
                if((hVoltageBuf=(FLOAT far *)GlobalAlloc(GHND,
                    sizeof(FLOAT) * gulConvNum)) == 0)
                {
                    MessageBox(hWnd,"Not enough memory for buffer ",
                        "High Speed",MB_OK);
                    GlobalFree(hBuf);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                // Step 4: Allocate memory for binary data
                if((hBinaryBuf=(USHORT far *)GlobalAlloc(GHND,
                    sizeof(USHORT) * gulConvNum)) == 0)
                {
                    MessageBox(hWnd,"Not enough memory for buffer ",
                        "High Speed",MB_OK);
                    GlobalFree(hBuf);
                    GlobalFree(hVoltageBuf);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                // Step 5: Allocate DMA buffer for DMA transfer
                ptAllocateDMABuffer.CyclicMode = gwCyclicMode;
                ptAllocateDMABuffer.RequestBufSize = gulConvNum * 2;
                ptAllocateDMABuffer.ActualBufSize = &gwActualBufSize;
                ptAllocateDMABuffer.buffer = &pBuffer;
                if ((ErrCde = DRV_AllocateDMABuffer(DriverHandle,
                    (LPT_AllocateDMABuffer)&ptAllocateDMABuffer)) != SUCCESS)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                    GlobalUnlock(hBuf); GlobalFree(hBuf);
                    GlobalUnlock(hVoltageBuf); GlobalFree(hVoltageBuf);
                    GlobalUnlock(hBinaryBuf); GlobalFree(hBinaryBuf);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                // Lock down buffer
                lpBuf        = (USHORT far *)GlobalLock(hBuf);
                lpBinaryBuf  = (USHORT far *)GlobalLock(hBinaryBuf);
                lpVoltageBuf = (FLOAT far  *)GlobalLock(hVoltageBuf);

                // set real voltage to hVoltageBuf
                SetRealBuffer(lpVoltageBuf, gulConvNum, &sWaveA);

                // Step 6: call FAOScale for transfer voltage to binary data
                ptFAOScale.VoltArray = lpVoltageBuf;
                ptFAOScale.BinArray  = lpBuf;
                ptFAOScale.chan      = gwStartChl;
                ptFAOScale.count     = gulConvNum;
                if ((ErrCde = DRV_FAOScale(DriverHandle,
                    (LPT_FAOScale)&ptFAOScale)) != 0)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                    GlobalUnlock(hBuf); GlobalFree(hBuf);
                    GlobalUnlock(hVoltageBuf); GlobalFree(hVoltageBuf);
                    GlobalUnlock(hBinaryBuf); GlobalFree(hBinaryBuf);
                    DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }


                // Step 7: Enable event feature
                ptEnableEvent.EventType = ADS_EVT_INTERRUPT  |
                    ADS_EVT_BUFCHANGE  |
                    ADS_EVT_TERMINATED |
                    ADS_EVT_OVERRUN;
                ptEnableEvent.Enabled = gwEvtFlag;
                ptEnableEvent.Count   = 1;
                if ((ErrCde = DRV_EnableEvent(DriverHandle,
                    (LPT_EnableEvent)&ptEnableEvent)) != 0)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                    GlobalUnlock(hBuf); GlobalFree(hBuf);
                    GlobalUnlock(hVoltageBuf); GlobalFree(hVoltageBuf);
                    GlobalUnlock(hBinaryBuf); GlobalFree(hBinaryBuf);
                    DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                // Step 8: call FAOIntStart for start action
                // Default steting - conversion number = 1000
                // start channel = 1, pacer rate = 1000 Hz
                // cyclic , enable event
                ptFAODmaStart.buffer     = (LONG far *)lpBuf;        // analog output data
                ptFAODmaStart.TrigSrc    = gwExtTrig;    // triggering
                ptFAODmaStart.SampleRate = gdwPacerRate; // pacer rate = 1K
                ptFAODmaStart.chan       = gwStartChl;   // start channel
                ptFAODmaStart.count      = gulConvNum;   // DA conversion number

                if ((ErrCde = DRV_FAODmaStart(DriverHandle,
                    (LPT_FAODmaStart)&ptFAODmaStart)) != 0)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                    GlobalUnlock(hBuf); GlobalFree(hBuf);
                    GlobalUnlock(hVoltageBuf); GlobalFree(hVoltageBuf);
                    GlobalUnlock(hBinaryBuf); GlobalFree(hBinaryBuf);
                    DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                gwRepeatCount = 0;

                // if event feature is enabled , then create one thread.
                if (gwEvtFlag)
                {
                    hThreadHandle = CreateThread(0, 0,
                        (LPTHREAD_START_ROUTINE) UserThread,
                        0, 0, (LPDWORD)&dwThreadID);
                }

                EnableMenuItem(hMenu, IDM_RUN, MF_DISABLED | MF_BYCOMMAND);

                // Enable stop and status buttons
                EnableWindow(hwndStopButton, TRUE);
                EnableWindow(hwndStatusButton, TRUE);

                return 0;

                //end IDM_RUN

           case IDC_STOPBUTTON:

                // Close thread
                bThreadStatus = GetExitCodeThread(hThreadHandle, &dwExitCode);
                if ((dwExitCode== (DWORD)STILL_ACTIVE) && (bThreadStatus))
                {
                    TerminateThread(hThreadHandle, 0);
                    CloseHandle(hThreadHandle);
                    CloseHandle(hEvent);
                    Sleep(0);
                }

                // Disable stop and status buttons
                EnableWindow(hwndStopButton, FALSE);
                EnableWindow(hwndStatusButton, FALSE);

                // Step 1: Stop D/A conversion for high speed
                if ((ErrCde = DRV_FAOStop(DriverHandle)) != 0)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                    GlobalUnlock(hBuf); GlobalFree(hBuf);
                    GlobalUnlock(hVoltageBuf); GlobalFree(hVoltageBuf);
                    GlobalUnlock(hBinaryBuf); GlobalFree(hBinaryBuf);
                    DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                // Step 2: Free buffer
                GlobalUnlock(hBuf);
                GlobalFree(hBuf);
                GlobalUnlock(hVoltageBuf);
                GlobalFree(hVoltageBuf);
                GlobalUnlock(hBinaryBuf);
                GlobalFree(hBinaryBuf);

                DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);

                // Step 4: Close driver
                DRV_DeviceClose((LONG far *)&DriverHandle);

                EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);

                return 0;

           case IDC_STATUSBUTTON:

                // Get Interrupt transfer status
                ptFAOCheck.ActiveBuf    = &gwActiveBuf;
                ptFAOCheck.stopped      = &gwStopped;
                ptFAOCheck.CurrentCount = &gulRetrieved;
                ptFAOCheck.overrun      = &gwOverrun;
                ptFAOCheck.HalfReady    = &gwHalfReady;

                if ((ErrCde = DRV_FAOCheck(DriverHandle,
                    (LPT_FAOCheck)&ptFAOCheck)) != 0)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                    GlobalUnlock(hBuf); GlobalFree(hBuf);
                    GlobalUnlock(hVoltageBuf); GlobalFree(hVoltageBuf);
                    GlobalUnlock(hBinaryBuf); GlobalFree(hBinaryBuf);
                    DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }
                else
                {   // Display the status of the A/D conversion
                    sprintf(szDest, "Cyclic Counts: %d\n", gwRepeatCount);
                    if (gwHalfReady == 0)
                        strcpy(szTemp, "Buffer : not  ready\n");
                    else if (gwHalfReady == 1)
                        strcpy(szTemp, "Buffer : half ready\n");
                    else if (gwHalfReady == 2)
                        strcpy(szTemp, "Buffer : full ready\n");
                    else
                        strcpy(szTemp, "Buffer : unknow \n");
                    strcat(szDest, szTemp);
                    ratio = 100.0 * gulRetrieved / gulConvNum;
                    szStr = fcvt((double)ratio, 0, &dec, &sign);
                    strcat(szDest, "Finish:  ");
                    strcat(szDest, szStr);
                    strcat(szDest, "%...");
                    MessageBox(hWnd,(LPCSTR)szDest, "Check Status", MB_OK);
                }

                return 0;
          }

    case WM_DESTROY:

         PostQuitMessage(0);
         break;

    default:
         return (DefWindowProc(hWnd, message, wParam, lParam));
      }

   return ((LONG)NULL);
}

void SetRealBuffer(float far *lpBuf, long num, LPSWAVE lpWave)
{
    int i, r, half;
    float slope;

    switch (lpWave->wWaveform)
    {
        case WAVE_SINE:
             for (i = 0; i < num; i ++)
               *(lpBuf+i) = (float)(lpWave->fMagnitude
                            * sin(6.28318*(double)i/(double)lpWave->wPeriod)
                            + lpWave->fOffset);
             break;
        case WAVE_TRIANGLE:
             slope = lpWave->fMagnitude * 4.0f / (float)lpWave->wPeriod;
             for (i = 0; i < num; i ++)
                {
                    r = i % lpWave->wPeriod;
                    half = (int)((float)lpWave->wPeriod / 2.0f);
                    if (r <= half)
                       *(lpBuf + i) = slope * r - lpWave->fMagnitude
                                      + lpWave->fOffset;
                    else
                       *(lpBuf + i) = slope * (lpWave->wPeriod - r)
                                      - lpWave->fMagnitude + lpWave->fOffset;
                }
             break;
        case WAVE_SQUARE:
             for (i = 0; i < num; i ++)
                {
                    r = i % lpWave->wPeriod;
                    half = (int)((float)lpWave->wPeriod / 2.0f);
                    if (r <= half)
                       *(lpBuf + i) = lpWave->fOffset - lpWave->fMagnitude;
                    else
                       *(lpBuf + i) = lpWave->fOffset + lpWave->fMagnitude;
                }
             break;
    }

}

//------------------------------------------------------------------
void UserThread()
{
    USHORT usEventType;
    LONG  ErrCde;
    DWORD TickValue;

    TickValue = GetTickCount();
    TickValue = TickValue + 500000;

    while(TickValue > GetTickCount())
    {
        // Check message
        ptCheckEvent.EventType = &usEventType;
        ptCheckEvent.Milliseconds = 5000;

        if ((ErrCde = DRV_CheckEvent(DriverHandle,
            (LPT_CheckEvent)&ptCheckEvent)) != 0)
        {
            MessageBox(hWnd,"Check Event Error !","Thread Message",MB_OK);
            return ;
        }

        // Process interrupt event
        if (usEventType & ADS_EVT_INTERRUPT)
            adInterruptEvent();

        // Process buffer change event
        if (usEventType & ADS_EVT_BUFCHANGE)
            adBufChangeEvent();

        // Process overrun event
        if (usEventType & ADS_EVT_OVERRUN)
            adOverrunEvent();

        // Process terminate event
        if (usEventType & ADS_EVT_TERMINATED)
        {
            adTerminateEvent();
            return ;
        }
    }

    MessageBox(hWnd,"Never Get Any Event !","Thread Message",MB_OK);
}

//------------------------------------------------------------------
void adInterruptEvent()
{
    return;
}

//------------------------------------------------------------------
void adBufChangeEvent()
{
    USHORT usWaveIndex;
    LPSWAVE lpWave;

    usWaveIndex = (gwRepeatCount + 1) % 3;
    switch (usWaveIndex)
    {
      case 0:
          lpWave = &sWaveA;
          break;
      case 1:
          lpWave = &sWaveB;
          break;
      case 2:
          lpWave = &sWaveC;
          break;
    }

    // Step 1: get status
    ptFAOCheck.ActiveBuf    = &gwActiveBuf;
    ptFAOCheck.stopped      = &gwStopped;
    ptFAOCheck.CurrentCount = &gulRetrieved;
    ptFAOCheck.overrun      = &gwOverrun;
    ptFAOCheck.HalfReady    = &gwHalfReady;

    if ((ErrCde = DRV_FAOCheck(DriverHandle,
         (LPT_FAOCheck)&ptFAOCheck)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    // Set real voltage to first half buffer
    SetRealBuffer(lpVoltageBuf, gulConvNum/2, lpWave);

    // Step 2: call FAOScale for transfer voltage to binary data
    ptFAOScale.VoltArray = lpVoltageBuf;
    ptFAOScale.BinArray  = lpBinaryBuf;
    ptFAOScale.chan      = gwStartChl;
    ptFAOScale.count     = gulConvNum/2;
    if ((ErrCde = DRV_FAOScale(DriverHandle,
        (LPT_FAOScale)&ptFAOScale)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    if (gwHalfReady == 2) // second half ready
    {
        // Step 3: call FAOLoad to pass data to driver
    //  ptFAOLoad.ActiveBuf  = 0;  // single buffer
    //  ptFAOLoad.DataBuffer = lpBinaryBuf;
    //  ptFAOLoad.start    = 0;
    //  ptFAOLoad.count    = gulConvNum;

    //  if ((ErrCde = DRV_FAOLoad(DriverHandle,
    //      (LPT_FAOLoad)&ptFAOLoad)) != 0)
    //  {
    //      DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
    //      MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
    //      return ;
    //  }

        gwRepeatCount ++;
    }

    return;
}
//------------------------------------------------------------------
void adOverrunEvent()
{
    // clear overrun
    if ((ErrCde = DRV_ClearOverrun(DriverHandle)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    return;
}
//------------------------------------------------------------------
void adTerminateEvent()
{
    // Disable stop and status buttons
    EnableWindow(hwndStopButton, FALSE);
    EnableWindow(hwndStatusButton, FALSE);

    // Stop A/D conversion
    if ((ErrCde = DRV_FAOStop(DriverHandle)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    // Free buffer
    GlobalUnlock(hBuf);
    GlobalFree(hBuf);
    GlobalUnlock(hVoltageBuf);
    GlobalFree(hVoltageBuf);
    GlobalUnlock(hBinaryBuf);
    GlobalFree(hBinaryBuf);

    DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);

    // Close driver
    DRV_DeviceClose((LONG far *)&DriverHandle);

    EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);

    return;
}


