/*
 ***********************************************************************
 *  Program     : DIGINEX.C                                            *
 *  Description : Demo program for PCI-1760                            *
 *  Revision    : 1.00                                                 *
 *  Date        : 3/31/1999                    Advantech Co., Ltd.     *
 ***********************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance() - Saves instance handle and creates main window
 *
 *   MainWndProc() - Process main window messages
 *
 *   ConfigDlgProc() - Process the dialog box messages for configuration
 */


#include <windows.h>
#include <stdlib.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "..\..\..\include\driver.h"
#include "..\..\..\include\os.h"
#include "resource.h"

// ------------------------------------------------------------------------
// CONSTANT DEFINITION
// ------------------------------------------------------------------------
#define     MAX_DEVICES     100


// ------------------------------------------------------------------------
// GLOBAL VARIABLES
// ------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hMainWnd;                   // main window handle

static      PT_DioReadBit           ptDioReadBit;      //xujin 2002.4.28
//static      PT_DioReadPortByte      ptDioReadPortByte;//xujin 2002.4.28
static      PT_EnableEventEx        ptEnableEventEx;     // Enable event
static      PT_FAICheck             ptFAICheck;        // FAICheck table
static      PT_CheckEvent           ptCheckEvent;      // Check event
static      PT_FDITransfer          ptFDITransfer;
static      ThreadhWnd;    

HANDLE      hThreadHandle;
DWORD       dwThreadID;

char        szErrMsg[80];               // Use for MESSAGEBOX function
LRESULT     ErrCde;                     // Return error code


//
// used for fixed memory for the installed devices
//
DEVLIST     DeviceList[MAX_DEVICES];
DEVLIST     SubDeviceList[MAX_DEVICES];

LONG        DriverHandle = (LONG)NULL;          // driver handle
BOOL        bRun = FALSE;                       // flag for running
BOOL        bThreadflag = FALSE;
USHORT      m_bContinue = FALSE;

USHORT      gwDevice = 0, gwSubDevice = 0;      // device index
USHORT      gwChannel = 0;                      // input channel
//USHORT      gwValue;                            // input value//xujin 2002.5.8
USHORT      gwState;                            //input state //xujin 2002.5.8
SHORT       gnNumOfDevices, gnNumOfSubdevices;  // number of installed devices
USHORT      gwScanTime = 1000;                  // scan time
DWORD       gdwStartTime;                       // start time
DWORD       gdwElapseTime = 0;                  // elapse time
USHORT      gwOverrunCount = 0;                 // overrun count
FARPROC     lpfnConfigDlgProc;                  // config. dialog procedure
FARPROC     lpfnScanDlgProc;                    // scan dialog procedure
FARPROC     lpfnConfigCounterProc;              // Counter dialog procedure 
FARPROC     lpfnConfigFilterProc;               // Filter dialog procedure
FARPROC     lpfnConfigPatternProc;              // Pattern dialog procedure
FARPROC     lpfnConfigStatusProc;               // Status dialog procedure
USHORT      usFHiValue[8]={0,0,0,0,0,0,0,0}, usFLowValue[8]={0,0,0,0,0,0,0,0}, usFEnable = 0;
USHORT      usPEnable = 0, usPValue = 0;
USHORT      usSEnable = 0, usSRiseEdge = 0, usSFallEdge =0;
USHORT      usCPreset[8] = {0,0,0,0,0,0,0,0}, usCMatchValue[8] = {0,0,0,0,0,0,0,0};
USHORT      usCEnable=0, usCMatchEnable=0, usTrigEdge=0, usCOverflow=0;
USHORT      usRSCount=0,usFSCount=0,usPCount=0,usCMCount=0,usCOCount=0;
ULONG       retValueP=0, retValueS=0, retValueCM=0, retValueCO=0;

// ------------------------------------------------------------------------
// FUNCTION DECLARATION
// ------------------------------------------------------------------------
BOOL InitApplication(HANDLE);
BOOL InitInstance(HANDLE, int);
long FTYPE MainWndProc(HWND, UINT, WPARAM, LPARAM);
int  FTYPE WinMain(HANDLE, HANDLE, LPSTR, int);

void UserThread();
void adPatternMatchEvent();
void adCounterMatchEvent();
void adCounterOverflowEvent();
void adStatusChangeEvent();


/***************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
****************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                          /* current instance             */
HANDLE hPrevInstance;                      /* previous instance            */
LPSTR lpCmdLine;                           /* command line                 */
int nCmdShow;                              /* show-window type (open/icon) */
{
    MSG msg;                               /* message                      */

    if (!hPrevInstance)                /* Other instances of app running?  */
      if (!InitApplication(hInstance)) /* Initialize shared things         */
          return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,      /* message structure                      */
        (HWND)NULL,              /* handle of window receiving the message */
        (UINT)NULL,              /* lowest message to examine              */
        (UINT)NULL))             /* highest message to examine             */
    {
        TranslateMessage(&msg);  /* Translates virtual key codes           */
        DispatchMessage(&msg);   /* Dispatches message to window           */
    }
    return (msg.wParam);         /* Returns the value from PostQuitMessage */
}

/****************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                            /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW; /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;   /* window process procedure   */
    wc.cbClsExtra = 0;                  /* No per-class extra data.        */
    wc.cbWndExtra = 0;                  /* No per-window extra data.       */
    wc.hInstance = hInstance;           /* Application that owns the class.*/
    wc.hIcon = LoadIcon(hInstance, "IDI_ICON1");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";   /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";  /* Name used in call to CreateWindow. */

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));

}

/***************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
****************************************************************************/

BOOL InitInstance
(
    HANDLE          hInstance,          // Current instance identifier.
    int             nCmdShow            // Param for first ShowWindow() call.
)
{
    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hMainWnd = CreateWindow(
        "MyClass",                 /* See RegisterClass() call.           */
        "Advantech Driver Demo : Digital Input extesion" , /* Window title bar  */
        WS_OVERLAPPEDWINDOW,        /* Window style.                       */
        CW_USEDEFAULT,              /* Default horizontal position.        */
        CW_USEDEFAULT,              /* Default vertical position.          */
        CW_USEDEFAULT,              /* Default width.                      */
        CW_USEDEFAULT,              /* Default height.                     */
        NULL,                       /* Overlapped windows have no parent.  */
        NULL,                       /* Use the window class menu.          */
        hInstance,                  /* This instance owns this window.     */
        NULL                        /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hMainWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hMainWnd, nCmdShow);   // Show the window
    UpdateWindow(hMainWnd);           // Sends WM_PAINT message
    return (TRUE);                // Returns the value from PostQuitMessage
}

/***************************************************************************
    FUNCTION: ConfigFilterProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ConfigFilterProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    char        szBuffer[40];
	USHORT      i;

    switch (message)
    {
    case WM_INITDIALOG :
		for(i=0; i<8; i++)
		 if(usFEnable & (0x01 << i))
		 {   EnableWindow(GetDlgItem(hDlg,(IDC_ED_HiFilter0 + i)),TRUE);
             EnableWindow(GetDlgItem(hDlg,(IDC_ED_LowFilter0 + i)),TRUE);
		     CheckDlgButton(hDlg,(IDC_Filter0 + i),TRUE);
             sprintf(szBuffer, "%d", usFHiValue[i]);
             SendDlgItemMessage(hDlg, (IDC_ED_HiFilter0 + i), EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             sprintf(szBuffer, "%d", usFLowValue[i]);
             SendDlgItemMessage(hDlg, (IDC_ED_LowFilter0 + i), EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
		 }
		 else
		 {   EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter0 + i),FALSE);
             EnableWindow(GetDlgItem(hDlg,(IDC_ED_LowFilter0 + i)),FALSE);
		     CheckDlgButton(hDlg,(IDC_Filter0 + i),FALSE);}

        return TRUE;

    case WM_COMMAND :

         switch (LOWORD(wParam))
         {
		    case IDC_Filter0:
                 if(IsDlgButtonChecked(hDlg, IDC_Filter0))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter0),TRUE);
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter0),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_HiFilter0));}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter0),FALSE);
				    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter0),FALSE);}
				 break;
		    case IDC_Filter1:
                 if(IsDlgButtonChecked(hDlg, IDC_Filter1))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter1),TRUE);
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter1),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_HiFilter1));}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter1),FALSE);
				    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter1),FALSE);}
				 break;
		    case IDC_Filter2:
                 if(IsDlgButtonChecked(hDlg, IDC_Filter2))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter2),TRUE);
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter2),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_HiFilter2));}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter2),FALSE);
				    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter2),FALSE);}
				 break;
		    case IDC_Filter3:
                 if(IsDlgButtonChecked(hDlg, IDC_Filter3))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter3),TRUE);
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter3),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_HiFilter3));}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter3),FALSE);
				    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter3),FALSE);}
				 break;
		    case IDC_Filter4:
                 if(IsDlgButtonChecked(hDlg, IDC_Filter4))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter4),TRUE);
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter4),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_HiFilter4));}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter4),FALSE);
				    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter4),FALSE);}
				 break;
		    case IDC_Filter5:
                 if(IsDlgButtonChecked(hDlg, IDC_Filter5))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter5),TRUE);
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter5),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_HiFilter5));}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter5),FALSE);
				    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter5),FALSE);}
				 break;
		    case IDC_Filter6:
                 if(IsDlgButtonChecked(hDlg, IDC_Filter6))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter6),TRUE);
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter6),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_HiFilter6));}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter6),FALSE);
				    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter6),FALSE);}
				 break;
		    case IDC_Filter7:
                 if(IsDlgButtonChecked(hDlg, IDC_Filter7))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter7),TRUE);
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter7),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_HiFilter7));}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_HiFilter7),FALSE);
				    EnableWindow(GetDlgItem(hDlg,IDC_ED_LowFilter7),FALSE);}
				 break;

            case IDOK :
                usFEnable = 0;
				for(i=0;i<8;i++)
                 if(IsDlgButtonChecked(hDlg, (IDC_Filter0 + i)))
				 {
                   SendDlgItemMessage(hDlg, (IDC_ED_HiFilter0 + i), EM_GETLINE, 0, (LPARAM)szBuffer) ;
                   usFHiValue[i] = LOWORD(atol(szBuffer));
                   SendDlgItemMessage(hDlg, (IDC_ED_LowFilter0 + i), EM_GETLINE, 0, (LPARAM)szBuffer) ;
                   usFLowValue[i] = LOWORD(atol(szBuffer));
                   usFEnable |= (0x01 << i); 				   
				 }
				 else
				 { usFHiValue[i] = 0;
                   usFLowValue[i] = 0;}
                
            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}
/***************************************************************************
    FUNCTION: ConfigCounterProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ConfigCounterProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    char        szBuffer[40];
    USHORT      i;

    switch (message)
    {
    case WM_INITDIALOG :
        for(i=0; i<8; i++)
		{

		 EnableWindow(GetDlgItem(hDlg,(IDC_Counter0 + i)),TRUE);

		 if( usCEnable & (0x01 << i))
		 {   CheckDlgButton(hDlg,(IDC_Counter0 + i),TRUE);
			 EnableWindow(GetDlgItem(hDlg,(IDC_EDPeValue0 + i)),TRUE);
             sprintf(szBuffer, "%d", usCPreset[i]);
             SendDlgItemMessage(hDlg, (IDC_EDPeValue0 + i), EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));

			 EnableWindow(GetDlgItem(hDlg,(IDC_CMatch0 + i)),TRUE);
             if(usCMatchEnable & (0x01 << i))
			 {   CheckDlgButton(hDlg,(IDC_CMatch0 + i),TRUE);
			     EnableWindow(GetDlgItem(hDlg,(IDC_ED_Match0 + i)),TRUE);
                 sprintf(szBuffer, "%d", usCMatchValue[i]);
                 SendDlgItemMessage(hDlg, (IDC_ED_Match0 + i), EM_REPLACESEL, 0,
                                   (LPARAM)((LPSTR)szBuffer));}
			 else
			 {   CheckDlgButton(hDlg,(IDC_CMatch0 + i),FALSE);
			     EnableWindow(GetDlgItem(hDlg,(IDC_ED_Match0 + i)),FALSE);}

		     EnableWindow(GetDlgItem(hDlg,(IDC_COverflow0 + i)),TRUE);
			 if(usCOverflow & (0x01 << i))
			     CheckDlgButton(hDlg,(IDC_COverflow0 + i),TRUE);
			 else
			     CheckDlgButton(hDlg,(IDC_COverflow0 + i),FALSE);

			 EnableWindow(GetDlgItem(hDlg,(IDC_CTrigRedge0 + i)),TRUE);
			 EnableWindow(GetDlgItem(hDlg,(IDC_CTrigFedge0 + i)),TRUE);
             if(usTrigEdge & (0x01 << i))
				 CheckDlgButton(hDlg, (IDC_CTrigFedge0 + i),TRUE);
			 else
				 CheckDlgButton(hDlg, (IDC_CTrigRedge0 + i),TRUE);
		 
		 }else{   
			 EnableWindow(GetDlgItem(hDlg,(IDC_EDPeValue0 + i)),FALSE);
		     CheckDlgButton(hDlg,(IDC_EDPeValue0 + i),FALSE);
             
		     EnableWindow(GetDlgItem(hDlg,(IDC_CMatch0 + i)),FALSE);

			 EnableWindow(GetDlgItem(hDlg,(IDC_ED_Match0 + i)),FALSE);
			 CheckDlgButton(hDlg,(IDC_CMatch0 + i),FALSE);

		     EnableWindow(GetDlgItem(hDlg,(IDC_COverflow0 + i)),FALSE);

			 CheckDlgButton(hDlg,(IDC_COverflow0 + i),FALSE);

			 EnableWindow(GetDlgItem(hDlg,(IDC_CTrigRedge0 + i)),FALSE);

			 EnableWindow(GetDlgItem(hDlg,(IDC_CTrigFedge0 + i)),FALSE);
		 }

		}

        return TRUE;

    case WM_COMMAND :

         switch (LOWORD(wParam))
         {
		    case IDC_Counter0:
                 if(IsDlgButtonChecked(hDlg, IDC_Counter0))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue0),TRUE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch0),TRUE);
					if(IsDlgButtonChecked(hDlg, IDC_CMatch0))
			           EnableWindow(GetDlgItem(hDlg,IDC_ED_Match0),TRUE);
					else
   		               EnableWindow(GetDlgItem(hDlg,IDC_ED_Match0),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow0),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge0),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge0),TRUE);
					CheckDlgButton(hDlg, IDC_CTrigRedge0,TRUE);
					CheckDlgButton(hDlg, IDC_CTrigFedge0,FALSE);
    	            SetFocus(GetDlgItem(hDlg,IDC_EDPeValue0));}
		         else
				 {
                    EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue0),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch0),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ED_Match0),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow0),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge0),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge0),FALSE);
				 }
				 break;
		    case IDC_Counter1:
                 if(IsDlgButtonChecked(hDlg, IDC_Counter1))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue1),TRUE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch1),TRUE);
					if(IsDlgButtonChecked(hDlg, IDC_CMatch1))
			           EnableWindow(GetDlgItem(hDlg,IDC_ED_Match1),TRUE);
					else
   		               EnableWindow(GetDlgItem(hDlg,IDC_ED_Match1),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow1),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge1),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge1),TRUE);
					CheckDlgButton(hDlg, IDC_CTrigRedge1,TRUE);
					CheckDlgButton(hDlg, IDC_CTrigFedge1,FALSE);
    	            SetFocus(GetDlgItem(hDlg,IDC_EDPeValue1));}
		         else
				 {
                    EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue1),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch1),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ED_Match1),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow1),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge1),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge1),FALSE);
				 }
				 break;
		    case IDC_Counter2:
                 if(IsDlgButtonChecked(hDlg, IDC_Counter2))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue2),TRUE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch2),TRUE);
					if(IsDlgButtonChecked(hDlg, IDC_CMatch2))
			           EnableWindow(GetDlgItem(hDlg,IDC_ED_Match2),TRUE);
					else
   		               EnableWindow(GetDlgItem(hDlg,IDC_ED_Match2),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow2),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge2),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge2),TRUE);
					CheckDlgButton(hDlg, IDC_CTrigRedge2,TRUE);
					CheckDlgButton(hDlg, IDC_CTrigFedge2,FALSE);
    	            SetFocus(GetDlgItem(hDlg,IDC_EDPeValue2));}
		         else
				 {
                    EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue2),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch2),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ED_Match2),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow2),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge2),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge2),FALSE);
				 }
				 break;
		    case IDC_Counter3:
                 if(IsDlgButtonChecked(hDlg, IDC_Counter3))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue3),TRUE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch3),TRUE);
					if(IsDlgButtonChecked(hDlg, IDC_CMatch3))
			           EnableWindow(GetDlgItem(hDlg,IDC_ED_Match3),TRUE);
					else
   		               EnableWindow(GetDlgItem(hDlg,IDC_ED_Match3),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow3),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge3),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge3),TRUE);
					CheckDlgButton(hDlg, IDC_CTrigRedge3,TRUE);
					CheckDlgButton(hDlg, IDC_CTrigFedge3,FALSE);
    	            SetFocus(GetDlgItem(hDlg,IDC_EDPeValue3));}
		         else
				 {
                    EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue3),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch3),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ED_Match3),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow3),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge3),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge3),FALSE);
				 }
				 break;
		    case IDC_Counter4:
                 if(IsDlgButtonChecked(hDlg, IDC_Counter4))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue4),TRUE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch4),TRUE);
					if(IsDlgButtonChecked(hDlg, IDC_CMatch4))
			           EnableWindow(GetDlgItem(hDlg,IDC_ED_Match4),TRUE);
					else
   		               EnableWindow(GetDlgItem(hDlg,IDC_ED_Match4),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow4),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge4),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge4),TRUE);
					CheckDlgButton(hDlg, IDC_CTrigRedge4,TRUE);
					CheckDlgButton(hDlg, IDC_CTrigFedge4,FALSE);
    	            SetFocus(GetDlgItem(hDlg,IDC_EDPeValue4));}
		         else
				 {
                    EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue4),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch4),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ED_Match4),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow4),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge4),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge4),FALSE);
				 }
				 break;
		    case IDC_Counter5:
                 if(IsDlgButtonChecked(hDlg, IDC_Counter5))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue5),TRUE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch5),TRUE);
					if(IsDlgButtonChecked(hDlg, IDC_CMatch5))
			           EnableWindow(GetDlgItem(hDlg,IDC_ED_Match5),TRUE);
					else
   		               EnableWindow(GetDlgItem(hDlg,IDC_ED_Match5),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow5),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge5),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge5),TRUE);
					CheckDlgButton(hDlg, IDC_CTrigRedge5,TRUE);
					CheckDlgButton(hDlg, IDC_CTrigFedge5,FALSE);
    	            SetFocus(GetDlgItem(hDlg,IDC_EDPeValue5));}
		         else
				 {
                    EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue5),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch5),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ED_Match5),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow5),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge5),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge5),FALSE);
				 }
				 break;
		    case IDC_Counter6:
                 if(IsDlgButtonChecked(hDlg, IDC_Counter6))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue6),TRUE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch6),TRUE);
					if(IsDlgButtonChecked(hDlg, IDC_CMatch6))
			           EnableWindow(GetDlgItem(hDlg,IDC_ED_Match6),TRUE);
					else
   		               EnableWindow(GetDlgItem(hDlg,IDC_ED_Match6),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow6),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge6),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge6),TRUE);
					CheckDlgButton(hDlg, IDC_CTrigRedge6,TRUE);
					CheckDlgButton(hDlg, IDC_CTrigFedge6,FALSE);
    	            SetFocus(GetDlgItem(hDlg,IDC_EDPeValue6));}
		         else
				 {
                    EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue6),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch6),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ED_Match6),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow6),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge6),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge6),FALSE);
				 }
				 break;
		    case IDC_Counter7:
                 if(IsDlgButtonChecked(hDlg, IDC_Counter7))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue7),TRUE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch7),TRUE);
					if(IsDlgButtonChecked(hDlg, IDC_CMatch7))
			           EnableWindow(GetDlgItem(hDlg,IDC_ED_Match7),TRUE);
					else
   		               EnableWindow(GetDlgItem(hDlg,IDC_ED_Match7),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow7),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge7),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge7),TRUE);
					CheckDlgButton(hDlg, IDC_CTrigRedge7,TRUE);
					CheckDlgButton(hDlg, IDC_CTrigFedge7,FALSE);
    	            SetFocus(GetDlgItem(hDlg,IDC_EDPeValue7));}
		         else
				 {
                    EnableWindow(GetDlgItem(hDlg,IDC_EDPeValue7),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_CMatch7),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ED_Match7),FALSE);
		            EnableWindow(GetDlgItem(hDlg,IDC_COverflow7),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigRedge7),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_CTrigFedge7),FALSE);
				 }
				 break;
            
			// set match value
			case IDC_CMatch0:
                 if(IsDlgButtonChecked(hDlg, IDC_CMatch0))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_Match0),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_Match0));}
		         else
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_Match0),FALSE);
				 break;
			case IDC_CMatch1:
                 if(IsDlgButtonChecked(hDlg, IDC_CMatch1))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_Match1),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_Match1));}
		         else
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_Match1),FALSE);
				 break;
			case IDC_CMatch2:
                 if(IsDlgButtonChecked(hDlg, IDC_CMatch2))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_Match2),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_Match2));}
		         else
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_Match2),FALSE);
				 break;
			case IDC_CMatch3:
                 if(IsDlgButtonChecked(hDlg, IDC_CMatch3))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_Match3),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_Match3));}
		         else
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_Match3),FALSE);
				 break;
			case IDC_CMatch4:
                 if(IsDlgButtonChecked(hDlg, IDC_CMatch4))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_Match4),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_Match4));}
		         else
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_Match4),FALSE);
				 break;
			case IDC_CMatch5:
                 if(IsDlgButtonChecked(hDlg, IDC_CMatch5))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_Match5),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_Match5));}
		         else
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_Match5),FALSE);
				 break;
			case IDC_CMatch6:
                 if(IsDlgButtonChecked(hDlg, IDC_CMatch6))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_Match6),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_Match6));}
		         else
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_Match6),FALSE);
				 break;
			case IDC_CMatch7:
                 if(IsDlgButtonChecked(hDlg, IDC_CMatch7))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ED_Match7),TRUE);
    	            SetFocus(GetDlgItem(hDlg,IDC_ED_Match7));}
		         else
                    EnableWindow(GetDlgItem(hDlg,IDC_ED_Match7),FALSE);
				 break;

            case IDOK :
                 usCEnable=0; usCMatchEnable=0; usTrigEdge=0; usCOverflow=0;
                 for(i=0;i<8;i++)
				  if(IsDlgButtonChecked(hDlg, (IDC_Counter0+i)))
				  {
					  usCEnable |= (0x01 << i);
					  GetDlgItemText(hDlg,(IDC_EDPeValue0+i),szBuffer,40);
                      usCPreset[i] = LOWORD(atol(szBuffer));

					  if(IsDlgButtonChecked(hDlg, (IDC_CMatch0+i)))
					  {
                       usCMatchEnable |= (0x01 << i);
					   GetDlgItemText(hDlg,(IDC_ED_Match0+i),szBuffer,40);
                       usCMatchValue[i] = LOWORD(atol(szBuffer));
					  }
					  else
                       usCMatchValue[i] = 0;

					  if(IsDlgButtonChecked(hDlg, (IDC_COverflow0+i)))
						usCOverflow |= (0x01 << i);

					  if(IsDlgButtonChecked(hDlg, (IDC_CTrigFedge0 + i)))
						usTrigEdge |= (0x01 << i);
				  }
				  else
				  {
					  usCPreset[i] = 0;
                      usCMatchValue[i] = 0;
				  }


            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}
/***************************************************************************
    FUNCTION: ConfigPatternProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ConfigPatternProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    USHORT      i;

    switch (message)
    {
    case WM_INITDIALOG :
	  for(i=0; i<8; i++)
	  {
		 EnableWindow(GetDlgItem(hDlg,(IDC_Pattern0 + i)),TRUE);

		 if(usPEnable & (0x01 << i))
		 {
		    CheckDlgButton(hDlg,(IDC_Pattern0 + i),TRUE);
			EnableWindow(GetDlgItem(hDlg,IDC_PB0HI + i),TRUE);
			EnableWindow(GetDlgItem(hDlg,IDC_PB0LOW + i),TRUE);
			if(usPValue & (0x01 << i))
			   CheckDlgButton(hDlg,(IDC_PB0HI + i),1);
			else
			   CheckDlgButton(hDlg,(IDC_PB0LOW + i),1);
		 } else	 {
		    CheckDlgButton(hDlg,(IDC_Pattern0 + i),FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_PB0HI + i),FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_PB0LOW + i),FALSE);
		 }
	  }

        return TRUE;

    case WM_COMMAND :

         switch (LOWORD(wParam))
         {
		    case IDC_Pattern0:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern0))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB0HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB0LOW),TRUE);
			        CheckDlgButton(hDlg,IDC_PB0HI,TRUE);	 
			        CheckDlgButton(hDlg,IDC_PB0LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB0HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB0LOW),FALSE);}
				 break;
		    case IDC_Pattern1:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern1))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB1HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB1LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_PB1HI,TRUE);
			        CheckDlgButton(hDlg,IDC_PB1LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB1HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB1LOW),FALSE);}
				 break;
		    case IDC_Pattern2:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern2))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB2HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB2LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_PB2HI,TRUE);
			        CheckDlgButton(hDlg,IDC_PB2LOW,FALSE);
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB2HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB2LOW),FALSE);}
				 break;
		    case IDC_Pattern3:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern3))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB3HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB3LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_PB3HI,TRUE);
			        CheckDlgButton(hDlg,IDC_PB3LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB3HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB3LOW),FALSE);}
				 break;
		    case IDC_Pattern4:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern4))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB4HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB4LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_PB4HI,TRUE);
			        CheckDlgButton(hDlg,IDC_PB4LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB4HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB4LOW),FALSE);}
				 break;
		    case IDC_Pattern5:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern5))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB5HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB5LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_PB5HI,TRUE);
			        CheckDlgButton(hDlg,IDC_PB5LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB5HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB5LOW),FALSE);}
				 break;
		    case IDC_Pattern6:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern6))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB6HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB6LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_PB6HI,TRUE);
			        CheckDlgButton(hDlg,IDC_PB6LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB6HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB6LOW),FALSE);}
				 break;
		    case IDC_Pattern7:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern7))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB7HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB7LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_PB7HI,TRUE);
			        CheckDlgButton(hDlg,IDC_PB7LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB7HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB7LOW),FALSE);}
				 break;
            case IDOK :
				 usPEnable = 0;
				 usPValue = 0;
		         for(i=0; i<8; i++)
		            if(IsDlgButtonChecked(hDlg,IDC_Pattern0 + i))
					{
					 usPEnable |= (0x01 << i);
					 if(IsDlgButtonChecked(hDlg,IDC_PB0HI + i))
						 usPValue |= (0x01 << i);
					}

            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}
/***************************************************************************
    FUNCTION: ConfigStatusProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ConfigStatusProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    USHORT   i;

    switch (message)
    {
    case WM_INITDIALOG :
      for(i=0; i<8; i++)
	  {
//		if(usFEnable & (0x01 << i))
//		{
		 EnableWindow(GetDlgItem(hDlg,(IDC_Status0 + i)),TRUE);
		 if(usSEnable & (0x01 << i))
		 {
		    CheckDlgButton(hDlg,(IDC_Status0 + i),TRUE);
			EnableWindow(GetDlgItem(hDlg,(IDC_SRiseTrig0+i)),TRUE);
			EnableWindow(GetDlgItem(hDlg,(IDC_SFallTrig0+i)),TRUE);
			if(usSRiseEdge & (0x01 << i))
			   CheckDlgButton(hDlg,(IDC_SRiseTrig0 + i),TRUE);
			else
			   CheckDlgButton(hDlg,(IDC_SRiseTrig0 + i),FALSE);

			if(usSFallEdge & (0x01 << i))
			   CheckDlgButton(hDlg,(IDC_SFallTrig0 + i),TRUE);
			else
			   CheckDlgButton(hDlg,(IDC_SFallTrig0 + i),FALSE);
		 }
		 else
		 {
		    CheckDlgButton(hDlg,(IDC_Status0 + i),FALSE);
			EnableWindow(GetDlgItem(hDlg,(IDC_SRiseTrig0+i)),FALSE);
			EnableWindow(GetDlgItem(hDlg,(IDC_SFallTrig0+i)),FALSE);
		 }
	  }

        return TRUE;

    case WM_COMMAND :

         switch (LOWORD(wParam))
         {
		    case IDC_Status0:
                 if(IsDlgButtonChecked(hDlg, IDC_Status0))
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig0),TRUE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig0),TRUE);}
		         else
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig0),FALSE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig0),FALSE);}
				 break;
		    case IDC_Status1:
                 if(IsDlgButtonChecked(hDlg, IDC_Status1))
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig1),TRUE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig1),TRUE);}
		         else
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig1),FALSE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig1),FALSE);}
				 break;
		    case IDC_Status2:
                 if(IsDlgButtonChecked(hDlg, IDC_Status2))
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig2),TRUE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig2),TRUE);}
		         else
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig2),FALSE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig2),FALSE);}
				 break;
		    case IDC_Status3:
                 if(IsDlgButtonChecked(hDlg, IDC_Status3))
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig3),TRUE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig3),TRUE);}
		         else
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig3),FALSE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig3),FALSE);}
				 break;
		    case IDC_Status4:
                 if(IsDlgButtonChecked(hDlg, IDC_Status4))
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig4),TRUE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig4),TRUE);}
		         else
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig4),FALSE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig4),FALSE);}
				 break;
		    case IDC_Status5:
                 if(IsDlgButtonChecked(hDlg, IDC_Status5))
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig5),TRUE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig5),TRUE);}
		         else
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig5),FALSE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig5),FALSE);}
				 break;
		    case IDC_Status6:
                 if(IsDlgButtonChecked(hDlg, IDC_Status6))
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig6),TRUE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig6),TRUE);}
		         else
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig6),FALSE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig6),FALSE);}
				 break;
		    case IDC_Status7:
                 if(IsDlgButtonChecked(hDlg, IDC_Status7))
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig7),TRUE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig7),TRUE);}
		         else
				 { EnableWindow(GetDlgItem(hDlg,IDC_SRiseTrig7),FALSE);
				   EnableWindow(GetDlgItem(hDlg,IDC_SFallTrig7),FALSE);}
				 break;
            case IDOK :
				 usSEnable = 0; usSRiseEdge = 0; usSFallEdge = 0;
		         for(i=0; i<8; i++)
				 {
		           if(IsDlgButtonChecked(hDlg, (IDC_Status0 + i)))
				   {
                      usSEnable |= (0x01 << i);

		           if(IsDlgButtonChecked(hDlg, (IDC_SRiseTrig0 + i)))
                      usSRiseEdge |= (0x01 << i);

		           if(IsDlgButtonChecked(hDlg, (IDC_SFallTrig0 + i)))
                      usSFallEdge |= (0x01 << i);

                   if(!IsDlgButtonChecked(hDlg, (IDC_SFallTrig0 + i)) && 
					  !IsDlgButtonChecked(hDlg, (IDC_SRiseTrig0 + i)))
					  usSEnable &= ~(0x01 << i);
				   }
				 }


            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}
/***************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
****************************************************************************/
BOOL FTYPE ConfigDlgProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    int         i;
    DWORD       dwIndex;
    char        szBuffer[40];
    int         nOutEntries;

    switch (message)
    {
    case WM_INITDIALOG :

        // --------------------------------
        // Initialize Device List Combobox
        // --------------------------------

        // get number of the installed devices


		if ((ErrCde = DRV_DeviceGetNumOfList((SHORT far *)&gnNumOfDevices)) !=
            SUCCESS)
        {
             DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
             MessageBox(hMainWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
             return TRUE;
        }



        if (gnNumOfDevices > MAX_DEVICES)
            gnNumOfDevices = MAX_DEVICES;

        // retrieve the information of all installed devices

        if ((ErrCde = DRV_DeviceGetList((DEVLIST far *)&DeviceList[0],
            (SHORT)gnNumOfDevices, (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return TRUE;
        }

        // initialize the Device List Combobox with the retrieved information

        for (i = 0; i < gnNumOfDevices; i ++)
            SendDlgItemMessage(hDlg, IDC_DEVICE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)DeviceList[i].szDeviceName));

        SendDlgItemMessage(hDlg, IDC_DEVICE, CB_SETCURSEL,(WPARAM)gwDevice,
                        (LPARAM)0);


        // -----------------------------------------------------------
        // Initialize Module List Combobox for COM port or CAN devices
        // -----------------------------------------------------------

        // check if there is any device attached on this COM port or CAN

        gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
        if (gnNumOfSubdevices > MAX_DEVICES)
            gnNumOfSubdevices = MAX_DEVICES;


        // retrieve the information of all installed devices
        if (gnNumOfSubdevices != 0)
        {
            if ((ErrCde = DRV_DeviceGetSubList(
                (DWORD)DeviceList[gwDevice].dwDeviceNum,
                (DEVLIST far *)&SubDeviceList[0],
                (SHORT)gnNumOfSubdevices,
                (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                return TRUE;
            }


            // initialize the Module List Combobox with the retrieved
            // information

            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));

            for (i = 0; i < gnNumOfSubdevices; i++)
                SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

            SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
					(WPARAM)gwSubDevice, (LPARAM)0);
        }
        else
        {
            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));
        }


        //
        // Initialize Input Channel
        //

        itoa(gwChannel, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ECHANNEL, EM_REPLACESEL, 0,
                        (LPARAM)((LPSTR)szBuffer));

        return TRUE;

    case WM_COMMAND :
         switch (LOWORD(wParam))
        {
            case IDC_FUNFILTER :
                 DialogBox (hInst, MAKEINTRESOURCE(IDD_FILTER),
                            hDlg, lpfnConfigFilterProc) ;
                 return 0;

            case IDC_FUNCOUNTER :
                 DialogBox (hInst, MAKEINTRESOURCE(IDD_COUNTER),
                            hDlg, lpfnConfigCounterProc) ;
                 return 0;

            case IDC_FUNPATTERN :
                 DialogBox (hInst, MAKEINTRESOURCE(IDD_PATTERN),
                            hDlg, lpfnConfigPatternProc) ;
                 return 0;

            case IDC_FUNSTATUS :
                 DialogBox (hInst, MAKEINTRESOURCE(IDD_STATUS),
                            hDlg, lpfnConfigStatusProc) ;
                 return 0;

            case IDOK :

              //
              // get device selection
              //

              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwDevice = (WORD)dwIndex;


              //
              // get sub-device selection
              //

              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_MODULE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwSubDevice = (WORD)dwIndex;


              //
              // get channel number
              //

              if (SendDlgItemMessage(hDlg, IDC_ECHANNEL,
                           EM_GETMODIFY, 0, 0))
              {
                    SendDlgItemMessage(hDlg, IDC_ECHANNEL,
                            WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwChannel = atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ECHANNEL,
                              EM_SETMODIFY, FALSE, 0) ;
              }

              EndDialog(hDlg, 0);
              return TRUE;

            case IDCANCEL :

                EndDialog(hDlg, 0);
                return TRUE;

            case IDC_DEVICE :

                //
                // When device selection is changed, it needs to
                // re-initialize sub-device combobox and input range
                // combobox.
                //

                if (HIWORD(wParam) == CBN_SELCHANGE)
                {
                    if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                        CB_GETCURSEL, 0, 0)) != CB_ERR)
                         gwDevice = (WORD)dwIndex;
                    else
                        return TRUE;


                    // ------------------------------------------------------
                    // Initialize Module Combobox for COM port or CAN devices
                    // ------------------------------------------------------

                    // check any device attached on this COM port or CAN

                    gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
                    if (gnNumOfSubdevices > MAX_DEVICES)
                        gnNumOfSubdevices = MAX_DEVICES;


                    // retrieve the information of all installed devices

                    if (gnNumOfSubdevices != 0)
                    {
                        if ((ErrCde = DRV_DeviceGetSubList(
                            (DWORD)DeviceList[gwDevice].dwDeviceNum,
                            (DEVLIST far *)&SubDeviceList[0],
                            (SHORT)gnNumOfSubdevices,
                            (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
                        {
                            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                            MessageBox(hMainWnd, (LPCSTR)szErrMsg,
                                "Driver Message", MB_OK);
                            return TRUE;
                        }


                        // initialize the Module List Combobox with the
                        // retrieved information

                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));

                        for (i = 0; i < gnNumOfSubdevices; i++)
                            SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                                (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

                        gwSubDevice = 0;

                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
							(WPARAM)gwSubDevice, (LPARAM)0);
                    }
                    else
                    {
                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));
                    }
                }

                return TRUE;
        }
        break;
    }

    return FALSE ;
}


/***************************************************************************
    FUNCTION: ScanDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ScanDlgProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    char        szBuffer[40];

    switch (message)
    {
    case WM_INITDIALOG :

        // ---------------------------------------------------
        // Initialize Scan Time Edit, Maximum accuracy: 40 ms
        // ---------------------------------------------------

        itoa(gwScanTime, szBuffer, 10);
        SendDlgItemMessage(hDlg, IDC_SCAN, EM_REPLACESEL, 0,
                        (LPARAM)((LPSTR)szBuffer));

        return TRUE;

    case WM_COMMAND :

         switch (LOWORD(wParam))
         {
            case IDOK :

                //
                // get scan time
                //

                if (SendDlgItemMessage(hDlg, IDC_SCAN,
                           EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_SCAN,
                            WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwScanTime = atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_SCAN,
                              EM_SETMODIFY, FALSE, 0) ;
                }

            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}


/***************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
****************************************************************************/

long FTYPE MainWndProc
(
    HWND        hWnd,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    static  HANDLE  hInstance ;
    HMENU           hMenu;
    HDC             hdc;
    char            szBuffer[120];
    RECT            rect;
    DWORD           dwCurrentTime;
    DWORD           dwErrorTime;
    BOOL            bTimerOverrun;
    PT_EnableEventEx      Zero = {0};

    switch (message)
    {
        case WM_CREATE:

            //
            // initialize Stop Menu
            //

            hMenu = GetMenu(hWnd);
            EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);

            hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

            lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

            lpfnScanDlgProc = MakeProcInstance (ScanDlgProc, hInstance) ;
            
            lpfnConfigCounterProc =  MakeProcInstance (ConfigCounterProc, hInstance) ;

            lpfnConfigFilterProc = MakeProcInstance (ConfigFilterProc, hInstance) ;
            
		    lpfnConfigPatternProc = MakeProcInstance (ConfigPatternProc, hInstance) ;
            
		    lpfnConfigStatusProc = MakeProcInstance (ConfigStatusProc, hInstance) ;
			return 0 ;

       case WM_COMMAND:     /* message: from application menu */

            hMenu = GetMenu(hWnd);

            switch (LOWORD(wParam))
            {
                case IDM_SETTING :
                    DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                         hWnd, lpfnConfigDlgProc) ;
                    return 0;

                case IDM_SCAN :
                    DialogBox (hInstance, MAKEINTRESOURCE(IDD_SCAN),
                         hWnd, lpfnScanDlgProc) ;
                    return 0;

                case IDM_STOP :
					{
                     BOOL bflag = TRUE;

                     // Close thread
                     m_bContinue = FALSE;
                     Sleep(10);

                    while(bflag)
					{
			         if(bThreadflag==FALSE)
				        bflag=FALSE;
					}


					ptEnableEventEx = Zero;
                    if ((ErrCde = DRV_EnableEventEx(DriverHandle,
                                                (LPT_EnableEventEx)&ptEnableEventEx)) != 0)
					  {
                       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                       MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                       DRV_DeviceClose((LONG far *)&DriverHandle);
                       return 0;
					  }
                    
                     // kill timer and close driver
                     //
                     KillTimer(hWnd, 1);
					 bRun = FALSE;

					DRV_DeviceClose((LONG far *)&DriverHandle);

                    //
                    // reset menu item status
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_ENABLED);

                    return 0;
					}

                case IDM_START :

                    //
                    // Open Device
                    //

                    if (gnNumOfSubdevices == 0)
                        ErrCde = DRV_DeviceOpen(
                            DeviceList[gwDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);
                    else
                        ErrCde = DRV_DeviceOpen(
                            SubDeviceList[gwSubDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);

                    if (ErrCde != SUCCESS)
                    {
                        strcpy(szErrMsg,"Device open error !");
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                        return 0;
                    }

                    //
                    // enable Stop Menu item and disable the others
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_ENABLED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_GRAYED);

                   // Enable Pattern Match event feature
                    if(usPEnable)
					{
				      ptEnableEventEx.Pattern.usEventType = ADS_EVT_PATTERNMATCH;
                      ptEnableEventEx.Pattern.usEventEnable = TRUE;
                      ptEnableEventEx.Pattern.usCount   = 1;
					  ptEnableEventEx.Pattern.usEnable = usPEnable;
					  ptEnableEventEx.Pattern.usValue = usPValue;
                      if ((ErrCde = DRV_EnableEventEx(DriverHandle,
                                                (LPT_EnableEventEx)&ptEnableEventEx)) != 0)
					  {
                       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                       MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                       DRV_DeviceClose((LONG far *)&DriverHandle);
                       return 0;
					  }

					}

					// Enable counter event feature
                    if(usCEnable)
					{
				      ptEnableEventEx.Counter.usEventType = ADS_EVT_COUNTER;
                      ptEnableEventEx.Counter.usEventEnable = TRUE;
                      ptEnableEventEx.Counter.usCount   = 1;
					  ptEnableEventEx.Counter.usEnable = usCEnable;
					  ptEnableEventEx.Counter.usTrigEdge = usTrigEdge;
					  ptEnableEventEx.Counter.usPreset = &usCPreset[0];
					  ptEnableEventEx.Counter.usMatchEnable = usCMatchEnable;
					  ptEnableEventEx.Counter.usValue = &usCMatchValue[0];
					  ptEnableEventEx.Counter.usOverflow = usCOverflow;
					  ptEnableEventEx.Counter.usDirection = 0;

                      if ((ErrCde = DRV_EnableEventEx(DriverHandle,
                                                (LPT_EnableEventEx)&ptEnableEventEx)) != 0)
					  {
                       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                       MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                       DRV_DeviceClose((LONG far *)&DriverHandle);
                       return 0;
					  }

					}
                   // Enable Status Change event feature
                    if(usSEnable)
					{
				      ptEnableEventEx.Status.usEventType = ADS_EVT_STATUSCHANGE;
                      ptEnableEventEx.Status.usEventEnable = TRUE;
                      ptEnableEventEx.Status.usCount   = 1;
					  ptEnableEventEx.Status.usEnable = usSEnable;
					  ptEnableEventEx.Status.usRisingedge = usSRiseEdge;
					  ptEnableEventEx.Status.usFallingedge = usSFallEdge;
                      if ((ErrCde = DRV_EnableEventEx(DriverHandle,
                                                (LPT_EnableEventEx)&ptEnableEventEx)) != 0)
					  {
                       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                       MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                       DRV_DeviceClose((LONG far *)&DriverHandle);
                       return 0;
					  }

					}
                   // Enable Filter event feature
                    if(usFEnable)
					{
				      ptEnableEventEx.Filter.usEventType = ADS_EVT_FILTER;
                      ptEnableEventEx.Filter.usEventEnable = TRUE;
                      ptEnableEventEx.Filter.usCount   = 1;
					  ptEnableEventEx.Filter.usEnable = usFEnable;
					  ptEnableEventEx.Filter.usHiValue = &usFHiValue[0];
					  ptEnableEventEx.Filter.usLowValue = &usFLowValue[0];
                      if ((ErrCde = DRV_EnableEventEx(DriverHandle,
                                                (LPT_EnableEventEx)&ptEnableEventEx)) != 0)
					  {
                       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                       MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                       DRV_DeviceClose((LONG far *)&DriverHandle);
                       return 0;
					  }

					}

					
					// create thread
					if(usPEnable || usCEnable || usSEnable)
					{
                     hThreadHandle = CreateThread(0, 0, (LPTHREAD_START_ROUTINE) UserThread,
                                                 0, 0, (LPDWORD)&dwThreadID);
                     m_bContinue = TRUE;
					}
					//
                    // Set the new timer type and start it up.
                    //

                    if (SetTimer(hWnd, 1, gwScanTime, NULL) == (UINT)NULL)
                    {
                        MessageBox(hWnd, "Couldn't create timer.", "Notice!",
                            MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    bRun = TRUE;

                    gdwStartTime = GetTickCount();
                    gwOverrunCount = 0;

                    return 0;
            }
            return 0;

        case WM_TIMER:

            //
            // Timer set to dispatch a WM_TIMER message has gone off.
            //

            //
            // check if timer is overrun
            //

            dwCurrentTime = GetTickCount();

            gdwElapseTime = gdwElapseTime + (DWORD)gwScanTime;
            dwErrorTime = dwCurrentTime - gdwElapseTime - gdwStartTime;
            if (dwErrorTime > (DWORD)40)        // maximum 40 ms scan time
            {
                gdwElapseTime = dwCurrentTime - gdwStartTime;
                gwOverrunCount ++;
                if (gwOverrunCount >= 10)
                {
                    bTimerOverrun = TRUE;
                    gwOverrunCount = 0;
                }
                else
                    bTimerOverrun = FALSE;
            }
            else
            {
                bTimerOverrun = FALSE;
                gwOverrunCount = 0;
            }


            //
            // read data
            //

            
		
			ptDioReadBit.port = gwChannel/8;
			ptDioReadBit.bit = gwChannel%8;
			ptDioReadBit.state = (USHORT far *)&gwState;

           	if ((ErrCde = DRV_DioReadBit(DriverHandle,
               (LPT_DioReadBit)&ptDioReadBit)) != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                KillTimer(hWnd, 1);
                MessageBox(hWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
                return TRUE;
            }

            //
            // Display Data
            //

            hdc = GetDC(hWnd);
            GetClientRect(hWnd, &rect);
            InvalidateRect(hWnd, NULL, TRUE);
            UpdateWindow(hWnd);
            if (bTimerOverrun == FALSE)
				sprintf(szBuffer, "Data (Hex) = %2X ",gwState);
            else
                sprintf(szBuffer, "Timer Overrun!");

            TextOut(hdc, 200, 50, szBuffer, strlen(szBuffer));
            
			sprintf(szBuffer,"Pattern Match count=%d", usPCount);
            TextOut(hdc, 100, 70, szBuffer, strlen(szBuffer));

            sprintf(szBuffer,"Counter Match channel=%d count=%d", retValueCM,usCMCount);
            TextOut(hdc, 100, 90, szBuffer, strlen(szBuffer));

            sprintf(szBuffer,"Counter Overflow channel=%d count=%d", retValueCO,usCOCount);
            TextOut(hdc, 100, 110, szBuffer, strlen(szBuffer));

	        if (retValueS & 0x0ff)
			{
		        sprintf(szBuffer,"Status Change(falling) channel=%d count=%d", retValueS, usFSCount);
                TextOut(hdc, 100, 130, szBuffer, strlen(szBuffer));
			}
	        else if (retValueS & 0x0ff00)
			{
		        sprintf(szBuffer,"Status Change(rising) channel=%d count=%d", retValueS>>8, usRSCount);
                TextOut(hdc, 100, 130, szBuffer, strlen(szBuffer));
			}

            ReleaseDC(hWnd, hdc);

            return 0;

        case WM_DESTROY:

            if (bRun == TRUE)
            {
                KillTimer(hWnd, 1);
                DRV_DeviceClose((LONG far *)&DriverHandle);
            }
            PostQuitMessage(0);
            break;

      default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
  }

   return ((LONG)NULL);
}

//------------------------------------------------------------------
void UserThread()
{
    USHORT usEventType;
    LONG  ErrCde;
    
	while(m_bContinue)
    {
        // Check message
        ptCheckEvent.EventType = &usEventType;
        ptCheckEvent.Milliseconds = 5000;

		bThreadflag = TRUE;
        if ((ErrCde = DRV_CheckEvent(DriverHandle,
            (LPT_CheckEvent)&ptCheckEvent)) != 0)
        {
            MessageBox(hMainWnd,"Check Event Error !","Thread Message",MB_OK);
            return ;
        }

		// Process pattern match event
        if (usEventType == ADS_EVT_PATTERNMATCH)
            adPatternMatchEvent();

        // Process counter match event
        if (usEventType == ADS_EVT_COUNTERMATCH)
            adCounterMatchEvent();

        // Process counter overflow event
        if (usEventType == ADS_EVT_COUNTEROVERFLOW)
            adCounterOverflowEvent();

        // Process status change event
        if (usEventType == ADS_EVT_STATUSCHANGE)
            adStatusChangeEvent();

		bThreadflag = FALSE;
    }

    return ;
}

//------------------------------------------------------------------
void adPatternMatchEvent()
{
	ULONG  retValue;

    ptFDITransfer.usEventType = ADS_EVT_PATTERNMATCH;
	ptFDITransfer.ulRetData = &retValue; 

    if ((ErrCde = DRV_FDITransfer(DriverHandle,
        (LPT_FDITransfer)&ptFDITransfer)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }
    
    usPCount++;
    return;
}

//------------------------------------------------------------------
void adCounterMatchEvent()
{
	ULONG  retValue;

    ptFDITransfer.usEventType = ADS_EVT_COUNTERMATCH;
	ptFDITransfer.ulRetData = &retValue; 

    if ((ErrCde = DRV_FDITransfer(DriverHandle,
        (LPT_FDITransfer)&ptFDITransfer)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    retValueCM = retValue;
    usCMCount++;

    return;
}
//------------------------------------------------------------------
void adCounterOverflowEvent()
{
	ULONG  retValue;

    ptFDITransfer.usEventType = ADS_EVT_COUNTEROVERFLOW;
	ptFDITransfer.ulRetData = &retValue; 

    if ((ErrCde = DRV_FDITransfer(DriverHandle,
        (LPT_FDITransfer)&ptFDITransfer)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    retValueCO = retValue;
    usCOCount++;

    return;
}
//------------------------------------------------------------------
void adStatusChangeEvent()
{
	ULONG    retValue;

    ptFDITransfer.usEventType = ADS_EVT_STATUSCHANGE;
	ptFDITransfer.ulRetData = (ULONG far *)&retValue; 

    if ((ErrCde = DRV_FDITransfer(DriverHandle,
        (LPT_FDITransfer)&ptFDITransfer)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    retValueS = retValue;
	if (retValue & 0x0ff)
		usFSCount++;
	else if (retValue & 0x0ff00)
		usRSCount++;

    return;
}
