/*
 **************************************************************************
 *  Program     : MADDMA.C                                                *
 *  Description : Demo program for Windows NT with DMA data transfer      *
 *  Revision    : 1.00                                                    *
 *  Date        : 10/15/1997                   Advantech Co., Ltd.        *
 ************************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance() - Saves instance handle and creates main window
 *
 *   MainWndProc() - Process main window messages
 *
 *   ShowDataDlgProc() - Process the dialog box messages for acquired data
 *
 *   DisplayDlgProc() - Process the dialog box messages for display range
 *
 *   ConfigDlgProc() - Process the dialog box messages for configuration
 *
 *   GainListDlgProc() - Process the dialog box messages for gain list
 */

/*****************************************************************************\
Modification Log:
	Vers    Date     By         Notes
	----    -------- ---        -----
	1.00    01/01/98 Rose    -  Created
	1.01    05/10/98 Charley -  Modify error of setting gain code
                                B980529
    1.03    2/5/99   Eric    -  Modify thread Problem
\*****************************************************************************/

#include <windows.h>              /* required for all Windows applications */
#include <stdlib.h>
#include <windowsx.h>
#include <commdlg.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "..\..\..\include\driver.h"
#include "..\..\..\include\os.h"
#include "resource.h"
#include "maddma.h"

// ------------------------------------------------------------------------
// CONSTANT DEFINITION
// ------------------------------------------------------------------------
#define     MAX_DEVICES     100

// ------------------------------------------------------------------------
// GLOBAL VARIABLES
// ------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hWnd;                       // main window handle

static      DEVFEATURES        DevFeatures;       // structure for device features
static      PT_FAIDmaScanStart ptFAIDmaScanStart; // FAIIntScanStart table
static      PT_FAITransfer     ptFAITransfer;     // FAITransfer table
static      PT_FAICheck        ptFAICheck;        // FAICheck table
static      PT_DeviceGetFeatures  ptDevFeatures;  // Devfeatures table
static      PT_AllocateDMABuffer  ptAllocateDMABuffer;  // buffer table

static      PT_EnableEvent     ptEnableEvent;     // Enable event
static      PT_CheckEvent      ptCheckEvent;      // Check event
HANDLE      hEvent;
HANDLE      hThreadHandle;
DWORD       dwThreadID;

char        szErrMsg[80];               // Use for MESSAGEBOX function
char        szBuffer[40];				// Temperatory buffer
LRESULT     ErrCde;                     // Return error code

//
// used for fixed memory for the installed devices
//

DEVLIST     DeviceList[MAX_DEVICES];
DEVLIST     SubDeviceList[MAX_DEVICES];

BOOL        bThreadflag = FALSE;
BOOL        bThreadloop = FALSE;
BOOL        bRun = FALSE;                       // flag for running
USHORT      gwScanTime = 1000;                  // scan time
DWORD       gdwStartTime;                       // start time
DWORD       gdwElapseTime = 0;                  // elapse time
USHORT      gwOverrunCount = 0;                 // overrun count

static  USHORT gwGain[16] = {0,0,0,0,0,0,0,0,
                             0,0,0,0,0,0,0,0};
static  USHORT gwGainCode;

static  USHORT gwGainCde[16];

static  LONG   DriverHandle = (LONG)NULL;         // driver handle
static  USHORT gwDevice = 0, gwSubDevice = 0;     // device index
static  SHORT  gnNumOfDevices, gnNumOfSubdevices; // number of installed devices
static  DWORD  gdwPacerRate = 4000;               // pacer rate
static  ULONG  gulConvNum = 4000;                 // conversion number
static  USHORT gwStartChl = 0, gwNumChl = 1;      // scan channels
static  USHORT gwDataType = 1;                    // display type : FLOAT(1)
static  USHORT gwEvtFlag = 1;                     // event enable(0)
static  ULONG  gwActualBufSize = 0;               // actual allocate buffer size
static  USHORT gwFifoEnable = 0;                  // Fifo disable(0)

static  USHORT gwGainList = 0;      // gain list flag
static  ULONG  gdwStartPt = 0, gdwStopPt = 99;   // display points
static  USHORT gwExtTrig = 0;       // external or internal trigger

static  USHORT gwCyclicMode = 0;    // cyclic or non-cyclic mode
static  USHORT gwBufferMode = 0;    // buffer: single or double
static  USHORT gwIntrCount = 1;
static  USHORT gwCyclicCount= 0;    // number of cyclic count

static  USHORT gwActiveBuf = 0;     // return by FAICheck
static  USHORT gwOverrun   = 0;     // return by FAICheck, FAITransfer
static  USHORT gwStopped   = 0;     // return by FAICheck
static  ULONG gulRetrieved = 0;     // return by FAICheck
static  USHORT gwHalfReady = 0;     // return by FAICheck

HGLOBAL hUserBuf;

BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, int nCmdShow);
long FTYPE MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
int  FTYPE WinMain(HANDLE hInstance, HANDLE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow);
static FARPROC lpfnConfigDlgProc, lpfnShowDataDlgProc, lpfnDisplayDlgProc,
    lpfnGainListDlgProc;

static  HWND hwndStopButton, hwndStatusButton;
static  HANDLE  hInstance ;
HMENU hMenu;
LONG pBuffer;

void UserThread();
void adInterruptEvent();
void adBufChangeEvent();
void adOverrunEvent();
void adTerminateEvent();
void adstopai();

/***************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
****************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                          /* current instance             */
HANDLE hPrevInstance;                      /* previous instance            */
LPSTR lpCmdLine;                           /* command line                 */
int nCmdShow;                              /* show-window type (open/icon) */
{
    MSG msg;                               /* message                      */

    if (!hPrevInstance)                /* Other instances of app running?  */
        if (!InitApplication(hInstance)) /* Initialize shared things         */
            return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,      /* message structure                      */
        (HWND)NULL,                    /* handle of window receiving the message */
        (UINT)NULL,                    /* lowest message to examine              */
        (UINT)NULL))                   /* highest message to examine             */
    {
        TranslateMessage(&msg);  /* Translates virtual key codes           */
        DispatchMessage(&msg);   /* Dispatches message to window           */
    }
    return (msg.wParam);         /* Returns the value from PostQuitMessage */
}

/****************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                            /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW; /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc; /* window process procedure     */
    wc.cbClsExtra = 0;                  /* No per-class extra data.        */
    wc.cbWndExtra = 0;                  /* No per-window extra data.       */
    wc.hInstance = hInstance;           /* Application that owns the class.*/
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";   /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";  /* Name used in call to CreateWindow. */

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));

}

/***************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
****************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
HANDLE          hInstance;          /* Current instance identifier.          */
int             nCmdShow;           /* Param for first ShowWindow() call.    */
{

    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hWnd = CreateWindow(
        "MyClass",                  /* See RegisterClass() call.           */
        "Advantech Driver Demo : DMA Data Transfer" ,
        /* Window title bar  */
        WS_OVERLAPPEDWINDOW,        /* Window style.                       */
        CW_USEDEFAULT,              /* Default horizontal position.        */
        CW_USEDEFAULT,              /* Default vertical position.          */
        CW_USEDEFAULT,              /* Default width.                      */
        CW_USEDEFAULT,              /* Default height.                     */
        NULL,                       /* Overlapped windows have no parent.  */
        NULL,                       /* Use the window class menu.          */
        hInstance,                  /* This instance owns this window.     */
        NULL                        /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hWnd, nCmdShow);   /* Show the window                       */
    UpdateWindow(hWnd);           /* Sends WM_PAINT message                */
    return (TRUE);                /* Returns the value from PostQuitMessage*/
}

/***************************************************************************
    FUNCTION: GainListDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for gain list
****************************************************************************/

BOOL FTYPE GainListDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                                /* window handle                 */
unsigned message;                         /* type of message               */
WPARAM wParam;                            /* additional information        */
LPARAM lParam;                            /* additional information        */
{
    USHORT i, j, temp;

    switch (message)
    {
    case WM_INITDIALOG :

        // ------------------------------------------------------------
        // Initialize Input Range List Combobox, it needs to get device
        // features for gain list
        // ------------------------------------------------------------

        // Step 1: Open Device
        if (gnNumOfSubdevices == 0)
        {
            ErrCde = DRV_DeviceOpen(DeviceList[gwDevice].dwDeviceNum,
                (LONG far *)&DriverHandle);
        }
        else
        {
            ErrCde = DRV_DeviceOpen(
                SubDeviceList[gwSubDevice].dwDeviceNum,
                (LONG far *)&DriverHandle);
        }

        if (ErrCde != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return 0;
        }

        // Step 2: Get device features
        ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
        ptDevFeatures.size = sizeof(DEVFEATURES);
        if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
            (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
            return 0;
        }

        // Step 3: Initialize Input Range List Combobox with device features
        for (i = 0; i < 16; i ++)
            EnableWindow(GetDlgItem(hDlg, IDC_CHL0 + i), FALSE);

        if ((DevFeatures.usNumGain != 0) && ((gwNumChl+gwStartChl) <= 16))
        {
            for(i = (USHORT)gwStartChl; i < (gwNumChl+gwStartChl); i++)
            {
                EnableWindow(GetDlgItem(hDlg, IDC_CHL0 + i), TRUE);
                for(j = 0; j < (USHORT)DevFeatures.usNumGain; j ++)
                    SendDlgItemMessage(hDlg, IDC_CHL0+i, CB_ADDSTRING, 0,
                    (LPARAM)((LPSTR)(DevFeatures.glGainList[j].szGainStr)));

                temp = gwGain[i];

                SendDlgItemMessage(hDlg, IDC_CHL0+i,
                    CB_SELECTSTRING, (WPARAM)(-1),
                    (LPARAM)((LPSTR)(DevFeatures.glGainList[temp].szGainStr)));
            }
        }

        // Step 4: Close device
        DRV_DeviceClose((LONG far *)&DriverHandle);

        return TRUE;

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDOK :
        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;

        case IDC_CHL0 :
        case IDC_CHL1 :
        case IDC_CHL2 :
        case IDC_CHL3 :
        case IDC_CHL4 :
        case IDC_CHL5 :
        case IDC_CHL6 :
        case IDC_CHL7 :
        case IDC_CHL8 :
        case IDC_CHL9 :
        case IDC_CHL10 :
        case IDC_CHL11 :
        case IDC_CHL12 :
        case IDC_CHL13 :
        case IDC_CHL14 :
        case IDC_CHL15 :
            if (HIWORD(wParam) == CBN_KILLFOCUS)
            {
                i = (int)SendDlgItemMessage(hDlg, LOWORD(wParam),
                    CB_GETCURSEL, 0, 0) ;
                if (i != CB_ERR)
                    gwGain[((USHORT)LOWORD(wParam)) - IDC_CHL0] = i;
            }
            return TRUE;

        }
        break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
****************************************************************************/

BOOL FTYPE ConfigDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                          /* additional information          */
LPARAM lParam;                          /* additional information          */
{
    int  i;
    DWORD       dwIndex;
    char        szBuffer[40];
    int         nOutEntries;


    switch (message)
    {
    case WM_INITDIALOG :

        // --------------------------------
        // Initialize Device List Combobox
        // --------------------------------

        // Get number of the installed devices
        if ((ErrCde = DRV_DeviceGetNumOfList((SHORT far *)&gnNumOfDevices)) !=
            SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
            return TRUE;
        }

        if (gnNumOfDevices > MAX_DEVICES)
            gnNumOfDevices = MAX_DEVICES;

        // Retrieve the information of all installed devices

        if ((ErrCde = DRV_DeviceGetList((DEVLIST far *)&DeviceList[0],
            (SHORT)gnNumOfDevices, (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return TRUE;
        }

        // Initialize the Device List Combobox with the retrieved information
        for (i = 0; i < gnNumOfDevices; i ++)
            SendDlgItemMessage(hDlg, IDC_DEVICE, CB_ADDSTRING, 0,
            (LPARAM)((LPSTR)DeviceList[i].szDeviceName));

        SendDlgItemMessage(hDlg, IDC_DEVICE, CB_SETCURSEL,(WPARAM)gwDevice,
            (LPARAM)0);


        // -----------------------------------------------------------
        // Initialize Module List Combobox for COM port or CAN devices
        // -----------------------------------------------------------

        // Check if there is any device attached on this COM port or CAN
        gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
        if (gnNumOfSubdevices > MAX_DEVICES)
            gnNumOfSubdevices = MAX_DEVICES;

        // Retrieve the information of all installed devices
        if (gnNumOfSubdevices != 0)
        {
            if ((ErrCde = DRV_DeviceGetSubList(
                (DWORD)DeviceList[gwDevice].dwDeviceNum,
                (DEVLIST far *)&SubDeviceList[0],
                (SHORT)gnNumOfSubdevices,
                (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                return TRUE;
            }


            // Initialize the Module List Combobox with the retrieved
            // information
            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                (LPARAM)((LPSTR)0));

            for (i = 0; i < gnNumOfSubdevices; i++)
                SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

            SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
                (WPARAM)gwSubDevice, (LPARAM)0);
        }
        else
        {
            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                (LPARAM)((LPSTR)0));
        }

        // ------------------------------------------------------------
        // Initialize Input Range List Combobox, it needs to get device
        // features for gain list
        // ------------------------------------------------------------

        // Step 1: Open device
        if (gnNumOfSubdevices == 0)
        {
            ErrCde = DRV_DeviceOpen(DeviceList[gwDevice].dwDeviceNum,
                (LONG far *)&DriverHandle);
        }
        else
        {
            ErrCde = DRV_DeviceOpen(
                SubDeviceList[gwSubDevice].dwDeviceNum,
                (LONG far *)&DriverHandle);
        }

        if (ErrCde != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return 0;
        }

        // Step 2: Get device features
        ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
        ptDevFeatures.size = sizeof(DEVFEATURES);
        if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
            (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
            return 0;
        }

        // Step 3: Initialize Input Range List Combobox with device features
        if (DevFeatures.usNumGain != 0)
        {
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
            for (i = 0; i < (int)DevFeatures.usNumGain; i ++)
                SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_ADDSTRING, 0,
                (LPARAM)((LPSTR)(DevFeatures.glGainList[i].szGainStr)));

            SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_SELECTSTRING, (WPARAM)(-1),
                (LPARAM)((LPSTR)(DevFeatures.glGainList[gwGainCode].szGainStr)));

        }
        else
        {
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
            SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                0, (LPARAM)((LPSTR)0));
        }

        // Initialize Conversion Number
        ultoa(gulConvNum, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_CONVERSION, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Pacer Rate
        ultoa(gdwPacerRate, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_PACER, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        if (gwExtTrig)
            EnableWindow(GetDlgItem(hDlg, IDC_PACER), FALSE);
        else
            EnableWindow(GetDlgItem(hDlg, IDC_PACER), TRUE);

        // Initialize Start Channel
        itoa((int)gwStartChl, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTART, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Stop Channel
        itoa((int)gwNumChl, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTOP, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Triggering
        CheckRadioButton(hDlg, IDC_INTTRIG, IDC_EXTTRIG,
            (gwExtTrig)?IDC_EXTTRIG:IDC_INTTRIG);

        // Initialize Repeated Mode
        CheckRadioButton(hDlg, IDC_CYCLIC, IDC_NONCYCLIC,
            (gwCyclicMode)?IDC_CYCLIC:IDC_NONCYCLIC);

        // Initialize Buffer Mode
        EnableWindow(GetDlgItem(hDlg, IDC_BUFSINGLE), FALSE);
        EnableWindow(GetDlgItem(hDlg, IDC_BUFDOUBLE), FALSE);

        // CheckRadioButton(hDlg, IDC_BUFSINGLE, IDC_BUFDOUBLE,
        //     (gwBufferMode)?IDC_BUFDOUBLE:IDC_BUFSINGLE);

        // Initialize Data Type
        CheckRadioButton(hDlg, IDC_DATAUSHORT, IDC_DATAFLOAT,
            (gwDataType)?IDC_DATAFLOAT:IDC_DATAUSHORT);

        // Initialize Message Flag
        CheckRadioButton(hDlg, IDC_EVTENABLE, IDC_EVTDISABLE,
            (gwEvtFlag)?IDC_EVTENABLE:IDC_EVTDISABLE);

        // Initialize number of interrupt count
        if (gwFifoEnable)
            SendDlgItemMessage(hDlg, IDC_FifoEnable,BM_SETCHECK,1,0);

        // Initialize gain table option
        if (gwGainList)
        {
            CheckRadioButton(hDlg, IDC_OVERALLGAIN, IDC_GAINARRAY,
                IDC_GAINARRAY);
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
            EnableWindow(GetDlgItem(hDlg, IDC_GAINLIST), TRUE);
        }
        else
        {
            CheckRadioButton(hDlg, IDC_OVERALLGAIN, IDC_GAINARRAY,
                IDC_OVERALLGAIN);
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
            EnableWindow(GetDlgItem(hDlg, IDC_GAINLIST), FALSE);
        }

        // Step 4: Close device
        DRV_DeviceClose((LONG far *)&DriverHandle);

        return TRUE;

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDOK :
        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;
        case IDC_GAINLIST :
            DialogBox (hInst, MAKEINTRESOURCE(IDD_GAINLIST),
                hDlg, lpfnGainListDlgProc) ;
            return TRUE;

        case IDC_DEVICE :

            //
            // When device selection is changed, it needs to
            // re-initialize sub-device combobox and input range
            // combobox.
            //

            if (HIWORD(wParam) == CBN_SELCHANGE)
            {
                if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                    CB_GETCURSEL, 0, 0)) != CB_ERR)
                    gwDevice = (WORD)dwIndex;
                else
                    return TRUE;


                // ------------------------------------------------------
                // Initialize Module Combobox for COM port or CAN devices
                // ------------------------------------------------------
                // Check any device attached on this COM port or CAN

                gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
                if (gnNumOfSubdevices > MAX_DEVICES)
                    gnNumOfSubdevices = MAX_DEVICES;


                // Retrieve the information of all installed devices
                if (gnNumOfSubdevices != 0)
                {
                    if ((ErrCde = DRV_DeviceGetSubList(
                        (DWORD)DeviceList[gwDevice].dwDeviceNum,
                        (DEVLIST far *)&SubDeviceList[0],
                        (SHORT)gnNumOfSubdevices,
                        (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd, (LPCSTR)szErrMsg,
                            "Driver Message", MB_OK);
                        return TRUE;
                    }


                    // Initialize the Module List Combobox with the
                    // retrieved information
                    EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
                    SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                        0, (LPARAM)((LPSTR)0));

                    for (i = 0; i < gnNumOfSubdevices; i++)
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

                    gwSubDevice = 0;

                    SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
                        (WPARAM)gwSubDevice, (LPARAM)0);
                }
                else
                {
                    EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
                    SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                        0, (LPARAM)((LPSTR)0));
                }


                // -------------------------------------------------
                // Initialize Input Range List Combobox, it needs to
                // get device features for gain list
                // -------------------------------------------------

                // Step 1: Open Device
                if (gnNumOfSubdevices == 0)
                {
                    // for non COM port or CAN device
                    ErrCde = DRV_DeviceOpen(
                        DeviceList[gwDevice].dwDeviceNum,
                        (LONG far *)&DriverHandle);
                }
                else
                {
                    // for COM port or CAN device
                    ErrCde = DRV_DeviceOpen(
                        SubDeviceList[gwSubDevice].dwDeviceNum,
                        (LONG far *)&DriverHandle);
                }

                if (ErrCde != SUCCESS)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd, (LPCSTR)szErrMsg,
                        "Driver Message", MB_OK);
                    return 0;
                }

                // Step 2: Get device features
                ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
                ptDevFeatures.size = sizeof(DEVFEATURES);
                if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                    (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd, (LPCSTR)szErrMsg, "Driver Message",
                        MB_OK);
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                // Step 3: Initialize Range Combobox with device features
                gwGainCode = 0;

                if (DevFeatures.usNumGain != 0)
                {
                    EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
                    SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                        0, (LPARAM)((LPSTR)0));
                    for (i = 0; i < (int)DevFeatures.usNumGain; i ++)
                        SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)(DevFeatures.glGainList[i].szGainStr)));

                    SendDlgItemMessage(hDlg, IDC_INPRANGE,
                        CB_SELECTSTRING, (WPARAM)(-1),
                        (LPARAM)((LPSTR)(DevFeatures.glGainList[gwGainCode].szGainStr)));
                }
                else
                {
                    EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
                    SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                        0, (LPARAM)((LPSTR)0));
                }

                // Step 4: Close device
                DRV_DeviceClose((LONG far *)&DriverHandle);
            }

            return TRUE;

        case IDC_CONVERSION :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_CONVERSION,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_CONVERSION,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gulConvNum = (ULONG)atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_CONVERSION,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;
        case IDC_PACER :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_PACER,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_PACER,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gdwPacerRate = (ULONG)atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_PACER,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;
        case IDC_INPRANGE :
            if (HIWORD(wParam) == CBN_KILLFOCUS)
            {
                i = (int)SendDlgItemMessage(hDlg, IDC_INPRANGE,
                    CB_GETCURSEL, 0, 0) ;
                if (i != CB_ERR)
                    gwGainCode = (USHORT)i;
            }
            return TRUE;
        case IDC_ESTART :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTART,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTART,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwStartChl = (USHORT)atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTART,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;

        case IDC_ESTOP :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTOP,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTOP,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwNumChl = (USHORT)atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTOP,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }

            return TRUE;

        case IDC_FifoEnable :
            gwFifoEnable = (int)SendDlgItemMessage(hDlg, IDC_FifoEnable,
											BM_GETCHECK,0,0);
            return TRUE;

        case IDC_OVERALLGAIN :
            gwGainList = 0;
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
            EnableWindow(GetDlgItem(hDlg, IDC_GAINLIST), FALSE);
            break;
        case IDC_GAINARRAY :
            gwGainList = 1;
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
            EnableWindow(GetDlgItem(hDlg, IDC_GAINLIST), TRUE);
            break;
        case IDC_INTTRIG :
            EnableWindow(GetDlgItem(hDlg, IDC_PACER), TRUE);
            gwExtTrig = 0;
            break;
        case IDC_EXTTRIG :
            EnableWindow(GetDlgItem(hDlg, IDC_PACER), FALSE);
            gwExtTrig = 1;
            break;
        case IDC_CYCLIC :
            gwCyclicMode = 1;
            break;
        case IDC_NONCYCLIC :
            gwCyclicMode = 0;
            break;
        case IDC_DATAUSHORT :
            gwDataType = 0;
            break;
        case IDC_DATAFLOAT :
            gwDataType = 1;
            break;
        case IDC_EVTENABLE :
            gwEvtFlag = 1;
            break;
        case IDC_EVTDISABLE :
            gwEvtFlag = 0;
            break;
        case IDC_BUFSINGLE :
            gwBufferMode = 0;
            break;
        case IDC_BUFDOUBLE :
            gwBufferMode = 1;
            break;
        }
        break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: ShowDataDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for acquired data
****************************************************************************/

BOOL FTYPE ShowDataDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                                /* window handle                 */
unsigned message;                         /* type of message               */
WPARAM wParam;                            /* additional information        */
LPARAM lParam;                            /* additional information        */
{
    ULONG  i;
    LPVOID temp;

    switch (message)
    {
    case WM_INITDIALOG :

        if (gwDataType == 0)
            temp = (USHORT far *)ptFAITransfer.DataBuffer;
        else
            temp = (FLOAT far *)ptFAITransfer.DataBuffer;

        for(i = gdwStartPt; i <= gdwStopPt; ++i)
        {

            if (gwDataType == 0)
                sprintf(szBuffer, "Buf[%ld] = %4x",i, ((USHORT far *)temp)[i]);
            else
                sprintf(szBuffer, "Buf[%ld] = %10.6f", i, ((FLOAT far *)temp)[i]);

            if(SendDlgItemMessage(hDlg, IDC_DATALIST, LB_ADDSTRING, 0,
                (LPARAM)(LPSTR)szBuffer) == LB_ERRSPACE)
            {   MessageBox(hWnd,(LPCSTR)"Not Enough Memory",
                    "Display Data",MB_OK);
                return TRUE;
            }
        }


        return TRUE;

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDOK :
        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;

        }
        break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: DisplayDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for display range
****************************************************************************/

BOOL FTYPE DisplayDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                               /* window handle                   */
unsigned message;                        /* type of message                 */
WPARAM wParam;                           /* additional information          */
LPARAM lParam;                           /* additional information          */
{

    switch (message)
    {
    case WM_INITDIALOG :

        // Initialize Start Point
        ultoa(gdwStartPt, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTAPT, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Stop Point
        ultoa(gdwStopPt, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTOPT, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        return TRUE;

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDOK :
        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;

        case IDC_ESTAPT :

            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTAPT,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTAPT,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gdwStartPt = (USHORT)atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTAPT,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;

        case IDC_ESTOPT :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTOPT,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTOPT,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gdwStopPt = atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTOPT,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;
        }
        break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
****************************************************************************/

long FTYPE MainWndProc(hWnd, message, wParam, lParam)
HWND hWnd;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                          /* additional information          */
LPARAM lParam;                          /* additional information          */
{
    int     dec, sign;
    TEXTMETRIC tm;
    HDC     hdc;
    static  int cxChar, cyChar;
    char    *szStr, szDest[100], szTemp[50];
    double  ratio;
//    DWORD   dwExitCode;
//    BOOL    bThreadStatus;
    USHORT temp,i;

    switch (message)
    {
    case WM_CREATE:

        hMenu = GetMenu(hWnd);

        hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

        lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

        lpfnShowDataDlgProc = MakeProcInstance (ShowDataDlgProc, hInstance) ;

        lpfnDisplayDlgProc = MakeProcInstance (DisplayDlgProc, hInstance) ;

        lpfnGainListDlgProc = MakeProcInstance (GainListDlgProc, hInstance) ;

        hdc = GetDC(hWnd);
        SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));
        GetTextMetrics(hdc, &tm);
        cxChar = tm.tmAveCharWidth;
        cyChar = tm.tmHeight + tm.tmExternalLeading;
        ReleaseDC(hWnd, hdc);

        //Create stop button and status button
        hwndStopButton = CreateWindow("button", "Stop",
            (DWORD)(WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON),
            0, 0, 8*cxChar, 3*cyChar/2, hWnd,
            (HMENU)IDC_STOPBUTTON,
            (HANDLE)hInstance, (LPVOID)NULL);

        hwndStatusButton = CreateWindow("button", "Status",
            (DWORD)(WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON),
            8*cxChar, 0, 8*cxChar, 3*cyChar/2, hWnd,
            (HMENU)IDC_STATUSBUTTON,
            (HANDLE)hInstance, (LPVOID)NULL);

        EnableWindow(hwndStopButton, FALSE);
        EnableWindow(hwndStatusButton, FALSE);

        return 0 ;

    case WM_COMMAND:     /* message: from application menu */

        hMenu = GetMenu(hWnd);

        switch(LOWORD(wParam))
        {
            // Show the dialog box of the input parameter
        case IDM_SETTING :

            DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                hWnd, lpfnConfigDlgProc) ;
            return 0;

            // Show the dialog box for the range of data to display
        case IDM_DISPLAY:

            DialogBox (hInstance, MAKEINTRESOURCE(IDD_DISPLAY),
                hWnd, lpfnDisplayDlgProc) ;

            return 0;

        case IDM_RUN :


            // Step 1: Device open
            if (gnNumOfSubdevices == 0)
                ErrCde = DRV_DeviceOpen(
                DeviceList[gwDevice].dwDeviceNum,
                (LONG far *)&DriverHandle);
            else
                ErrCde = DRV_DeviceOpen(
                SubDeviceList[gwSubDevice].dwDeviceNum,
                (LONG far *)&DriverHandle);

            if (ErrCde != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                return 0;
            }

            // Step 2: Get device features
            ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
            ptDevFeatures.size = sizeof(DEVFEATURES);
            if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            // Step 3: Allocate DMA buffer for DMA transfer
            ptAllocateDMABuffer.CyclicMode = gwCyclicMode;
            ptAllocateDMABuffer.RequestBufSize = gulConvNum * 2;
            ptAllocateDMABuffer.ActualBufSize = &gwActualBufSize;
            ptAllocateDMABuffer.buffer = &pBuffer;
            if ((ErrCde = DRV_AllocateDMABuffer(DriverHandle,
                (LPT_AllocateDMABuffer)&ptAllocateDMABuffer)) != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            // Step 4: Allocate memory for Voltage data or Raw data
            if((hUserBuf=(FLOAT far *)GlobalAlloc(GHND,
                sizeof(FLOAT) * gulConvNum)) == 0)
            {
                MessageBox(hWnd,"Not enough memory for buffer ",
                    "High Speed",MB_OK);
                DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            ptFAITransfer.DataBuffer = (FLOAT far  *)GlobalLock(hUserBuf);

            // Step 5: Enable event feature
            ptEnableEvent.EventType = ADS_EVT_INTERRUPT  |
                ADS_EVT_BUFCHANGE  |
                ADS_EVT_TERMINATED |
                ADS_EVT_OVERRUN;
            ptEnableEvent.Enabled = gwEvtFlag;
            ptEnableEvent.Count   = 512;

            if ((ErrCde = DRV_EnableEvent(DriverHandle,
                (LPT_EnableEvent)&ptEnableEvent)) != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                GlobalUnlock(hUserBuf);
                GlobalFree(hUserBuf);
                DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

//          for (i=0 ; i < gwNumChl ; i++)
            for (i=0 ; i < 16 ; i++)        // B980529: Modify by Charley
            {
                if (!gwGainList)
                {
                    gwGainCde[i] = DevFeatures.glGainList[gwGainCode].usGainCde;      // no gain list
                }
                else
                {
                    temp = gwGain[i];
                    gwGainCde[i] = DevFeatures.glGainList[temp].usGainCde;
                }
            }

            // Step 6: Start DMA transfer
            ptFAIDmaScanStart.TrigSrc    = gwExtTrig;
            ptFAIDmaScanStart.StartChan  = gwStartChl;
            ptFAIDmaScanStart.NumChans   = gwNumChl;
//          ptFAIDmaScanStart.GainList   = &gwGainCde[0];
            ptFAIDmaScanStart.GainList   = &gwGainCde[gwStartChl];    // B980529: Modify by Charley
            ptFAIDmaScanStart.SampleRate = gdwPacerRate;
            ptFAIDmaScanStart.count  = gulConvNum;
            ptFAIDmaScanStart.buffer = (USHORT far *)pBuffer;

            if ((ErrCde = DRV_FAIDmaScanStart(DriverHandle,
                (LPT_FAIDmaScanStart)&ptFAIDmaScanStart)) != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                GlobalUnlock(hUserBuf);
                GlobalFree(hUserBuf);
                DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            gwCyclicCount = 0;

            if (gwEvtFlag)
			{
                hThreadHandle = CreateThread(0, 0,
                (LPTHREAD_START_ROUTINE) UserThread,
                0, 0, (LPDWORD)&dwThreadID);

				bThreadloop = TRUE;
			}

            Sleep(0);

            EnableMenuItem(hMenu, IDM_RUN, MF_DISABLED | MF_BYCOMMAND);

            // Enable stop and status buttons
            EnableWindow(hwndStopButton, TRUE);
            EnableWindow(hwndStatusButton, TRUE);

            return 0;
            //end IDM_RUN

        case IDC_STOPBUTTON:
			{						
               BOOL bflag = TRUE;
               // Close thread
			   bThreadloop = FALSE;
               Sleep(100);

			   while(bflag)
			   {
			      if ( bThreadflag==FALSE )
				  {
                     EnableWindow(hwndStopButton, FALSE);
                     EnableWindow(hwndStatusButton, FALSE);
   			         bflag=FALSE;
				  }
			   }
               // Close thread
               // bThreadStatus = GetExitCodeThread(hThreadHandle, &dwExitCode);
               // if ((dwExitCode== (DWORD)STILL_ACTIVE) && (bThreadStatus))
               // {
               // TerminateThread(hThreadHandle, 0);
               // CloseHandle(hThreadHandle);
               // CloseHandle(hEvent);
               // Sleep(0);
               // }

               // Disable stop and status buttons
               // EnableWindow(hwndStopButton, FALSE);
               // EnableWindow(hwndStatusButton, FALSE);

               // Step 1: Stop A/D conversion for high speed
               if ((ErrCde = DRV_FAIStop(DriverHandle)) != 0)
			   {
                  DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                  MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                  GlobalUnlock(hUserBuf);
                  GlobalFree(hUserBuf);
                  DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                  DRV_DeviceClose((LONG far *)&DriverHandle);
                  return 0;
			   }

               // Step 2: Get real voltage of Buffer from driver memory buffer
               ptFAITransfer.ActiveBuf = 0;   // single buffer
               ptFAITransfer.DataType = gwDataType;
               ptFAITransfer.start    = 0;
               ptFAITransfer.count    = gulConvNum;
               ptFAITransfer.overrun  = &gwOverrun;

               if ((ErrCde = DRV_FAITransfer(DriverHandle,
                (LPT_FAITransfer)&ptFAITransfer)) != 0)
			   {
                  DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                  MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                  GlobalUnlock(hUserBuf);
                  GlobalFree(hUserBuf);
                  DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                  DRV_DeviceClose((LONG far *)&DriverHandle);
                  return 0;
			   }

               // Display data
               DialogBox (hInstance, MAKEINTRESOURCE(IDD_DATA),
                  hWnd, lpfnShowDataDlgProc) ;

               // Step 3: Free buffer
               GlobalUnlock(hUserBuf);
               GlobalFree(hUserBuf);
               DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);

               // Step 4: Close driver
               DRV_DeviceClose((LONG far *)&DriverHandle);
 
               EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);

               return 0;			
			}			

        case IDC_STATUSBUTTON:

            // Get DMA transfer status
            ptFAICheck.ActiveBuf = &gwActiveBuf;
            ptFAICheck.stopped   = &gwStopped;
            ptFAICheck.retrieved = &gulRetrieved;
            ptFAICheck.overrun   = &gwOverrun;
            ptFAICheck.HalfReady = &gwHalfReady;

            if ((ErrCde = DRV_FAICheck(DriverHandle,
                (LPT_FAICheck)&ptFAICheck)) != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                GlobalUnlock(hUserBuf);
                GlobalFree(hUserBuf);
                DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }
            else
            {   // Display the status of the A/D conversion
                sprintf(szDest, "Cyclic Counts: %d\n", gwCyclicCount);
                if (gwHalfReady == 0)
                    strcpy(szTemp, "Buffer : not  ready\n");
                else if (gwHalfReady == 1)
                    strcpy(szTemp, "Buffer : half ready\n");
                else if (gwHalfReady == 2)
                    strcpy(szTemp, "Buffer : full ready\n");
                else
                    strcpy(szTemp, "Buffer : unknow \n");
                strcat(szDest, szTemp);
                ratio = 100.0 * gulRetrieved / gulConvNum;
                szStr = fcvt((double)ratio, 0, &dec, &sign);
                strcat(szDest, "Finish:  ");
                strcat(szDest, szStr);
                strcat(szDest, "%...");
                MessageBox(hWnd,(LPCSTR)szDest, "Check Status", MB_OK);
            }

            return 0;
        }

    case WM_DESTROY:
		{
		   BOOL bflag = TRUE;
           
		   bThreadloop = FALSE;
           Sleep(100);

		   while(bflag)
		   {
		      if ( bThreadflag==FALSE )
			  {
                 EnableWindow(hwndStopButton, FALSE);
                 EnableWindow(hwndStatusButton, FALSE);
   		         bflag=FALSE;
			  }
		   }        
		   
           if ((ErrCde = DRV_FAIStop(DriverHandle)) != 0)
		   {
              DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
              MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
              GlobalUnlock(hUserBuf);
              GlobalFree(hUserBuf);
              DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
              DRV_DeviceClose((LONG far *)&DriverHandle);
              return 0;
		   }

           GlobalUnlock(hUserBuf);
           GlobalFree(hUserBuf);
           DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);

           DRV_DeviceClose((LONG far *)&DriverHandle); 
           EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);
           	
           PostQuitMessage(0);
		}
        break;

    default:
        return (DefWindowProc(hWnd, message, wParam, lParam));
    }

    return ((LONG)NULL);
}
//------------------------------------------------------------------
void UserThread()
{
    USHORT usEventType;
    LONG  ErrCde;
    DWORD TickValue;

    TickValue = GetTickCount();
    TickValue = TickValue + 500000;

//    while(TickValue > GetTickCount())
    while(bThreadloop)
    {
        // Check message
        ptCheckEvent.EventType = &usEventType;
        ptCheckEvent.Milliseconds = INFINITE;

		bThreadflag = TRUE;
        if ((ErrCde = DRV_CheckEvent(DriverHandle,
            (LPT_CheckEvent)&ptCheckEvent)) != 0)
        {
            MessageBox(hWnd,"Check Event Error !","Thread Message",MB_OK);
            return ;
        }

        // Process interrupt event
        if (usEventType & ADS_EVT_INTERRUPT)
            adInterruptEvent();

        // Process buffer change event
        if (usEventType & ADS_EVT_BUFCHANGE)
            adBufChangeEvent();

        // Process overrun event
        if (usEventType & ADS_EVT_OVERRUN)
            adOverrunEvent();

        // Process terminate event
        if (usEventType & ADS_EVT_TERMINATED)
        {
            adTerminateEvent();
            return ;
        }
		bThreadflag = FALSE;
    }

//    MessageBox(hWnd,"Never Get Any Event !","Thread Message",MB_OK);
}

//------------------------------------------------------------------
void adInterruptEvent()
{
    return;
}

//------------------------------------------------------------------
void adBufChangeEvent()
{
    ptFAICheck.ActiveBuf = &gwActiveBuf;
    ptFAICheck.stopped   = &gwStopped;
    ptFAICheck.retrieved = &gulRetrieved;
    ptFAICheck.overrun   = &gwOverrun;
    ptFAICheck.HalfReady = &gwHalfReady;

    if ((ErrCde = DRV_FAICheck(DriverHandle,
         (LPT_FAICheck)&ptFAICheck)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    if (gwHalfReady == 1)  // first ready
    {
        ptFAITransfer.ActiveBuf = 0;   // single buffer
        ptFAITransfer.DataType = gwDataType;
        ptFAITransfer.start    = 0;
        ptFAITransfer.count    = gulConvNum/2;
        ptFAITransfer.overrun  = &gwOverrun;

        if ((ErrCde = DRV_FAITransfer(DriverHandle,
            (LPT_FAITransfer)&ptFAITransfer)) != 0)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
            return ;
        }
    }
    else if (gwHalfReady == 2) // second half ready
    {
        ptFAITransfer.ActiveBuf = 0;   // single buffer
        ptFAITransfer.DataType = gwDataType;
        ptFAITransfer.start    = gulConvNum/2;
        ptFAITransfer.count    = gulConvNum/2;
        ptFAITransfer.overrun  = &gwOverrun;

        if ((ErrCde = DRV_FAITransfer(DriverHandle,
            (LPT_FAITransfer)&ptFAITransfer)) != 0)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
            return ;
        }

        gwCyclicCount ++;
    }

    return;
}
//------------------------------------------------------------------
void adOverrunEvent()
{
    // clear overrun
    if ((ErrCde = DRV_ClearOverrun(DriverHandle)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    return;
}
//------------------------------------------------------------------
void adTerminateEvent()
{
    // Disable stop and status buttons
    EnableWindow(hwndStopButton, FALSE);
    EnableWindow(hwndStatusButton, FALSE);

    // Stop A/D conversion
    if ((ErrCde = DRV_FAIStop(DriverHandle)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    // Get real voltage of buffer from driver
    ptFAITransfer.ActiveBuf = 0;   // single buffer
    ptFAITransfer.DataType = gwDataType;
    ptFAITransfer.start    = 0;
    ptFAITransfer.count    = gulConvNum;
    ptFAITransfer.overrun  = &gwOverrun;

    if ((ErrCde = DRV_FAITransfer(DriverHandle,
        (LPT_FAITransfer)&ptFAITransfer)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    // Display data
    DialogBox (hInstance, MAKEINTRESOURCE(IDD_DATA),
        hWnd, lpfnShowDataDlgProc) ;

    // Free buffer
    GlobalUnlock(hUserBuf);
    GlobalFree(hUserBuf);
    DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);

    // Close driver
    DRV_DeviceClose((LONG far *)&DriverHandle);

    EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);

    return;
}

void adstopai()
{
   BOOL bflag = TRUE;
      
   // Close thread
   bThreadloop = FALSE;
   Sleep(100);

   while(bflag)
   {
	   if(bThreadflag==FALSE)
	   {
		   EnableWindow(hwndStopButton, FALSE);
		   EnableWindow(hwndStatusButton, FALSE);
		   bflag=FALSE;
	   }
	}
    
   // Step 1: Stop A/D conversion for high speed
   if ((ErrCde = DRV_FAIStop(DriverHandle)) != 0)
   {
	   DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
	   MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
	   GlobalUnlock(hUserBuf);
	   GlobalFree(hUserBuf);
	   DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
	   DRV_DeviceClose((LONG far *)&DriverHandle);
	   return ;
   }

   // Step 2: Get real voltage of Buffer from driver memory buffer
   ptFAITransfer.ActiveBuf = 0;   // single buffer
   ptFAITransfer.DataType = gwDataType;
   ptFAITransfer.start    = 0;
   ptFAITransfer.count    = gulConvNum;
   ptFAITransfer.overrun  = &gwOverrun;

   if ((ErrCde = DRV_FAITransfer(DriverHandle,
	   (LPT_FAITransfer)&ptFAITransfer)) != 0)
   {
	   DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
	   MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
	   GlobalUnlock(hUserBuf);
	   GlobalFree(hUserBuf);
	   DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);
	   DRV_DeviceClose((LONG far *)&DriverHandle);
	   return ;
   }

   // Display data
   DialogBox (hInstance, MAKEINTRESOURCE(IDD_DATA),
              hWnd, lpfnShowDataDlgProc) ;

   // Step 3: Free buffer
   GlobalUnlock(hUserBuf);
   GlobalFree(hUserBuf);
   DRV_FreeDMABuffer(DriverHandle, (LPARAM)&pBuffer);

   // Step 4: Close driver
   DRV_DeviceClose((LONG far *)&DriverHandle);

   EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);

   return ;
}



