/*
 ***********************************************************************
 *  Program     : MAIEXP.C                                             *
 *  Description : Demo program for multiple channel AI with expansion  *
 *  Revision    : 1.00                                                 *
 *  Date        : 9/5/1996                     Advantech Co., Ltd.     *
 ***********************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance() - Saves instance handle and creates main window
 *
 *   MainWndProc() - Process main window messages
 *
 *   ConfigDlgProc() - Process the dialog box messages for configuration
 */


#include <windows.h>              /* required for all Windows applications */
#include <stdlib.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "..\..\..\include\driver.h"
#include "..\..\..\include\os.h"
#include "resource.h"

// ------------------------------------------------------------------------
// CONSTANT DEFINITION
// ------------------------------------------------------------------------
#define     MAX_DEVICES     100


// ------------------------------------------------------------------------
// GLOBAL VARIABLES
// ------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hMainWnd;                   // main window handle


static      DEVFEATURES     DevFeatures;    // structure for device features
static      DEVCONFIG_AI    DevCfg;         // structure for DEVCONFIG_AI table
static      PT_MAIConfig        ptMAIConfig;        // structure for MAIConfig table
static      PT_MAIVoltageIn     ptMAIVoltageIn;     // structure for MAIVoltageIn table
static      PT_MAIVoltageInExp  ptMAIVoltageInExp;  // structure for MAIVoltageInExp table
static      PT_DeviceGetFeatures  ptDevFeatures;    // structure for DeviceGetFeatures
static      PT_AIGetConfig  ptAIGetConfig;          // structure for AIGetConfig table

char        szErrMsg[80];               // Use for MESSAGEBOX function
LRESULT     ErrCde;                     // Return error code

//
// used for dynamic allocation memory for the installed devices
//
// static  HGLOBAL     hDeviceList,hSubDeviceList;
// static  LPDEVLIST   DeviceList,SubDeviceList;
//

//
// used for fixed memory for the installed devices
//
DEVLIST     DeviceList[MAX_DEVICES];
DEVLIST     SubDeviceList[MAX_DEVICES];

LONG        DriverHandle = (LONG)NULL;          // driver handle
BOOL        bRun = FALSE;                       // flag for running
USHORT      gwDevice = 0, gwSubDevice = 0;      // Device index
SHORT       gnNumOfDevices, gnNumOfSubdevices;  // number of installed devices
USHORT      gwScanTime = 1000;                  // scan time
DWORD       gdwStartTime;                       // start time
DWORD       gdwElapseTime = 0;                  // elapse time
USHORT      gwOverrunCount = 0;                 // overrun count
FARPROC     lpfnConfigDlgProc;                  // config. dialog procedure
FARPROC     lpfnScanDlgProc;                    // scan dialog procedure
char szBuffer[40];
USHORT      usStartChan = 0;                    // start channel
USHORT      usNumChan   = 1;                      // Number of channel
USHORT      usCurrentChan = 0;                  // Current channel number
USHORT      usDasChan[32];					    // das Channel array
USHORT      usGainIndex[32];                    // varied input range index
USHORT      usGainCode[32];                     // varied input range array
USHORT      usExpChan[32];                      // varied expansion chan
USHORT      usExpChanTemp[32];                  // varied for PCLD-788 use
USHORT      usBoardID [16];                     // varied board ID for 788

USHORT      usExpFlag = 0;					    // expansion board flag
USHORT      usMaxChannel = 0;                   // max channel
CHAR        szExpName[15];                      // expansion name
CHAR        szBuf[64];                          // temp buffer

// ------------------------------------------------------------------------
// FUNCTION DECLARATION
// ------------------------------------------------------------------------
BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, int nCmdShow);
long FTYPE MainWndProc(HWND, UINT, WPARAM, LPARAM);
int  FTYPE WinMain(HANDLE hInstance, HANDLE hPrevInstance,
     LPSTR lpCmdLine, int nCmdShow);


PRIVATE void AllWindows(HWND hDlg);
PRIVATE void SetChannel(HWND hDlg);
PRIVATE void SetCurrentChan(HWND hDlg, BOOL bflag);
PRIVATE void SetGain(HWND hDlg, BOOL bflag);
PRIVATE void SetExpChan(HWND hDlg, BOOL bflag);
PRIVATE void SetBoardID(HWND hDlg, BOOL bflag);

/***************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
****************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                          /* current instance             */
HANDLE hPrevInstance;                      /* previous instance            */
LPSTR lpCmdLine;                           /* command line                 */
int nCmdShow;                              /* show-window type (open/icon) */
{
    MSG msg;                               /* message                      */

    if (!hPrevInstance)                /* Other instances of app running?  */
      if (!InitApplication(hInstance)) /* Initialize shared things         */
          return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,      /* message structure                      */
        (HWND)NULL,              /* handle of window receiving the message */
        (UINT)NULL,              /* lowest message to examine              */
        (UINT)NULL))             /* highest message to examine             */
    {
        TranslateMessage(&msg);  /* Translates virtual key codes           */
        DispatchMessage(&msg);   /* Dispatches message to window           */
    }
    return (msg.wParam);         /* Returns the value from PostQuitMessage */
}

/****************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                            /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW; /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;  /* window process procedure    */
    wc.cbClsExtra = 0;                  /* No per-class extra data.        */
    wc.cbWndExtra = 0;                  /* No per-window extra data.       */
    wc.hInstance = hInstance;           /* Application that owns the class.*/
    wc.hIcon = LoadIcon(hInstance, "IDI_ICON1");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";   /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";  /* Name used in call to CreateWindow. */

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));

}

/***************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
****************************************************************************/

BOOL InitInstance
(
    HANDLE          hInstance,          // Current instance identifier.
    int             nCmdShow            // Param for first ShowWindow() call.
)
{
    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hMainWnd = CreateWindow(
        "MyClass",                 /* See RegisterClass() call.           */
        "Advantech Driver Demo : Software Data Transfer" , /* Window title bar  */
        WS_OVERLAPPEDWINDOW,        /* Window style.                       */
        CW_USEDEFAULT,              /* Default horizontal position.        */
        CW_USEDEFAULT,              /* Default vertical position.          */
        CW_USEDEFAULT,              /* Default width.                      */
        CW_USEDEFAULT,              /* Default height.                     */
        NULL,                       /* Overlapped windows have no parent.  */
        NULL,                       /* Use the window class menu.          */
        hInstance,                  /* This instance owns this window.     */
        NULL                        /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hMainWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hMainWnd, nCmdShow);   // Show the window
    UpdateWindow(hMainWnd);           // Sends WM_PAINT message
    return (TRUE);                // Returns the value from PostQuitMessage
}

/***************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
****************************************************************************/

BOOL FTYPE ConfigDlgProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    USHORT      i;
    DWORD       dwIndex;
    int         nOutEntries;
    USHORT      usStopChan;

    usStopChan = usStartChan + usNumChan - 1;

    switch (message)
    {
    case WM_INITDIALOG :

        // --------------------------------
        // Initialize Device List Combobox
        // --------------------------------

        // get number of the installed devices

        if ((ErrCde = DRV_DeviceGetNumOfList((SHORT far *)&gnNumOfDevices)) !=
            SUCCESS)
        {
             DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
             MessageBox(hMainWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
             return TRUE;
        }


        // ----------------------------------------------------------------
        // used for dynamic allocation memory for these installed devices
        //
        // if ((hDeviceList = GlobalAlloc(GHND, gnNumOfDevices *
        //      sizeof(DEVLIST))) == NULL)
        // {
        //    MessageBox(hMainWnd, (LPCSTR)"Not Enough Memory",
        //        "Memory Allocation", MB_OK);
        //    return TRUE;
        // }
        // DeviceList = (LPDEVLIST)GlobalLock(hDeviceList);
        // ----------------------------------------------------------------

        // initialize the number of channel       
        ultoa(usNumChan, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_NUMCHAN, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));


        if (gnNumOfDevices > MAX_DEVICES)
            gnNumOfDevices = MAX_DEVICES;

        // retrieve the information of all installed devices

        if ((ErrCde = DRV_DeviceGetList((DEVLIST far *)&DeviceList[0],
            (SHORT)gnNumOfDevices, (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return TRUE;
        }

        // initialize the Device List Combobox with the retrieved information

        for (i = 0; i < (USHORT)gnNumOfDevices; i ++)
            SendDlgItemMessage(hDlg, IDC_DEVICE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)DeviceList[i].szDeviceName));

        SendDlgItemMessage(hDlg, IDC_DEVICE, CB_SETCURSEL,(WPARAM)gwDevice,
                        (LPARAM)0);


        // -----------------------------------------------------------
        // Initialize Module List Combobox for COM port or CAN devices
        // -----------------------------------------------------------

        // check if there is any device attached on this COM port or CAN

        gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
        if (gnNumOfSubdevices > MAX_DEVICES)
            gnNumOfSubdevices = MAX_DEVICES;


        // --------------------------------------------------------------
        // used for dynamic allocation memory for these installed devices
        //
        // if ((hSubDeviceList = GlobalAlloc(GHND, gnNumOfSubdevices *
        //    sizeof(DEVLIST))) == NULL)
        // {
        //    MessageBox(hMainWnd, (LPCSTR)"Not Enough Memory",
        //        "Memory Allocation", MB_OK);
        //    return TRUE;
        // }
        //
        // SubDeviceList = (LPDEVLIST)GlobalLock(hSubDeviceList);
        // --------------------------------------------------------------

        // retrieve the information of all installed devices
        if (gnNumOfSubdevices != 0)
        {
            if ((ErrCde = DRV_DeviceGetSubList(
                (DWORD)DeviceList[gwDevice].dwDeviceNum,
                (DEVLIST far *)&SubDeviceList[0],
                (SHORT)gnNumOfSubdevices,
                (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                return TRUE;
            }


            // initialize the Module List Combobox with the retrieved
            // information

            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));

            for (i = 0; i < (USHORT)gnNumOfSubdevices; i++)
                SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

            SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
					(WPARAM)gwSubDevice, (LPARAM)0);
        }
        else
        {
            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));
        }

        // ------------------------------------------------------------
        // Initialize Input Range List Combobox, it needs to get device
        // features for gain list
        // ------------------------------------------------------------

        // first : Open Device

        if (gnNumOfSubdevices == 0)
        {
            ErrCde = DRV_DeviceOpen(DeviceList[gwDevice].dwDeviceNum,
               (LONG far *)&DriverHandle);
        }
        else
        {
            ErrCde = DRV_DeviceOpen(
                SubDeviceList[gwSubDevice].dwDeviceNum,
                (LONG far *)&DriverHandle);
        }

        if (ErrCde != SUCCESS)
        {
            strcpy(szErrMsg,"Device open error !");
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return 0;
        }

        // second: get device features

        ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
        ptDevFeatures.size = sizeof(DEVFEATURES);
        if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
            (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
            return 0;
        }

        // Have AI function support ?
        if ((DevFeatures.usMaxAIDiffChl == 0) &&
            (DevFeatures.usMaxAISiglChl == 0))
            AllWindows(hDlg);
        else
        {
            ptAIGetConfig.buffer = (LPDEVCONFIG_AI)&DevCfg;
            ptAIGetConfig.size = sizeof(DEVCONFIG_AI);

            // call AIGetConfig
            if ((ErrCde = DRV_AIGetConfig(DriverHandle,
                (LPT_AIGetConfig)&ptAIGetConfig)) != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

                usMaxChannel = DevFeatures.usMaxAISiglChl;

            if (usCurrentChan > usStopChan)
                usCurrentChan = usStopChan;
            else if (usCurrentChan < usStartChan)
                usCurrentChan = usStartChan;

            SetChannel(hDlg);
            SetCurrentChan(hDlg, TRUE);

            // if the selected channel is associated with an expansion board,
            // fill in the Exp. channel listbox with the available expansion board
            // channel.
            if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[usCurrentChan&0xf].dwBoardID
                && usCurrentChan != DevCfg.usCjcChannel &&
                DevCfg.Daughter[usCurrentChan&0xf].dwBoardID != (DWORD)BD_PCLD8115)
                SetExpChan(hDlg, TRUE);
			   else
                SetExpChan(hDlg, FALSE);

            if (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == (DWORD)BD_PCLD788)
                SetBoardID(hDlg,TRUE);
            else
                SetBoardID(hDlg,FALSE);

            // third: initialize Input Range List Combobox with device features
            if ((DevFeatures.usNumGain != 0) &&
                (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == 0))// &&
                //(DevCfg.usCjcChannel != usCurrentChan)
                SetGain(hDlg, TRUE);
            else
                SetGain(hDlg, FALSE);
        }


        // fourth: close device

        DRV_DeviceClose((LONG far *)&DriverHandle);

        return TRUE;

    case WM_COMMAND :
         switch (LOWORD(wParam))
         {
            case IDOK :

              // get device selection
              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwDevice = (WORD)dwIndex;

              // get sub-device selection
              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_MODULE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwSubDevice = (WORD)dwIndex;


              EndDialog(hDlg, 0);
              return TRUE;


            case IDCANCEL :
                 EndDialog(hDlg, 0);
                 return TRUE;

            case IDC_STARTCHAN :

                // When device selection is changed, it needs to
                // re-initialize sub-device combobox and input range
                // combobox.
                if (HIWORD(wParam) == CBN_SELCHANGE)
                {
                  if ((dwIndex = SendDlgItemMessage(hDlg, IDC_STARTCHAN,
                               CB_GETCURSEL, 0, 0)) != CB_ERR)
                       usStartChan = (WORD)dwIndex;

                  if (usStopChan < usStartChan)
                  {
                       usStopChan = usStartChan;
                       SendDlgItemMessage(hDlg, IDC_STOPCHAN, CB_SETCURSEL,
                          usStopChan,0L);
                  }

                  if (usCurrentChan > usStopChan)
                      usCurrentChan = usStopChan;
                  else// if (usCurrentChan < usStartChan)
                      usCurrentChan = usStartChan;
                  // setting usCurrentChan
                  SetCurrentChan(hDlg, TRUE);

                  // if the selected channel is associated with an expansion board,
                  // fill in the Exp. channel listbox with the available expansion board
                  // channel.
                  if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[usCurrentChan&0xf].dwBoardID
                      && usCurrentChan != DevCfg.usCjcChannel &&
                      DevCfg.Daughter[usCurrentChan&0xf].dwBoardID != (DWORD)BD_PCLD8115)
                      SetExpChan(hDlg, TRUE);
                  else
                      SetExpChan(hDlg, FALSE);

                  if (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == (DWORD)BD_PCLD788)
                      SetBoardID(hDlg, TRUE);
                  else
                      SetBoardID(hDlg,FALSE);

                  // third: initialize Input Range List Combobox with device features
                  if ((DevFeatures.usNumGain != 0) &&
                      (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == 0))// &&
                      //(DevCfg.usCjcChannel != usCurrentChan)
                      SetGain(hDlg, TRUE);
                  else
                      SetGain(hDlg,FALSE);
                }

              return TRUE;
            case IDC_NUMCHAN:
               if (HIWORD(wParam) == EN_KILLFOCUS)
               {
                   if(SendDlgItemMessage(hDlg, IDC_NUMCHAN,
                       EM_GETMODIFY, 0, 0))
                   {
                       SendDlgItemMessage(hDlg, IDC_NUMCHAN,
                           WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                       usNumChan = (ULONG)atol(szBuffer);
                       usStopChan = usStartChan + usNumChan - 1;
                       SendDlgItemMessage(hDlg, IDC_NUMCHAN,
                           EM_SETMODIFY, FALSE, 0) ;
                   }

                   if (usCurrentChan > usStopChan)
                      usCurrentChan = usStopChan;
                  else if (usCurrentChan < usStartChan)
                      usCurrentChan = usStartChan;

                  // setting usCurrentChan
                  SetCurrentChan(hDlg, TRUE);

                  // if the selected channel is associated with an expansion board,
                  // fill in the Exp. channel listbox with the available expansion board
                  // channel.
                  if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[usCurrentChan&0xf].dwBoardID
                      && usCurrentChan != DevCfg.usCjcChannel &&
                      DevCfg.Daughter[usCurrentChan&0xf].dwBoardID != (DWORD)BD_PCLD8115)
                      SetExpChan(hDlg, TRUE);
                  else
                      SetExpChan(hDlg, FALSE);

                  if (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == (DWORD)BD_PCLD788)
                      SetBoardID(hDlg, TRUE);
                  else
                      SetBoardID(hDlg,FALSE);

                  // third: initialize Input Range List Combobox with device features
                  if ((DevFeatures.usNumGain != 0) &&
                      (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == 0))// &&
                      //(DevCfg.usCjcChannel != usCurrentChan)
                      SetGain(hDlg, TRUE);
                  else
                      SetGain(hDlg, FALSE);
               }
               
            case IDC_STOPCHAN :

                // When device selection is changed, it needs to
                // re-initialize sub-device combobox and input range
                // combobox.
                if (HIWORD(wParam) == CBN_SELCHANGE)
                {
                  if ((dwIndex = SendDlgItemMessage(hDlg, IDC_STOPCHAN,
                               CB_GETCURSEL, 0, 0)) != CB_ERR)
                       usStopChan = (WORD)dwIndex;

                  if (usStopChan < usStartChan)
                  {
                       usStopChan = usStartChan;
                       SendDlgItemMessage(hDlg, IDC_STOPCHAN, CB_SETCURSEL,
                          usStopChan,0L);
                  }

                  if (usCurrentChan > usStopChan)
                      usCurrentChan = usStopChan;
                  else if (usCurrentChan < usStartChan)
                      usCurrentChan = usStartChan;

                  // setting usCurrentChan
                  SetCurrentChan(hDlg, TRUE);

                  // if the selected channel is associated with an expansion board,
                  // fill in the Exp. channel listbox with the available expansion board
                  // channel.
                  if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[usCurrentChan&0xf].dwBoardID
                      && usCurrentChan != DevCfg.usCjcChannel &&
                      DevCfg.Daughter[usCurrentChan&0xf].dwBoardID != (DWORD)BD_PCLD8115)
                      SetExpChan(hDlg, TRUE);
                  else
                      SetExpChan(hDlg, FALSE);

                  if (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == (DWORD)BD_PCLD788)
                      SetBoardID(hDlg, TRUE);
                  else
                      SetBoardID(hDlg,FALSE);

                  // third: initialize Input Range List Combobox with device features
                  if ((DevFeatures.usNumGain != 0) &&
                      (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == 0))// &&
                      //(DevCfg.usCjcChannel != usCurrentChan)
                      SetGain(hDlg, TRUE);
                  else
                      SetGain(hDlg, FALSE);
                }

              return TRUE;

            case IDC_INPRANGE :

              if (HIWORD(wParam) == CBN_SELCHANGE)
              {
                 if ((dwIndex = SendDlgItemMessage(hDlg, IDC_INPRANGE,
                    CB_GETCURSEL, 0, 0)) != CB_ERR)
                       usGainIndex[usCurrentChan] = (WORD)dwIndex;
              }
              return TRUE;

            case IDC_EXPCHAN :

              if (HIWORD(wParam) == CBN_SELCHANGE)
              {
                 if ((dwIndex = SendDlgItemMessage(hDlg, IDC_EXPCHAN,
                    CB_GETCURSEL, 0, 0)) != CB_ERR)
                       usExpChan[usCurrentChan] = (WORD)dwIndex;
              }
              return TRUE;

            case IDC_BOARDID :

              if (HIWORD(wParam) == CBN_SELCHANGE)
              {
                 if ((dwIndex = SendDlgItemMessage(hDlg, IDC_BOARDID,
                    CB_GETCURSEL, 0, 0)) != CB_ERR)
                       usBoardID[usCurrentChan] = (WORD)dwIndex;
              }
              return TRUE;

            case IDC_CURRENTCHAN :

              if (HIWORD(wParam) == CBN_SELCHANGE)
              {
                 if ((dwIndex = SendDlgItemMessage(hDlg, IDC_CURRENTCHAN,
                    CB_GETCURSEL, 0, 0)) != CB_ERR)
                       usCurrentChan = (WORD)dwIndex+ usStartChan;
                 else
                       return TRUE;

                 // setting usCurrentChan
                 SetCurrentChan(hDlg, TRUE);

                 // if the selected channel is associated with an expansion board,
                 // fill in the Exp. channel listbox with the available expansion board
                 // channel.
                 if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[usCurrentChan&0xf].dwBoardID
                     && usCurrentChan != DevCfg.usCjcChannel &&
                     DevCfg.Daughter[usCurrentChan&0xf].dwBoardID != (DWORD)BD_PCLD8115)
                     SetExpChan(hDlg, TRUE);
                 else
                     SetExpChan(hDlg, FALSE);

                 if (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == (DWORD)BD_PCLD788)
                     SetBoardID(hDlg, TRUE);
                 else
                     SetBoardID(hDlg,FALSE);

                 // third: initialize Input Range List Combobox with device features
                 if ((DevFeatures.usNumGain != 0) &&
                     (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == 0))// &&
                     // (DevCfg.usCjcChannel != usCurrentChan)
                     SetGain(hDlg, TRUE);
                 else
                     SetGain(hDlg, FALSE);
              }
              return TRUE;

            case IDC_DEVICE :

                //
                // When device selection is changed, it needs to
                // re-initialize sub-device combobox and input range
                // combobox.
                //

                if (HIWORD(wParam) == CBN_SELCHANGE)
                {
                    if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                        CB_GETCURSEL, 0, 0)) != CB_ERR)
                         gwDevice = (WORD)dwIndex;
                    else
                        return TRUE;


                    // ------------------------------------------------------
                    // Initialize Module Combobox for COM port or CAN devices
                    // ------------------------------------------------------

                    // check any device attached on this COM port or CAN

                    gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
                    if (gnNumOfSubdevices > MAX_DEVICES)
                        gnNumOfSubdevices = MAX_DEVICES;


                    // retrieve the information of all installed devices

                    if (gnNumOfSubdevices != 0)
                    {
                        if ((ErrCde = DRV_DeviceGetSubList(
                            (DWORD)DeviceList[gwDevice].dwDeviceNum,
                            (DEVLIST far *)&SubDeviceList[0],
                            (SHORT)gnNumOfSubdevices,
                            (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
                        {
                            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                            MessageBox(hMainWnd, (LPCSTR)szErrMsg,
                                "Driver Message", MB_OK);
                            return TRUE;
                        }


                        // initialize the Module List Combobox with the
                        // retrieved information

                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));

                        for (i = 0; i < (USHORT)gnNumOfSubdevices; i++)
                            SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                                (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

                        gwSubDevice = 0;

                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
							(WPARAM)gwSubDevice, (LPARAM)0);
                    }
                    else
                    {
                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));
                    }


                    // -------------------------------------------------
                    // Initialize Input Range List Combobox, it needs to
                    // get device features for gain list
                    // -------------------------------------------------

                    // first : Open Device

                    // for non COM port or CAN device
                    if (gnNumOfSubdevices == 0)
                    {
                        ErrCde = DRV_DeviceOpen(
                            DeviceList[gwDevice].dwDeviceNum,
                            (LONG far *)&DriverHandle);
                    }

                    // for COM port or CAN device
                    else
                    {
                        ErrCde = DRV_DeviceOpen(
                            SubDeviceList[gwSubDevice].dwDeviceNum,
                            (LONG far *)&DriverHandle);
                    }

                    if (ErrCde != SUCCESS)
                    {
                        strcpy(szErrMsg,"Device open error !");
                        MessageBox(hMainWnd, (LPCSTR)szErrMsg,
                            "Driver Message", MB_OK);
                        return 0;
                    }

                    // second: get device features

                    ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
                    ptDevFeatures.size = sizeof(DEVFEATURES);
                    if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                        (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hMainWnd, (LPCSTR)szErrMsg, "Driver Message",
                            MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    // Have AI function support ?
                    if ((DevFeatures.usMaxAIDiffChl == 0) &&
                        (DevFeatures.usMaxAISiglChl == 0))
                        AllWindows(hDlg);
                    else
                    {
                        ptAIGetConfig.buffer = (LPDEVCONFIG_AI)&DevCfg;
                        ptAIGetConfig.size = sizeof(DEVCONFIG_AI);

                        // call AIGetConfig
                        if ((ErrCde = DRV_AIGetConfig(DriverHandle,
                            (LPT_AIGetConfig)&ptAIGetConfig)) != SUCCESS)
                        {
                            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                            DRV_DeviceClose((LONG far *)&DriverHandle);
                            return 0;
                        }

                        usMaxChannel = DevFeatures.usMaxAISiglChl;

                        if (usCurrentChan > usStopChan)
                            usCurrentChan = usStopChan;
                        else if (usCurrentChan < usStartChan)
                            usCurrentChan = usStartChan;

                        SetChannel(hDlg);
                        SetCurrentChan(hDlg, TRUE);

                        // if the selected channel is associated with an expansion board,
                        // fill in the Exp. channel listbox with the available expansion board
                        // channel.
                        if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[usCurrentChan&0xf].dwBoardID
                            && (int)usCurrentChan != (int)DevCfg.usCjcChannel &&
                            DevCfg.Daughter[usCurrentChan&0xf].dwBoardID != (DWORD)BD_PCLD8115)
                            SetExpChan(hDlg, TRUE);
                        else
                            SetExpChan(hDlg, FALSE);

                        if (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == (DWORD)BD_PCLD788)
                            SetBoardID(hDlg,TRUE);
                        else
                            SetBoardID(hDlg,FALSE);

                        // third: initialize Input Range List Combobox with device features
                        if ((DevFeatures.usNumGain != 0) &&
                            (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == 0))// &&
                            //(DevCfg.usCjcChannel != usCurrentChan)
                            SetGain(hDlg, TRUE);
                        else
                            SetGain(hDlg, FALSE);

                    }

                    // fourth: close device
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                }

                return TRUE;
			case IDC_MODULE:
				{
                    if (HIWORD(wParam) == CBN_SELCHANGE)
					{
                        if ((dwIndex = SendDlgItemMessage(hDlg, IDC_MODULE,
                            CB_GETCURSEL, 0, 0)) != CB_ERR)
                            gwSubDevice = (WORD)dwIndex;
                        else
                            return TRUE;
                    // for non COM port or CAN device
                    if (gnNumOfSubdevices == 0)
                    {
                        ErrCde = DRV_DeviceOpen(
                            DeviceList[gwDevice].dwDeviceNum,
                            (LONG far *)&DriverHandle);
                    }

                    // for COM port or CAN device
                    else
                    {
                        ErrCde = DRV_DeviceOpen(
                            SubDeviceList[gwSubDevice].dwDeviceNum,
                            (LONG far *)&DriverHandle);
                    }

                    if (ErrCde != SUCCESS)
                    {
                        strcpy(szErrMsg,"Device open error !");
                        MessageBox(hMainWnd, (LPCSTR)szErrMsg,
                            "Driver Message", MB_OK);
                        return 0;
                    }

                    // second: get device features

                    ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
                    ptDevFeatures.size = sizeof(DEVFEATURES);
                    if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                        (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hMainWnd, (LPCSTR)szErrMsg, "Driver Message",
                            MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    // Have AI function support ?
                    if ((DevFeatures.usMaxAIDiffChl == 0) &&
                        (DevFeatures.usMaxAISiglChl == 0))
                        AllWindows(hDlg);
                    else
                    {
                        ptAIGetConfig.buffer = (LPDEVCONFIG_AI)&DevCfg;
                        ptAIGetConfig.size = sizeof(DEVCONFIG_AI);

                        // call AIGetConfig
                        if ((ErrCde = DRV_AIGetConfig(DriverHandle,
                            (LPT_AIGetConfig)&ptAIGetConfig)) != SUCCESS)
                        {
                            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                            DRV_DeviceClose((LONG far *)&DriverHandle);
                            return 0;
                        }

                        usMaxChannel = DevFeatures.usMaxAISiglChl;

                        if (usCurrentChan > usStopChan)
                            usCurrentChan = usStopChan;
                        else if (usCurrentChan < usStartChan)
                            usCurrentChan = usStartChan;

                        SetChannel(hDlg);
                        SetCurrentChan(hDlg, TRUE);

                        // if the selected channel is associated with an expansion board,
                        // fill in the Exp. channel listbox with the available expansion board
                        // channel.
                        if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[usCurrentChan&0xf].dwBoardID
                            && (int)usCurrentChan != (int)DevCfg.usCjcChannel &&
                            DevCfg.Daughter[usCurrentChan&0xf].dwBoardID != (DWORD)BD_PCLD8115)
                            SetExpChan(hDlg, TRUE);
                        else
                            SetExpChan(hDlg, FALSE);

                        if (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == (DWORD)BD_PCLD788)
                            SetBoardID(hDlg,TRUE);
                        else
                            SetBoardID(hDlg,FALSE);

                        // third: initialize Input Range List Combobox with device features
                        if ((DevFeatures.usNumGain != 0) &&
                            (DevCfg.Daughter[usCurrentChan&0xf].dwBoardID == 0))// &&
                            //(DevCfg.usCjcChannel != usCurrentChan)
                            SetGain(hDlg, TRUE);
                        else
                            SetGain(hDlg, FALSE);

                    }

                    // fourth: close device
                    DRV_DeviceClose((LONG far *)&DriverHandle);
				    return TRUE;
					}
				}
         }
         break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: ScanDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ScanDlgProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    char        szBuffer[40];

    switch (message)
    {
    case WM_INITDIALOG :

        // ---------------------------------------------------
        // Initialize Scan Time Edit, Maximum accuracy: 40 ms
        // ---------------------------------------------------

        itoa(gwScanTime, szBuffer, 10);
        SendDlgItemMessage(hDlg, IDC_SCAN, EM_REPLACESEL, 0,
                        (LPARAM)((LPSTR)szBuffer));

        return TRUE;

    case WM_COMMAND :

         switch (LOWORD(wParam))
         {
            case IDOK :

                //
                // get scan time
                //

                if (SendDlgItemMessage(hDlg, IDC_SCAN,
                           EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_SCAN,
                            WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwScanTime = atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_SCAN,
                              EM_SETMODIFY, FALSE, 0) ;
                }

            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}


/***************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
****************************************************************************/

long FTYPE MainWndProc
(
    HWND        hWnd,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    USHORT          i;
    static  HANDLE  hInstance ;
	HMENU           hMenu;
    HDC             hdc;
    char            szBuffer[120];
    RECT            rect;
    float           fVoltage[32];
    DWORD           dwCurrentTime;
    DWORD           dwErrorTime;
    BOOL            bTimerOverrun;

    USHORT          usStopChan;
    usStopChan = usStartChan + usNumChan - 1;

    switch (message)
    {
        case WM_CREATE:
            // initialize value
            usStartChan   = 0;
            usStopChan    = 0;
            usCurrentChan = 0;
            for (i = 0 ; i < 32 ; i++)     // leon.jin mod to 32
            {
                usGainIndex[i]   = 0;
                usExpChan[i]     = 0;
                usExpChanTemp[i] = 0;
                usBoardID[i]     = 0;
            }

            //
            // initialize Stop Menu
            //

            hMenu = GetMenu(hWnd);
            EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);

            hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

            lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

            lpfnScanDlgProc = MakeProcInstance (ScanDlgProc, hInstance) ;

            return 0 ;

        case WM_COMMAND:     /* message: from application menu */

            hMenu = GetMenu(hWnd);

            switch (LOWORD(wParam))
            {
                case IDM_SETTING :
                    DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                         hWnd, lpfnConfigDlgProc) ;
                    return 0;

                case IDM_SCAN :
                    DialogBox (hInstance, MAKEINTRESOURCE(IDD_SCAN),
                         hWnd, lpfnScanDlgProc) ;
                    return 0;

                case IDM_STOP :

                    //
                    // kill timer and close driver
                    //

                    KillTimer(hWnd, 1);
                    DRV_DeviceClose((LONG far *)&DriverHandle);

                    bRun = FALSE;

                    //
                    // reset menu item status
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_ENABLED);

                    return 0;

                case IDM_START :

                    //
                    // Open Device
                    //

                    if (gnNumOfSubdevices == 0)
                        ErrCde = DRV_DeviceOpen(
                            DeviceList[gwDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);
                    else
                        ErrCde = DRV_DeviceOpen(
                            SubDeviceList[gwSubDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);

                    if (ErrCde != SUCCESS)
                    {
                        strcpy(szErrMsg,"Device open error !");
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                        return 0;
                    }

                    //
                    // get gain code
                    //

                    ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
                    ptDevFeatures.size = sizeof(DEVFEATURES);
                    if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                        (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    //
                    // configures the gain for the specifed analog input channel
                    //

                    // check start and stop channel if expansion board ?
                    usExpFlag = 0;
                    for (i = usStartChan ; i <= usStopChan ; i++)
                    {
                        if (DevCfg.Daughter[i&0xf].dwBoardID != 0)
                            usExpFlag = 1;
                    }

                    if(usExpFlag == 0)
                    {
                       ptMAIConfig.NumChan   = usStopChan - usStartChan + 1;
                       ptMAIConfig.StartChan = usStartChan;
                       for (i=0 ; i< ptMAIConfig.NumChan ; i++)
                         usGainCode[i+usStartChan] =
                            DevFeatures.glGainList[usGainIndex[i+usStartChan]].usGainCde;
                       ptMAIConfig.GainArray = (USHORT far *) &usGainCode[usStartChan];

                       if ((ErrCde = DRV_MAIConfig(DriverHandle,
                            (LPT_MAIConfig)&ptMAIConfig)) != 0)
                       {
                           DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                           MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                           DRV_DeviceClose((LONG far *)&DriverHandle);
                           return 0;
                       }
                    }
                    //
                    // enable Stop Menu item and disable the others
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_ENABLED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_GRAYED);

                    //
                    // Set the new timer type and start it up.
                    //

                    if (SetTimer(hWnd, 1, gwScanTime, NULL) == (UINT)NULL)
                    {
                        MessageBox(hWnd, "Couldn't create timer.", "Notice!",
                            MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    bRun = TRUE;

                    gdwStartTime = GetTickCount();
                    gwOverrunCount = 0;

                    return 0;
            }
            return 0;

        case WM_TIMER:

            //
            // Timer set to dispatch a WM_TIMER message has gone off.
            //

            //
            // check if timer is overrun
            //

            dwCurrentTime = GetTickCount();

            gdwElapseTime = gdwElapseTime + (DWORD)gwScanTime;
            dwErrorTime = dwCurrentTime - gdwElapseTime - gdwStartTime;
            if (dwErrorTime > (DWORD)40)        // maximum 40 ms scan time
            {
                gdwElapseTime = dwCurrentTime - gdwStartTime;
                gwOverrunCount ++;
                if (gwOverrunCount >= 10)
                {
                    bTimerOverrun = TRUE;
                    gwOverrunCount = 0;
                }
                else
                    bTimerOverrun = FALSE;
            }
            else
            {
                bTimerOverrun = FALSE;
                gwOverrunCount = 0;
            }

            //
            // reads an analog input channel
            //

            // check start and stop channel if expansion board ?
            usExpFlag = 0;
            for (i = usStartChan ; i <= usStopChan ; i++)
            {
                if (DevCfg.Daughter[i&0xf].dwBoardID != 0)
                    usExpFlag = 1;
            }

            if(usExpFlag == 0)
            {
               ptMAIVoltageIn.NumChan   = usStopChan - usStartChan + 1;
               ptMAIVoltageIn.StartChan = usStartChan;

               for (i=0 ; i< ptMAIVoltageIn.NumChan ; i++)
                 usGainCode[i+usStartChan] =
                    DevFeatures.glGainList[usGainIndex[i+usStartChan]].usGainCde;
               ptMAIVoltageIn.GainArray = (USHORT far *) &usGainCode[usStartChan];

               ptMAIVoltageIn.TrigMode = 0;            // internal trigger
               ptMAIVoltageIn.VoltageArray = (FLOAT far *)&fVoltage[usStartChan];

               if ((ErrCde = DRV_MAIVoltageIn(DriverHandle,
                          (LPT_MAIVoltageIn)&ptMAIVoltageIn)) != 0)
               {
                   DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
				   KillTimer(hWnd, 1);
                   MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                   DRV_DeviceClose((LONG far *)&DriverHandle);
                   return 0;
               }

            }
            else
            {
               for (i = usStartChan ; i <= usStopChan ; i++)
               {
                  usDasChan[i] = i;

                  // for PCLD-788
                  if  (DevCfg.Daughter[i&0xf].dwBoardID == BD_PCLD788)
                      usExpChanTemp[i] = ((usBoardID[i] << 4) | usExpChan[i]);
                  else
                      usExpChanTemp[i] = usExpChan[i];
               }

               ptMAIVoltageInExp.NumChan      = usStopChan - usStartChan + 1;
               ptMAIVoltageInExp.DasChanArray = (USHORT far *)&usDasChan[usStartChan];
               ptMAIVoltageInExp.DasGainArray = (USHORT far *)&usGainIndex[usStartChan];
               ptMAIVoltageInExp.ExpChanArray = (USHORT far *)&usExpChanTemp[usStartChan];
               ptMAIVoltageInExp.VoltageArray = (FLOAT far *)&fVoltage[usStartChan];

               if ((ErrCde = DRV_MAIVoltageInExp(DriverHandle,
                          (LPT_MAIVoltageInExp)&ptMAIVoltageInExp)) != 0)
               {
                   DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
				   KillTimer(hWnd, 1);
                   MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                   DRV_DeviceClose((LONG far *)&DriverHandle);
                   return 0;
               }

            }
            //
            // Display Data
            //

            hdc = GetDC(hWnd);
            GetClientRect(hWnd, &rect);
            InvalidateRect(hWnd, NULL, TRUE);
            UpdateWindow(hWnd);
            if (bTimerOverrun == FALSE)
            {
                for (i=usStartChan; i <= usStopChan ; i++)
                {
                    sprintf(szBuffer, "data[%d] = %10.6f", i-usStartChan, fVoltage[i]);
                    TextOut(hdc, 300, 20*(i-usStartChan), szBuffer, 20);
                }
            }
            else
            {
                sprintf(szBuffer, "Timer Overrun!\n data = %10.6f", fVoltage[0]);
                DrawText(hdc, szBuffer, -1, &rect,
                    DT_SINGLELINE | DT_CENTER | DT_VCENTER);
            }

            ReleaseDC(hWnd, hdc);

            return 0;

        case WM_DESTROY:
            if (bRun == TRUE)
            {
                KillTimer(hWnd, 1);
                DRV_DeviceClose((LONG far *)&DriverHandle);
            }
            PostQuitMessage(0);
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }

    return ((LONG)NULL);
}

PRIVATE void AllWindows(HWND hDlg)
{
    EnableWindow(GetDlgItem(hDlg, IDC_STARTCHAN),   FALSE);
    EnableWindow(GetDlgItem(hDlg, IDC_STOPCHAN),    FALSE);

    EnableWindow(GetDlgItem(hDlg, IDC_CURRENTCHAN), FALSE);
    EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE),    FALSE);
    EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN),     FALSE);
    EnableWindow(GetDlgItem(hDlg, IDC_BOARDID),     FALSE);
}

PRIVATE void SetChannel(HWND hDlg)
{
    // this subroutine use variable usMaxChannel, usStartChan, usStopChan
    USHORT i;
    USHORT usStopChan;
    usStopChan = usStartChan + usNumChan - 1;

    EnableWindow(GetDlgItem(hDlg, IDC_STARTCHAN),   TRUE);
    EnableWindow(GetDlgItem(hDlg, IDC_STOPCHAN),    TRUE);
    SendDlgItemMessage(hDlg, IDC_STARTCHAN, CB_RESETCONTENT, 0, 0L);
    SendDlgItemMessage(hDlg, IDC_STOPCHAN,  CB_RESETCONTENT, 0, 0L);
    // setting dialog box for start channel setting
    for (i=0; i< usMaxChannel; ++i)
    {
        wsprintf(szBuf, "%d", i);
        /*
        if (i == DevCfg.usCjcChannel) // CJC channel
        {
            lstrcat(szBuf, " (CJC Channel)");
        }
        else */
        if (DevCfg.Daughter[i&0xf].dwBoardID!=0L)// exp. board exist
        {
            lstrcat(szBuf, " (");
            DRV_BoardTypeMapBoardName(DevCfg.Daughter[i&0xf].dwBoardID,
                                      (LPSTR)szExpName);
            lstrcat(szBuf, szExpName);
            lstrcat(szBuf, ")");
        }
        
        SendDlgItemMessage(hDlg, IDC_STARTCHAN, CB_ADDSTRING,(WPARAM) -1,
            (LPARAM)(LPSTR)szBuf);
    }

    // setting dialog box for end channel setting
    for (i=0; i< usMaxChannel; ++i)
    {
        wsprintf(szBuf, "%d", i);
        if (i == DevCfg.usCjcChannel) // CJC channel
        {
            lstrcat(szBuf, " (CJC Channel)");
        }
        else if (DevCfg.Daughter[i&0xf].dwBoardID!=0L)// exp. board exist
        {
            lstrcat(szBuf, " (");
            DRV_BoardTypeMapBoardName(DevCfg.Daughter[i&0xf].dwBoardID,
                                      (LPSTR)szExpName);
            lstrcat(szBuf, szExpName);
            lstrcat(szBuf, ")");
        }
        SendDlgItemMessage(hDlg, IDC_STOPCHAN, CB_ADDSTRING,(WPARAM) -1,
            (LPARAM)(LPSTR)szBuf);
    }

    // set usStartChan & usStopChan
    if (usStopChan < usStartChan)
        usStopChan = usStartChan;
    SendDlgItemMessage(hDlg, IDC_STARTCHAN, CB_SETCURSEL, usStartChan,0L);
    SendDlgItemMessage(hDlg, IDC_STOPCHAN,  CB_SETCURSEL, usStopChan,0L);
}

PRIVATE void SetCurrentChan(HWND hDlg, BOOL bflag)
{
    USHORT i;
    USHORT usStopChan;
    usStopChan = usStartChan + usNumChan - 1;

    if (bflag == TRUE)   // Enable current chan setting for Varied
    {
        EnableWindow(GetDlgItem(hDlg, IDC_CURRENTCHAN), TRUE);
        SendDlgItemMessage(hDlg, IDC_CURRENTCHAN, CB_RESETCONTENT,
            0, (LPARAM)((LPSTR)0));
        for (i = usStartChan; i <= usStopChan; i ++)
        {
            wsprintf(szBuf, "%d", i);
            SendDlgItemMessage(hDlg, IDC_CURRENTCHAN, CB_ADDSTRING, 0,
                (LPARAM)(LPSTR)szBuf);
        }

        wsprintf(szBuf, "%d", usCurrentChan);
        SendDlgItemMessage(hDlg, IDC_CURRENTCHAN,
           CB_SELECTSTRING, (WPARAM)(-1), (LPARAM)((LPSTR)szBuf));
    }
    else
    {
        EnableWindow(GetDlgItem(hDlg, IDC_CURRENTCHAN), FALSE);
    }
}

PRIVATE void SetGain(HWND hDlg, BOOL bflag)
{
    USHORT i;

    if (bflag == TRUE)   // Enable gain setting
    {
        EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
        SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
            0, (LPARAM)((LPSTR)0));
        for (i = 0; i < DevFeatures.usNumGain; i ++)
            SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_ADDSTRING, 0,
                (LPARAM)((LPSTR)(DevFeatures.glGainList[i].szGainStr)));

        SendDlgItemMessage(hDlg, IDC_INPRANGE,
           CB_SELECTSTRING, (WPARAM)(-1),
           (LPARAM)((LPSTR)(DevFeatures.glGainList[usGainIndex[usCurrentChan]].szGainStr)));
    }
    else
    {
        EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
    }
}

PRIVATE void SetExpChan(HWND hDlg, BOOL bflag)
{
    USHORT i;

    if (bflag == TRUE)   // Enable expansion channel
    {
        EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), TRUE);
        SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_RESETCONTENT,
            0, (LPARAM)((LPSTR)0));

        for (i=0; i<DevCfg.Daughter[usCurrentChan&0xf].usNum; ++i)
        {
            wsprintf(szBuf, "%d", i);
            SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_ADDSTRING,(WPARAM)-1,
                (LPARAM)(LPSTR)szBuf);
        }

        SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_SETCURSEL,
            usExpChan[usCurrentChan],0L);
    }
    else
    {
        EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), FALSE);
    }
}

PRIVATE void SetBoardID(HWND hDlg, BOOL bflag)
{
    USHORT i;

    if (bflag == TRUE)   // Enable PCLD-788 board ID setting
    {
        EnableWindow(GetDlgItem(hDlg, IDC_BOARDID), TRUE);
        SendDlgItemMessage(hDlg, IDC_BOARDID, CB_RESETCONTENT, 0, 0L);

        for (i=0; i<DevCfg.Daughter[usCurrentChan&0xf].usCards; ++i)
        {
            wsprintf(szBuf, "%d", i);
            SendDlgItemMessage(hDlg, IDC_BOARDID, CB_ADDSTRING, (WPARAM)-1,
                (LPARAM)(LPSTR)szBuf);
        }
        SendDlgItemMessage(hDlg, IDC_BOARDID, CB_SETCURSEL,
            usBoardID[usCurrentChan],0L);
    }
    else
    {
        EnableWindow(GetDlgItem(hDlg, IDC_BOARDID), FALSE);
    }
}
