
/*--------------------------------------
FILE NAME       : can841.C
VERSION         : V1.0
--------------------------------------*/
#include "can841.h"
#include <dos.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
 
#define IRQ_POLLING     0
#define NULL            0
/*--------------------------------------------------------------------------*/
BYTE gIrq0, gIrq1;
BYTE old_irq0=IRQ_POLLING;
BYTE old_irq1=IRQ_POLLING;
UI   gSeg;
int hw_ok=0;

MSG_RECE_STRUCT Rece_Buf[PACKRECEMAX];
MSG_REPLY_STRUCT ReplyPackBuf[PACKRECEMAX];
MSG_SEND_STRUCT Send_Buf[PACKSENDMAX];

void interrupt (*old_isr0)();
void interrupt (*old_isr1)();

void interrupt can_isr0();
void interrupt can_isr1();

int can_err_no;
	/*------------------------------------------------
	0: no error
	0x01 : hardware not release.
	0x10: seg_addr set error
	0x11: hardware not set successful
	0x12: irq_no set error
	0x20: bus off
	------------------------------------------------*/
/*--------------------------------------------------------------------------*/
void interrupt can_isr0();
void interrupt can_isr1();
void can0w(BYTE addr, BYTE v);
void can1w(BYTE addr, BYTE v);
BYTE can0r( BYTE addr);
BYTE can1r( BYTE addr);
void  can_reset( BYTE port );
/*--------------------------------------------------------------------------*/
void getIntNo(BYTE irq_no, BYTE * int_no)
{
  BYTE int_tbl[16]={
  0xff,0xff,0xff,0x0b,0x0c,0x0d,0x0e,0x0f,
  0xff,0x71,0x72,0x73,0x74,0xff,0xff,0x77
  };
  *int_no = int_tbl[irq_no];
}
/*--------------------------------------------------------------------------*/
int canInitHW( UI segment, BYTE irq0, BYTE irq1)
{
  BYTE data, data2;
  BYTE int_no;

  /*-----------------------
  If app. call this function again, error it.
  --------------------------*/
  /*-----------------------
  if( hw_ok!=0)
  {
    can_err_no=0x1;
    return(0);
  }
  -------------------------*/
  if( segment >0xdf00 || segment <0xc000 ||(segment%0x100))
  {
    can_err_no=0x10;
    return(0);
  }
  gSeg=segment;
  data=can0r(0);
  if( data &0x01)
  {
    can0w(0,0x60);
    if(can0r(0) &0x01)
    {
      can_err_no=0x10;
      return(0);
    }
  }
  can_reset(0);
  if(!(can0r(0) &0x01))
  {
      can_err_no=0x10;
      return(0);
  }
  /*----- irq_set  error check. ------------------*/
  if( irq0 != 0 && irq0 != 3 && irq0 != 4 && irq0 !=5 && irq0 !=6 && irq0 !=7
      && irq0 !=9 &&  irq0 !=10 && irq0 !=11 && irq0 !=12 && irq0 != 15)
  {
      can_err_no=0x12;
      return(0);
  }
  if( irq1 != 0 && irq1 != 3 && irq1 != 4 && irq1 !=5 && irq1 !=6 && irq1 !=7
      && irq1 !=9 && irq1 !=10 && irq1 !=11 && irq1 !=12 && irq1 != 15)
  {
      can_err_no=0x12;
      return(0);
  }
  if( irq0 == irq1  && irq0 !=IRQ_POLLING)
  {
      can_err_no=0x12;
      return(0);
  }
  getIntNo(irq0, &data);
  if( data == 0xff && irq0 !=0)
  {
      can_err_no=0x12;
      return(0);
  }
  getIntNo(irq1, &data);
  if( data == 0xff && irq1 !=0)
  {
      can_err_no=0x12;
      return(0);
  }/*..... irq_set  error check. ..................*/

  gIrq0=irq0;
  if( gIrq0 != old_irq0)
  {
    /*---- restore old irq int service routine -------*/
    if( old_irq0 !=IRQ_POLLING)
    {
      data=inportb(0x21);
      data2=inportb(0x21);
      outportb(0x21,0xff);
      outportb(0xA1,0xff);
      disable();
      getIntNo(old_irq0, &int_no);
      setvect(int_no,old_isr0);

      enable();

      old_irq0 = IRQ_POLLING;
    }/*.... restore old irq int service routine .......*/

    /*---- set new irq int service routine -------*/
    if( gIrq0 != IRQ_POLLING) /* not polling */
    {
      getIntNo(gIrq0,&int_no);
      old_isr0=getvect(int_no);
      old_irq0 = gIrq0;

      data=inportb(0x21);
      outportb(0x21,0xff);
      disable();
      getIntNo(gIrq0,&int_no);
      setvect(int_no,can_isr0);
      outportb(0x20,0x20);
      enable();
      /*outportb(0x21,data);*/
      outportb(0x21,0);
      outportb(0xA1,1);
    }
  }

  gIrq1=irq1;
  if( gIrq1 != old_irq1)
  {
    /*---- restore old irq int service routine -------*/
    if( old_irq1 !=IRQ_POLLING)
    {
      data=inportb(0x21);
      outportb(0x21,0xff);
      disable();
      getIntNo(old_irq1, &int_no);
      setvect(int_no,old_isr1);
      outportb(0x20,0x20);
      enable();
      outportb(0x21,data);

      old_irq1 = IRQ_POLLING;
    }/*.... restore old irq int service routine .......*/

    /*---- set new irq int service routine -------*/
    if( gIrq1 != IRQ_POLLING) /* not polling */
    {
      getIntNo(gIrq1, &int_no);
      old_isr1=getvect(int_no);
      old_irq1 = gIrq1;

      data=inportb(0x21);
      outportb(0x21,0xff);
      outportb(0xA1,0xff);
      disable();

      getIntNo(gIrq1, &int_no);
      setvect(int_no,can_isr1);

      enable();
      /*outportb(0x21,data);*/
      outportb(0x21,0);
      outportb(0xA1,1);
    }
  }

  hw_ok=1;
  return(1);
}
/*--------------------------------------------------------------------------*/
BYTE rBuf0[10], rBuf1[10];
BYTE gSendFlag0, gSendFlag1;
BYTE gRecFlag0, gRecFlag1;
/*--------------------------------------------------------------------------*/
void interrupt can_isr0()
{
  unsigned char flag;
  int i;

  disable();
  flag=can0r(3);
  if( flag & 1) /* RIF */
  {
    rBuf0[0]=can0r(20);
    rBuf0[1]=can0r(21);
    for(i=0; i< (rBuf0[1]&0x0f); i++)
      rBuf0[i+2]=can0r(22+i);
    /* release receive buffer*/
    can0w(1,4);
    gRecFlag0=1;
  }
  if( flag & 2) /* TIF */
  {
    gSendFlag0=1;
  }
   outportb(0x20,0x20);
   outportb(0xA0,0x20);
   enable();
}
/*--------------------------------------------------------------------------*/
void interrupt can_isr1()
{
  static int rece_flag = 0,rece_last = 0;
  static unsigned short checksum = 0;
  unsigned char flag;
  int i,j;


  disable();
  flag=can1r(3);
  if( flag & 1) /* RIF */
  {
    rBuf1[0]=can1r(20); //addr
    rBuf1[1]=can1r(21);//len
    for(i=0; i< (rBuf1[1]&0x0f); i++)
      rBuf1[i+2]=can1r(22+i);
    /* release receive buffer*/
    can1w(1,4);
    //gRecFlag1=1;
    if ( rBuf1[2] == 0xF0 )
      {
	 for ( i = 0; i < PACKRECEMAX; i ++ )
	 {
	    rece_last %= PACKRECEMAX;
	    if ( Rece_Buf[rece_last].flag == 2 && Rece_Buf[rece_last].timeout > PACKTIMEOUT )
		 Rece_Buf[rece_last].flag = 0;
	    if ( Rece_Buf[rece_last++].flag == 0 )
	    {
		rece_flag = rece_last;
		memset(&Rece_Buf[rece_flag-1],0,sizeof(Rece_Buf[rece_flag-1]));
		Rece_Buf[rece_flag-1].id=rBuf1[0] & 0x7f;
		//Rece_Buf[rece_flag-1].id <<=3;
		//Rece_Buf[rece_flag-1].id |=(rBuf1[1] >>5);
		Rece_Buf[rece_flag-1].flag = 2;
		Rece_Buf[rece_flag-1].timeout = 0;
		Rece_Buf[rece_flag-1].plen = rBuf1[3];
		memset(&Rece_Buf[rece_flag-1].data,0,sizeof(Rece_Buf[0].data));
		break;
	    }
	 }
	 if ( i == PACKRECEMAX ) cprintf("\n\rfull");
      }
    else  if ( rBuf1[2] == 0xF1 && rece_flag )
      { // cprintf("\r\n%d %d  ",rece_flag);

	 checksum = 0;
	 for ( i = 0; i < Rece_Buf[rece_flag-1].plen; i ++ )
	    checksum += Rece_Buf[rece_flag-1].data[i];

	 ReplyPackBuf[0].port = 0;
	 ReplyPackBuf[0].rid = rBuf1[0] & 0x7f;
	 ReplyPackBuf[0].data[0] = 0xF2;
	 if ( checksum == rBuf1[5]+rBuf1[6]*256 )
	 {
	    Rece_Buf[rece_flag-1].flag = 1; //ok
	    ReplyPackBuf[0].data[1] = 1;
	 }
	 else
	 {
	    Rece_Buf[rece_flag-1].flag = 0; //fail
	    ReplyPackBuf[0].data[1] = 0;
            cprintf("\r\nerror\r\n");
	 }
	 rece_flag = 0;
	 ReplyPackBuf[0].data[2] = rBuf1[3];
	 ReplyPackBuf[0].data[3] = rBuf1[4];
	 ReplyPackBuf[0].flag = 1;
	 //cprintf("\n\r%5x %5x",rBuf1[3],rBuf1[4]);
      }
    else if ( rBuf1[2] == 0xF2 )
    {
       if ( rBuf1[3] == 1 )
       {
       j = rBuf1[4]+rBuf1[5]*256;
       for ( i = 0; i < PACKSENDMAX ; i ++ )
	  if ( Send_Buf[i].flag && Send_Buf[i].order == j )
	     {  Send_Buf[i].flag = 0; break; }
       }
	else     cprintf(" error %d",rBuf1[3]);

    }
    if ( rBuf1[2] < 0x80  && rece_flag )
       if ( rBuf1[2] < PACKORDERMAX )
	     memcpy(&Rece_Buf[rece_flag -1].data[rBuf1[2]*7],&rBuf1[3],rBuf1[1]&0x0f);
  }
  if( flag & 2) /* TIF */
  {
    gSendFlag1=1;
  }
   outportb(0x20,0x20);
   outportb(0xA0,0x20);
   enable();
}
/*--------------------------------------------------------------------------*/
void  can_reset( BYTE port )
{

	BYTE huge *ptr=MK_FP(gSeg+0x200*port,0x100);
	BYTE a;

	a=*ptr;
	*ptr=a;
	delay(100);
}
/*--------------------------------------------------------------------------*/
void can0w(BYTE addr, BYTE v)
{
	BYTE huge *ptr;
	ptr = MK_FP( gSeg,addr);
	*ptr=v;
}
/*--------------------------------------------------------------------------*/
void can1w(BYTE addr, BYTE v)
{
	BYTE huge *ptr;
	ptr = MK_FP( gSeg,addr+0x200);
	*ptr=v;
}
/*--------------------------------------------------------------------------*/
BYTE can0r( BYTE addr)
{
  BYTE v;
  BYTE huge *ptr;
	ptr = MK_FP(gSeg,addr);
	v=*ptr;
	return(v);
}
/*--------------------------------------------------------------------------*/
BYTE can1r( BYTE addr)
{
  BYTE v;
  BYTE huge *ptr;
	ptr = MK_FP(gSeg,addr+0x200);
	v=*ptr;
	return(v);
}
/*--------------------------------------------------------------------------*/
BYTE canExitHW()
{
  unsigned char d=inportb(0x21);
  BYTE int_no;
  /* reset old isr */
  outportb(0x21,0xff);
  disable();
  if( old_isr0 != NULL )
  {
       getIntNo(old_irq0,&int_no);
       setvect(int_no,old_isr0);
       old_irq0=IRQ_POLLING;
  }
  if( old_isr1 != NULL )
  {
       getIntNo(old_irq0,&int_no);
       setvect(old_irq1,old_isr1);
       old_irq1=IRQ_POLLING;
  }
  enable();
  outportb(0x21,d);
}
/*--------------------------------------------------------------------------*/
int  canReset( BYTE port )
{
  can_reset( port );
}
/*--------------------------------------------------------------------------*/
int  canConfig( BYTE port, CAN_STRUCT can)
{
  BYTE temp;
  canReset( port );
  if( port ==0)
  {
    if( gIrq0 == IRQ_POLLING)
      can0w(0,1);
    else
     can0w(0,3);/* reset mode and receive irq */
  }
  else
  {
    if( gIrq1 == IRQ_POLLING)
      can1w(0,1);
    else
     can1w(0,3);/* reset mode and receive irq */
  }
  if( port ==0)
  {
     can0w(6,can.bt0); /* BT0 */
     can0w(7,can.bt1); /* BT1 */
     temp=can0r(6);
     if( temp !=can.bt0)
       return(0);
     temp=can0r(7);
     if( temp !=can.bt1)
       return(0);
  }
  else
  {
     can1w(6,can.bt0); /* BT0 */
     can1w(7,can.bt1); /* BT1 */
     temp=can1r(6);
     if( temp !=can.bt0)
       return(0);
     temp=can1r(7);
     if( temp !=can.bt1)
       return(0);
  }
   if( port ==0)
   {
     can0w(4,can.acc_code);/* accept code */
     can0w(5,can.acc_mask);/* accept code */
   }
   else
   {
     can1w(4,can.acc_code);/* accept code */
     can1w(5,can.acc_mask);/* accept code */
   }
   if( port ==0)
   {
     can0w(8,0xfa);
   }
   else
   {
     can1w(8,0xfa);
   }
  return(1);
}
/*--------------------------------------------------------------------------*/
int  canNormalRun( BYTE port )
{
BYTE v;
  if( port ==0)
  {
	v=can0r(0);
        if( v & 1)
	  can0w(0,v&0xfe);
  }
  else
  {
        v=can1r(0);
        if( v & 1)
	  can1w(0,v&0xfe);
  }
}
/*--------------------------------------------------------------------------*/
int canSendMsg( BYTE  port, MSG_STRUCT msg)
{
  BYTE v;
  BYTE i;

  if( port==0)
  {
    //v=msg.id>>3;
    v = msg.id & 0x7f;
//    v = msg.id | 0x80;
    can0w(10,v);
    //v=msg.id &7;
    //v<<=5;
    v=msg.dlen;
    if( msg.rtr ==1)
      v|=0x10;
    //v+=msg.dlen;
    can0w(11,v);
    for(i=0;i<msg.dlen; i++)
    {
      can0w(12+i,msg.data[i]);
    }
    can0w(1,1);
    while(1)
    {
      v=can0r(2);
      if( v & 0x40)
      {
	return( 0);
      }
      if( v &0x08)
        return( 1);
    }
  }
  else
  {
    v=msg.id>>3;
    can1w(10,v);
    v=msg.id &7;
    v<<=5;
    if( msg.rtr ==1)
      v|=0x10;
      v+=msg.dlen;
    can1w(11,v);
    for(i=0;i<msg.dlen; i++)
    {
      can1w(12+i,msg.data[i]);
    }
    can1w(1,1);
    while(1)
    {
      v=can1r(2);
      if( v & 0x40)
      {
	return( 0);
      }
      if( v &0x08)
        return( 1);
    }
  }
}
/*--------------------------------------------------------------------------*/
int  canReceiveMsg( BYTE port, MSG_STRUCT *msg_ptr)
{
BYTE i;

   if( port ==0)
   {
     if( gIrq0 == IRQ_POLLING)
     {
        if( can0r(2)&1)
        {
          rBuf0[0]=can0r(20);
          rBuf0[1]=can0r(21);
	  for(i=0; i< (rBuf0[1]&0x0f); i++)
	    rBuf0[i+2]=can0r(22+i);
	  /* release receive buffer*/
	  can0w(1,4);
	  gRecFlag0=1;
	}
     }
      if( gRecFlag0 == 1)
      {
	gRecFlag0 = 0;
	msg_ptr->id=rBuf0[0];
	msg_ptr->id <<=3;
	msg_ptr->id |=(rBuf0[1] >>5);
	msg_ptr->rtr =(rBuf0[1]>>4)&1;
	msg_ptr->dlen =rBuf0[1]&0x0f;
	for(i=0;i< msg_ptr->dlen; i++)
	  msg_ptr->data[i]=rBuf0[2+i];
        return( 1);
      }
      else
        return( 0);
   }
   else if(port ==1)
   {
     if( gIrq1 == IRQ_POLLING)
     {
        if( can1r(2)&1)
        {
          rBuf1[0]=can1r(20);
          rBuf1[1]=can1r(21);
          for(i=0; i< ( rBuf1[1]&0x0f); i++)
            rBuf1[i+2]=can1r(22+i);
	  /* release receive buffer*/
          can1w(1,4);
          gRecFlag1=1;
        }
     }
      if( gRecFlag1 == 1)
      {
        gRecFlag1 = 0;
        msg_ptr->id=rBuf1[0];
        msg_ptr->id <<=3;
        msg_ptr->id |=(rBuf1[1] >>5);
        msg_ptr->rtr =(rBuf1[1]>>4)&1;
        msg_ptr->dlen =rBuf1[1]&0x0f;
	for(i=0;i< msg_ptr->dlen; i++)
          msg_ptr->data[i]=rBuf1[2+i];
        return( 1);
      }
      else
        return( 0);
   }
   else
     return( 0);
}
/*--------------------------------------------------------------------------*/
_SendPack(MSG_SEND_STRUCT *send)
{
   int packcount,i,j,k;
   unsigned short checksum = 0;
   MSG_STRUCT smsg;

   packcount = send->plen / 8 + 1;
   smsg.id = send->id;
   smsg.rtr=0;		    /* data lengths = 8	    */
   smsg.dlen = 2; smsg.data[0] = 0xf0;
   smsg.data[1] = send->plen;
   canSendMsg( send->port, smsg);
   for ( i = 0,k=0; i < packcount ; i ++ )
     {
	delay(2);
	smsg.data[0] = i;
	for(j=1;j < 8 && k < send->plen ;j++, k ++ )
	 {
	   smsg.data[j] = send->data[k] ;
	   checksum += send->data[k];
	 }
	smsg.dlen = j;
	canSendMsg( send->port, smsg);
     }
    smsg.dlen = 5;
    smsg.data[0] = 0xf1;
    smsg.data[1] = send->order%256;
    smsg.data[2] = send->order/256;
    smsg.data[3] = checksum%256;
    smsg.data[4] = checksum/256;
//    if ( rand() % 3  )
    canSendMsg( send->port, smsg);
	delay(2);

}

int RecePack(int *addr,int *len,char *buf)
{
  static int last=0;
  int i,j;
  for (i = 0 ; i < PACKRECEMAX ; i ++ )
  {
     last %= PACKRECEMAX;
     if ( Rece_Buf[last++].flag == 1 )
     {
       j = last - 1;
       *addr = Rece_Buf[j].id;
       *len =  Rece_Buf[j].plen;
       memcpy(buf,&Rece_Buf[j].data,Rece_Buf[j].plen);
       Rece_Buf[j].flag = 0;
       //memset(&Rece_Buf[j],0,sizeof(Rece_Buf[j]));
       return 1;
     }
  }
  return 0;
}
int SendPack(int port,int addr,int len,char *buf)
{
  static unsigned sendorder = 0;
  static int last=0;
  int i,j;
  for (i = 0 ; i < PACKSENDMAX ; i ++ )
  {
     last %= PACKRECEMAX;
     if ( Send_Buf[last++].flag == 0 )
     {
       j = last - 1;
       Send_Buf[j].port = port;
       Send_Buf[j].id = addr;
       Send_Buf[j].plen = len;
       Send_Buf[j].order = sendorder++;
       memcpy(&Send_Buf[j].data,buf,len);
       Send_Buf[j].retry = 0;
       Send_Buf[j].flag = 1;
       return 1;
     }
  }
  return 0;
}

void PackReply(void)
{
  static MSG_STRUCT smsg;
  static int last=0;
  int i,j;

  for (i = 0 ; i < PACKRECEMAX ; i ++ )
  {
     last %= PACKRECEMAX;
     if ( ReplyPackBuf[last++].flag == 1 )
     {  j = last - 1;
	smsg.id = ReplyPackBuf[j].rid;
	smsg.rtr=0;		    /* data lengths = 8	    */
	smsg.dlen = 4;
	smsg.data[0] = 0xf2;
	smsg.data[1] = ReplyPackBuf[j].data[1];
	smsg.data[2] = ReplyPackBuf[j].data[2];
	smsg.data[3] = ReplyPackBuf[j].data[3];
	canSendMsg( ReplyPackBuf[j].port, smsg);
	ReplyPackBuf[j].flag = 0;
     }
  }
}
void TimeOutCount()
{
  int i;
  for (i = 0 ; i < PACKRECEMAX ; i ++ )
     if ( Rece_Buf[i].flag == 2 )
	 Rece_Buf[i].timeout ++ ;
  for ( i = 0; i < PACKSENDMAX; i ++ )
     if ( Send_Buf[i].flag == 2 )
	 Send_Buf[i].timeout ++ ;
}
void interrupt  ( *oldClock ) (void);     //DOSʱж
void interrupt newClock ()
{
   static int running = 0;
   static unsigned int ticker=0,ticker1=0;
   int  i;

   ticker++;                //timer: display time
   TimeOutCount();
   if ( running ) return;
   running = 1;
   if ( ticker >=18 )
   {  ticker1++;
      if ( ticker1 >= 5 )
	  { ticker=-1; ticker1=0; }
      else ticker=0;
      //SecClock();
   }

   //PackReply();
   oldClock();
   running = 0;
   return;
}
