// DeviceDlg.cpp : implementation file
//

#include "stdafx.h"
#include "841m.h"
#include "841MDoc.h"
#include "841MView.h"
#include "DeviceDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define DEVICE_NUM_LEN   3
#define MODULE_NUM_LEN   3
#define MAX_COM_MODULES  1023
#define MAX_CAN_MODULES  1023
#define MaxDev           255


BOOL ConvertDeviceNumToString(DWORD dwNum,int count,char * szResult)
{
    char szBuf[10];
    int  len,index;

    if (count > 9) return FALSE;
    _ultoa(dwNum,szBuf,10);
    if ((len = strlen(szBuf))  > count) return FALSE;
    for (index = 0; index < count; index ++)
        szResult[index] = '0';

    for (index = 0; index <= len; index ++)
        szResult[count-len+index] = szBuf[index];

    return TRUE;
}

// --------------------------------------------------------------------------
// FUNCTION: QueryConfigDeviceData (for DeviceXXX)
//
// PURPOSE: Get one configuration data from Registry or Config. File
//
// Parameters: dwDeviceNum (In) - device number
//             szEntry (In)     - queried entry
//             szDataBuf (Out)  - buffer for returned data
//             dwDataSize (In)  - size of szDataBuf
//
// Return:     (1) FALSE if not found
//             (2) TRUE if success
// --------------------------------------------------------------------------
BOOL QueryConfigDeviceData(DWORD dwDeviceNum,char * szEntry,
    LPBYTE szDataBuf, DWORD dwDataSize)
{
    char szBuf[120];
    char szDeviceName[50];
    HKEY khDevice;
    DWORD dwDataType;

    // Prepare szDeviceName (Ex: 000,001,002...)
    ConvertDeviceNumToString( dwDeviceNum, DEVICE_NUM_LEN, szDeviceName );

    // open the \\ADSDAQ\\DeviceXXX key
    strcpy( szBuf, ADSDAQ_REGISTRY_BASE );
    strcat( szBuf, "\\Device" );
    strcat( szBuf, szDeviceName );
    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, szBuf, 0, KEY_READ, &khDevice) != ERROR_SUCCESS)
        return FALSE;

    // query the value of DeviceName put in gszDeviceName
    if (RegQueryValueEx(khDevice,
        szEntry,			// the value to query
        NULL,				// reserved
        &dwDataType,		// returned type of the value
        (LPBYTE)szDataBuf,  // returned data
        &dwDataSize)		// size of the buffer
        != ERROR_SUCCESS)
    {
        RegCloseKey(khDevice);
        return FALSE;
    }
    else
    {
        RegCloseKey(khDevice);
        return TRUE;
    }
}

// --------------------------------------------------------------------------
// FUNCTION: QueryConfigModuleData (for DeviceXXX)
//
// PURPOSE: Get one configuration data from Registry or Config. File
//
// Parameters: dwDeviceNum (In) - device number
//             dwModuleNum (In) - module number
//             szEntry (In)     - queried entry
//             szDataBuf (Out)  - buffer for returned data
//             dwDataSize (In)  - size of szDataBuf
//
// Return:     (1) FALSE if not found
//             (2) TRUE if success
// --------------------------------------------------------------------------
BOOL QueryConfigModuleData(DWORD dwDeviceNum, DWORD dwModuleNum, char * szEntry,
    LPBYTE szDataBuf, DWORD dwDataSize)
{
    char szBuf[120];
    char szDeviceName[50];
    HKEY khDevice;
    DWORD dwDataType;

    // open the \\ADSDAQ\\DeviceXXX key
    strcpy( szBuf, ADSDAQ_REGISTRY_BASE );
    strcat( szBuf, "\\Device" );
    ConvertDeviceNumToString( dwDeviceNum, DEVICE_NUM_LEN, szDeviceName );
    strcat( szBuf, szDeviceName );
    strcat( szBuf, "\\Module" );
    ConvertDeviceNumToString( dwModuleNum, DEVICE_NUM_LEN, szDeviceName );
    strcat( szBuf, szDeviceName );
    if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, szBuf, 0, KEY_READ, &khDevice) != ERROR_SUCCESS)
        return FALSE;

    // query the value of DeviceName put in gszDeviceName
    if (RegQueryValueEx(khDevice,
        szEntry,			// the value to query
        NULL,				// reserved
        &dwDataType,		// returned type of the value
        (LPBYTE)szDataBuf,  // returned data
        &dwDataSize)		// size of the buffer
        != ERROR_SUCCESS)
    {
        RegCloseKey(khDevice);
        return FALSE;
    }
    else
    {
        RegCloseKey(khDevice);
        return TRUE;
    }
}

/////////////////////////////////////////////////////////////////////////////
// DeviceDlg dialog


DeviceDlg::DeviceDlg(CWnd* pParent /*=NULL*/)
	: CDialog(DeviceDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(DeviceDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
//	ASSERT(pParent != NULL);

	m_pParent = pParent;
	m_DeviceNum  = -1;
}


void DeviceDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DeviceDlg)
	DDX_Control(pDX, IDC_DEVICETREE, m_DeviceTree);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(DeviceDlg, CDialog)
	//{{AFX_MSG_MAP(DeviceDlg)
	ON_NOTIFY(TVN_SELCHANGED, IDC_DEVICETREE, OnSelchangedDevicetree)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// DeviceDlg message handlers

BOOL DeviceDlg::OnInitDialog() 
{
    char	szBuf[128];
	char	gszDeviceName[50];
	DWORD	dwDeviceNum;
	DWORD	dwIndex;
	DWORD	dwResult;
	HKEY	keyHandle;
	DWORD	dwDataSize;
    FILETIME LastUpdate;
	HTREEITEM hTreeRootItem;

	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	TV_INSERTSTRUCT TreeCtrlItem;

	TreeCtrlItem.hParent = TVI_ROOT;
	TreeCtrlItem.hInsertAfter = TVI_LAST;
	TreeCtrlItem.item.mask = TVIF_TEXT | TVIF_PARAM;
	TreeCtrlItem.item.pszText = "ADSDAQ";
	TreeCtrlItem.item.lParam = 0;
	hTreeRootItem = m_DeviceTree.InsertItem(&TreeCtrlItem);

	// open the ADSDAQ key
	// enumerate the DeviceXXX key under ADSDAQ

	if ( (dwResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
								  ADSDAQ_REGISTRY_BASE,
								  0,
								  KEY_READ,
								  &keyHandle)) != ERROR_SUCCESS)
	{
//		LoadString( hInst, errOpenRegistryKey, szBuf, 80 );
		MessageBox( szBuf, "Device Setup", MB_ICONEXCLAMATION );
		return FALSE;
	}

	dwIndex = 0;
	do {
		dwDataSize = 80;
		// retrieve DeviceXXX in szBuf
		dwResult = RegEnumKeyEx(keyHandle,		// handle to an open key
								dwIndex,		// index of the key to get
								szBuf,			// buffer for returned name
								&dwDataSize,	// size of the buffer
								NULL,			// reserved
								NULL,			// returned data for the class
								NULL,			// size of the buffer
								&LastUpdate);	// time of last write

		if (dwResult == ERROR_SUCCESS) {
			// reject Enum key for NT 4.0 by Rose-Chou
			if (!((szBuf[0] == (char)'D') && (szBuf[1] == (char)'e'))) {
				dwIndex++;
				continue;
			}
			gszDeviceName[0] = '\0';
			dwDeviceNum = atoi(&szBuf[6]);
			if (QueryConfigDeviceData( dwDeviceNum,	"DeviceName", (LPBYTE)gszDeviceName, 50 ) == FALSE) {
//				LoadString( hInst, errDataLost, szBuf, 80 );
				MessageBox( szBuf, "Device Setup", MB_ICONEXCLAMATION );
				RegCloseKey( keyHandle );
				return TRUE;
			}

			strcat(szBuf,":");
			strcat(szBuf,gszDeviceName);
			dwIndex++;
			TreeCtrlItem.hParent = hTreeRootItem;
			TreeCtrlItem.item.pszText = szBuf;
			TreeCtrlItem.item.lParam = dwDeviceNum;
//			m_DeviceTree.InsertItem( &TreeCtrlItem );
			GetModuleList( m_DeviceTree.InsertItem( &TreeCtrlItem ), dwDeviceNum);
		}
	} while (dwResult == ERROR_SUCCESS);

	RegCloseKey(keyHandle);
	m_DeviceTree.Expand( hTreeRootItem, TVE_EXPAND );

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL DeviceDlg::GetModuleList(HTREEITEM hTreeRootItem, DWORD dwDeviceNum)
{
    char	szBuf[128];
	char	szDeviceName[50];
	DWORD	dwIndex;
	DWORD	dwResult;
	DWORD	dwModuleNum;
	HKEY	keyHandle;
	DWORD	dwDataSize;
    FILETIME LastUpdate;

	TV_INSERTSTRUCT TreeCtrlItem;

	TreeCtrlItem.hInsertAfter = TVI_LAST;
	TreeCtrlItem.item.mask = TVIF_TEXT | TVIF_PARAM;
	//
	// open the ADSDAQ key
	//
    // Prepare szDeviceName (Ex: 000,001,002...)
    ConvertDeviceNumToString( dwDeviceNum, DEVICE_NUM_LEN, szDeviceName );
    // open the \\ADSDAQ\\DeviceXXX key
    strcpy( szBuf, ADSDAQ_REGISTRY_BASE );
    strcat( szBuf, "\\Device" );
    strcat( szBuf, szDeviceName );

	// enumerate the ModuleXXX key under ADSDAQ\DeviceXXX

	if ( (dwResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
								  szBuf,
								  0,
								  KEY_READ,
								  &keyHandle)) != ERROR_SUCCESS)
	{
		MessageBox( szBuf, "Device Setup", MB_ICONEXCLAMATION );
		return FALSE;
	}

	dwIndex = 0;
	do {
		dwDataSize = 80;
		// retrieve ModuleXXX in szBuf
		dwResult = RegEnumKeyEx(keyHandle,		// handle to an open key
								dwIndex,		// index of the key to get
								szBuf,			// buffer for returned name
								&dwDataSize,	// size of the buffer
								NULL,			// reserved
								NULL,			// returned data for the class
								NULL,			// size of the buffer
								&LastUpdate);	// time of last write

		if (dwResult == ERROR_SUCCESS) {
			// reject Enum key for NT 4.0 by Rose-Chou
			if (!((szBuf[0] == (char)'M') && (szBuf[1] == (char)'o'))) {
				dwIndex++;
				continue;
			}
			szDeviceName[0] = '\0';
			dwModuleNum = atoi(&szBuf[6]);
			if (QueryConfigModuleData( dwDeviceNum, dwModuleNum, "DeviceName", (LPBYTE)szDeviceName, 50 ) == FALSE) {
				MessageBox( szBuf, "Device Setup", MB_ICONEXCLAMATION );
				RegCloseKey( keyHandle );
				return TRUE;
			}

			strcat(szBuf, ":");
			strcat(szBuf, szDeviceName);
			dwIndex++;
			TreeCtrlItem.hParent = hTreeRootItem;
			TreeCtrlItem.item.pszText = szBuf;
			TreeCtrlItem.item.lParam = dwModuleNum * 1000 + dwDeviceNum;
			m_DeviceTree.InsertItem( &TreeCtrlItem );
		}
	} while (dwResult == ERROR_SUCCESS);

	RegCloseKey(keyHandle);

	return TRUE;
}

void DeviceDlg::OnSelchangedDevicetree ( NMHDR* pNMHDR, LRESULT* pResult )
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	m_DeviceNum = pNMTreeView->itemNew.lParam;

	*pResult = 0;
}

void DeviceDlg::OnOK() 
{
	// TODO: Add your control notification handler code here

	CDialog::OnOK();
}
