{ ********************************************************************* }
{ Module Name: EVENT.PAS                                                }
{ Purpose: Event type definition headfile                               }
{ Version: 3.01                                                         }
{ Date: 01/08/2003                                                      }
{ Copyright (c) 1996-2003 Advantech Corp. Ltd.                          }
{ All rights reserved.                                                  }
{ ********************************************************************* }

{ Notes:								}
{ 1. Bit-mapped ID:							}
{    From 0x0001 to 0x0200 : defined at Driver.h, BIT-mapped Event ID.	}
{    From 0x0201 to 0x0205 : defined at Driver.h  ID-mapped Event ID.	}
{    From 0x0206 to 0x0fff : not used					}
{    From 0x1000 to 0xffff : defined at Event.h   ID-mapped Event ID.	}

{ 2. Version 2.0A and before don't use Event.h.				}

{ 3. EventID = 0x0000 is reserved for non event or error.		}
{    EventID = 0xffff is reserved for time out.				}

{ 4. The sub event naming standard from 0x1000 to 0x6fff: 		}
{      0x1000 ~ 0x1fff    for AI Event Group				}
{         0x1000 ~ 0x17ff for General AI Event ID			}
{         0x1800 ~ 0x18ff for AI others Event ID			}
{      0x2000 ~ 0x2fff    for AO Event Group				}
{         0x2000 ~ 0x27ff for General AO Event ID			}
{         0x2800 ~ 0x28ff for AO others Event ID			}
{      0x3000 ~ 0x3fff    for DI Event Group				}
{         0x3000 ~ 0x30ff for General DI Event ID			}
{         0x3400 ~ 0x34ff for General IDI Event ID			}
{         0x3800 ~ 0x3fff for others Event ID				}
{      0x4000 ~ 0x4fff for DO Event Group				}
{         0x4000 ~ 0x47ff for DO Gernal Event ID			}
{         0x4800 ~ 0x48ff for DO others Event ID			}
{      0x5000 ~ 0x5fff for Counter/Timer Event Group			}
{         0x5000 ~ 0x50ff for Counter TC Event ID			}
{         0x5100 ~ 0x51ff for counter OverFlow Event ID			}
{         0x5200 ~ 0x52ff for counter UnderFlow Event ID		}
{         0x5300 ~ 0x53ff for counter OverCompare Event ID		}
{         0x5400 ~ 0x54ff for counter UnderCompare Event ID		}
{         0x5500 ~ 0x55ff for General Timer Event ID			}
{         0x5800 ~ 0x58ff for Counter/Timer others Event ID		}
{      0x6000 ~ 0x6fff not used at Version 2.0B				}

unit EVENT;
interface
uses windows;
const
{ ==================================					 }
{ 0x0000 and 0xffff}
{ ----------------------------------					 }
  ADS_EVT_NO_EVENT = $0000;
  ADS_EVT_TIME_OUT = $ffff;

{   ==================================                                    }
{ 0x1000 ~ 0x1fff for AI Group                                            }
{   ----------------------------------                                    }
  ADS_EVT_AI_BUFFER1_FULL =     $1800;
  ADS_EVT_AI_BUFFER2_FULL =     $1801;

      ADS_EVT_AI_BUFFER1_FULL_NAME = 'ADS_EVT_AI_BUFFER1_FULL';
      ADS_EVT_AI_BUFFER2_FULL_NAME = 'ADS_EVT_AI_BUFFER2_FULL';
{ ----------------------------------                                      }
{ 0x2000 ~ 0x2fff for AO Group                                            }
{ ----------------------------------                                      }
  { sub group 0x2000 ~ 0x27ff for AO General ID                           }
  ADS_EVT_AO_BUFFER1_EMPTY = $2800;
  ADS_EVT_AO_BUFFER2_EMPTY = $2801;

       ADS_EVT_AO_BUFFER1_FULL_NAME = 'ADS_EVT_AO_BUFFER1_FULL';
       ADS_EVT_AO_BUFFER2_FULL_NAME = 'ADS_EVT_AO_BUFFER2_FULL';

  { sub group 0x2800 ~ 0x2fff for AO other ID                             }

{ ----------------------------------                                      }
{ 0x3000 ~ 0x3fff for DI Group                                            }
{ ----------------------------------                                      }
  { sub group 0x3000 ~ 0x37ff for DI General ID                           }
  ADS_EVT_INTERRUPT_DI0 = $3000;
  ADS_EVT_INTERRUPT_DI1 = ADS_EVT_INTERRUPT_DI0 + 1;
  ADS_EVT_INTERRUPT_DI2 = ADS_EVT_INTERRUPT_DI0 + 2;
  ADS_EVT_INTERRUPT_DI3 = ADS_EVT_INTERRUPT_DI0 + 3;
  ADS_EVT_INTERRUPT_DI4 = ADS_EVT_INTERRUPT_DI0 + 4;
  ADS_EVT_INTERRUPT_DI5 = ADS_EVT_INTERRUPT_DI0 + 5;
  ADS_EVT_INTERRUPT_DI6 = ADS_EVT_INTERRUPT_DI0 + 6;
  ADS_EVT_INTERRUPT_DI7 = ADS_EVT_INTERRUPT_DI0 + 7;

     ADS_EVT_INTERRUPT_DI0_NAME = 'ADS_EVT_INTERRUPT_DI0';
     ADS_EVT_INTERRUPT_DI1_NAME = 'ADS_EVT_INTERRUPT_DI1';
     ADS_EVT_INTERRUPT_DI2_NAME = 'ADS_EVT_INTERRUPT_DI2';
     ADS_EVT_INTERRUPT_DI3_NAME = 'ADS_EVT_INTERRUPT_DI3';
     ADS_EVT_INTERRUPT_DI4_NAME = 'ADS_EVT_INTERRUPT_DI4';
     ADS_EVT_INTERRUPT_DI5_NAME = 'ADS_EVT_INTERRUPT_DI5';
     ADS_EVT_INTERRUPT_DI6_NAME = 'ADS_EVT_INTERRUPT_DI6';
     ADS_EVT_INTERRUPT_DI7_NAME = 'ADS_EVT_INTERRUPT_DI7';

  { sub group 0x3400 ~ 0x37ff for General IDI Event ID 			}

  ADS_EVT_INTERRUPT_IDI0 = $3400;
  ADS_EVT_INTERRUPT_IDI1 = ADS_EVT_INTERRUPT_IDI0 + 1;
  ADS_EVT_INTERRUPT_IDI2 = ADS_EVT_INTERRUPT_IDI0 + 2;
  ADS_EVT_INTERRUPT_IDI3 = ADS_EVT_INTERRUPT_IDI0 + 3;
  ADS_EVT_INTERRUPT_IDI4 = ADS_EVT_INTERRUPT_IDI0 + 4;
  ADS_EVT_INTERRUPT_IDI5 = ADS_EVT_INTERRUPT_IDI0 + 5;
  ADS_EVT_INTERRUPT_IDI6 = ADS_EVT_INTERRUPT_IDI0 + 6;
  ADS_EVT_INTERRUPT_IDI7 = ADS_EVT_INTERRUPT_IDI0 + 7;
  ADS_EVT_INTERRUPT_IDI16= ADS_EVT_INTERRUPT_IDI0 + 16;
  ADS_EVT_INTERRUPT_IDI17= ADS_EVT_INTERRUPT_IDI0 + 17;

     ADS_EVT_INTERRUPT_IDI0_NAME	 =  'ADS_EVT_INTERRUPT_IDI0';
     ADS_EVT_INTERRUPT_IDI1_NAME	 =  'ADS_EVT_INTERRUPT_IDI1';
     ADS_EVT_INTERRUPT_IDI2_NAME	 =  'ADS_EVT_INTERRUPT_IDI2';
     ADS_EVT_INTERRUPT_IDI3_NAME	 =  'ADS_EVT_INTERRUPT_IDI3';
     ADS_EVT_INTERRUPT_IDI4_NAME	 =  'ADS_EVT_INTERRUPT_IDI4';
     ADS_EVT_INTERRUPT_IDI5_NAME	 =  'ADS_EVT_INTERRUPT_IDI5';
     ADS_EVT_INTERRUPT_IDI6_NAME	 =  'ADS_EVT_INTERRUPT_IDI6';
     ADS_EVT_INTERRUPT_IDI7_NAME	 =  'ADS_EVT_INTERRUPT_IDI7';
     ADS_EVT_INTERRUPT_IDI16_NAME        =  'ADS_EVT_INTERRUPT_IDI16';
     ADS_EVT_INTERRUPT_IDI17_NAME        =  'ADS_EVT_INTERRUPT_IDI17';

  { sub group 0x38000 ~ 0x3fff for DI General ID                             }

  ADS_EVT_FIFO_OVERFLOW_DI  =  $3800;
  ADS_EVT_FIFO_UNDERFLOW_DI =  $3801;
  ADS_EVT_BUFFER1_FULL_DI   =  $3802;
  ADS_EVT_BUFFER2_FULL_DI   =  $3803;
  {------------------- V2.0C ------------------ }
  ADS_EVT_DI_LOBUFREADY  =      $3804; 
  ADS_EVT_DI_HIBUFREADY  =      $3805;
  ADS_EVT_DI_TERMINATED  =      $3810;
  ADS_EVT_DI_OVERRUN     =      $3811;
  ADS_EVT_DI_PATTERNMATCH   =   $3812;
  ADS_EVT_DI_STATUSCHANGE   =   $3813; 
  {------------------- V2.0C ------------------ }

     ADS_EVT_FIFO_OVERFLOW_DI_NAME =    'ADS_EVT_FIFO_OVERFLOW_DI';
     ADS_EVT_FIFO_UNDERFLOW_DI_NAME=    'ADS_EVT_FIFO_UNDERFLOW_DI';
     ADS_EVT_BUFFER1_FULL_DI_NAME	 =    'ADS_EVT_BUFFER1_FULL_DI';
     ADS_EVT_BUFFER2_FULL_DI_NAME	 =    'ADS_EVT_BUFFER2_FULL_DI';
     {------------------- V2.0C ------------------ }
     ADS_EVT_DI_LOBUFREADY_NAME  =      'ADS_EVT_DI_LOBUFREADY'; 
     ADS_EVT_DI_HIBUFREADY_NAME  =      'ADS_EVT_DI_HIBUFREADY';
     ADS_EVT_DI_TERMINATED_NAME  =      'ADS_EVT_DI_TERMINATED';
     ADS_EVT_DI_OVERRUN_NAME     =      'ADS_EVT_DI_OVERRUN';
     ADS_EVT_DI_PATTERNMATCH_NAME   =   'ADS_EVT_DI_PATTERNMATCH';
     ADS_EVT_DI_STATUSCHANGE_NAME   =   'ADS_EVT_DI_STATUSCHANGE'; 
     {------------------- V2.0C ------------------ }

{ ----------------------------------                                       }
{ 0x4000 ~ 0x4fff for DO Group                                             }
{ ----------------------------------                                       }
  { sub group 0x4000 ~ 0x47ff for DO General ID                            }

  { sub group 0x4800 ~ 0x4fff for DO others ID                             }

  ADS_EVT_FIFO_OVERFLOW_DO  =  $4800;
  ADS_EVT_FIFO_UNDERFLOW_DO =  $4801;
  ADS_EVT_BUFFER1_EMPTY_DO  =  $4802;
  ADS_EVT_BUFFER2_EMPTY_DO  =  $4803;
  {------------------- V2.0C ------------------ }
  ADS_EVT_DO_LOBUFTRANS      =  $4804;
  ADS_EVT_DO_HIBUFTRANS      =  $4805;
  ADS_EVT_DO_TERMINATED      =  $4810;
  ADS_EVT_DO_UNDERRUN        =  $4811; 
  {------------------- V2.0C ------------------ }

     ADS_EVT_FIFO_OVERFLOW_DO_NAME	=    'ADS_EVT_FIFO_OVERFLOW_DO';
     ADS_EVT_FIFO_UNDERFLOW_DO_NAME	=    'ADS_EVT_FIFO_UNDERFLOW_DO';
     ADS_EVT_BUFFER1_EMPTY_DO_NAME	=    'ADS_EVT_BUFFER1_EMPTY_DO';
     ADS_EVT_BUFFER2_EMPTY_DO_NAME	=    'ADS_EVT_BUFFER2_EMPTY_DO';
     {------------------- V2.0C ------------------ }
     ADS_EVT_DO_LOBUFTRANS_NAME      =  'ADS_EVT_DO_LOBUFTRANS';
     ADS_EVT_DO_HIBUFTRANS_NAME      =  'ADS_EVT_DO_HIBUFTRANS';
     ADS_EVT_DO_TERMINATED_NAME      =  'ADS_EVT_DO_TERMINATED';
     ADS_EVT_DO_UNDERRUN_NAME        =  'ADS_EVT_DO_UNDERRUN'; 
     {------------------- V2.0C ------------------ }

{ ---------------------------------------                                  }
{ 0x5000 ~ 0x5fff for Counter/Timer                                        }
{ ---------------------------------------                                  }
  { sub group 0x5000 ~ 0x57ff for Counter/Timer general ID                 }

  ADS_EVT_TERMINATE_CNT0  =    $5000;
  ADS_EVT_TERMINATE_CNT1  =    ADS_EVT_TERMINATE_CNT0 + 1;
  ADS_EVT_TERMINATE_CNT2  =    ADS_EVT_TERMINATE_CNT0 + 2;
  ADS_EVT_TERMINATE_CNT3  =    ADS_EVT_TERMINATE_CNT0 + 3;
  ADS_EVT_TERMINATE_CNT4  =    ADS_EVT_TERMINATE_CNT0 + 4;
  ADS_EVT_TERMINATE_CNT5  =    ADS_EVT_TERMINATE_CNT0 + 5;
  ADS_EVT_TERMINATE_CNT6  =    ADS_EVT_TERMINATE_CNT0 + 6;
  ADS_EVT_TERMINATE_CNT7  =    ADS_EVT_TERMINATE_CNT0 + 7;

     ADS_EVT_TERMINATE_CNT0_NAME  =       'ADS_EVT_TERMINATE_CNT0';
     ADS_EVT_TERMINATE_CNT1_NAME  =       'ADS_EVT_TERMINATE_CNT1';
     ADS_EVT_TERMINATE_CNT2_NAME  =       'ADS_EVT_TERMINATE_CNT2';
     ADS_EVT_TERMINATE_CNT3_NAME  =       'ADS_EVT_TERMINATE_CNT3';
     ADS_EVT_TERMINATE_CNT4_NAME  =       'ADS_EVT_TERMINATE_CNT4';
     ADS_EVT_TERMINATE_CNT5_NAME  =       'ADS_EVT_TERMINATE_CNT5';
     ADS_EVT_TERMINATE_CNT6_NAME  =       'ADS_EVT_TERMINATE_CNT6';
     ADS_EVT_TERMINATE_CNT7_NAME  =       'ADS_EVT_TERMINATE_CNT7';

  { sub group 0x5100 ~ 0x51ff for counter overflow Event ID          }

  ADS_EVT_OVERFLOW_CNT0  =     $5100;
  ADS_EVT_OVERFLOW_CNT1  =     ADS_EVT_OVERFLOW_CNT0 + 1;
  ADS_EVT_OVERFLOW_CNT2  =     ADS_EVT_OVERFLOW_CNT0 + 2;
  ADS_EVT_OVERFLOW_CNT3  =     ADS_EVT_OVERFLOW_CNT0 + 3;
  ADS_EVT_OVERFLOW_CNT4  =     ADS_EVT_OVERFLOW_CNT0 + 4;
  ADS_EVT_OVERFLOW_CNT5  =     ADS_EVT_OVERFLOW_CNT0 + 5;
  ADS_EVT_OVERFLOW_CNT6  =     ADS_EVT_OVERFLOW_CNT0 + 6;
  ADS_EVT_OVERFLOW_CNT7  =     ADS_EVT_OVERFLOW_CNT0 + 7;

    ADS_EVT_OVERFLOW_CNT0_NAME =  'ADS_EVT_OVERFLOW_CNT0';
    ADS_EVT_OVERFLOW_CNT1_NAME =  'ADS_EVT_OVERFLOW_CNT1';
    ADS_EVT_OVERFLOW_CNT2_NAME =  'ADS_EVT_OVERFLOW_CNT2';
    ADS_EVT_OVERFLOW_CNT3_NAME =  'ADS_EVT_OVERFLOW_CNT3';
    ADS_EVT_OVERFLOW_CNT4_NAME =  'ADS_EVT_OVERFLOW_CNT4';
    ADS_EVT_OVERFLOW_CNT5_NAME =  'ADS_EVT_OVERFLOW_CNT5';
    ADS_EVT_OVERFLOW_CNT6_NAME =  'ADS_EVT_OVERFLOW_CNT6';
    ADS_EVT_OVERFLOW_CNT7_NAME =  'ADS_EVT_OVERFLOW_CNT7';

  { sub group 0x5200 ~ 0x52ff for counter underflow Event ID   }

  ADS_EVT_UNDERFLOW_CNT0   =   $5200;
  ADS_EVT_UNDERFLOW_CNT1   =   ADS_EVT_UNDERFLOW_CNT0 + 1;
  ADS_EVT_UNDERFLOW_CNT2   =   ADS_EVT_UNDERFLOW_CNT0 + 2;
  ADS_EVT_UNDERFLOW_CNT3   =   ADS_EVT_UNDERFLOW_CNT0 + 3;
  ADS_EVT_UNDERFLOW_CNT4   =   ADS_EVT_UNDERFLOW_CNT0 + 4;
  ADS_EVT_UNDERFLOW_CNT5   =   ADS_EVT_UNDERFLOW_CNT0 + 5;
  ADS_EVT_UNDERFLOW_CNT6   =   ADS_EVT_UNDERFLOW_CNT0 + 6;
  ADS_EVT_UNDERFLOW_CNT7   =   ADS_EVT_UNDERFLOW_CNT0 + 7;

     ADS_EVT_UNDERFLOW_CNT0_NAME   =      'ADS_EVT_UNDERFLOW_CNT0';
     ADS_EVT_UNDERFLOW_CNT1_NAME   =      'ADS_EVT_UNDERFLOW_CNT1';
     ADS_EVT_UNDERFLOW_CNT2_NAME   =      'ADS_EVT_UNDERFLOW_CNT2';
     ADS_EVT_UNDERFLOW_CNT3_NAME   =      'ADS_EVT_UNDERFLOW_CNT3';
     ADS_EVT_UNDERFLOW_CNT4_NAME   =      'ADS_EVT_UNDERFLOW_CNT4';
     ADS_EVT_UNDERFLOW_CNT5_NAME   =      'ADS_EVT_UNDERFLOW_CNT5';
     ADS_EVT_UNDERFLOW_CNT6_NAME   =      'ADS_EVT_UNDERFLOW_CNT6';
     ADS_EVT_UNDERFLOW_CNT7_NAME   =      'ADS_EVT_UNDERFLOW_CNT7';

  { sub group 0x5300 ~ 0x53ff for counter OVER COMPARE Event ID }

  { sub group 0x5400 ~ 0x54ff for counter UNDER COMPARE Event ID}

  { sub group 0x5500 ~ 0x55ff for GENERAL TIMER Event ID     }

  ADS_EVT_INTERRUPT_TIMER0  =  $5500;
  ADS_EVT_INTERRUPT_TIMER1  =  ADS_EVT_INTERRUPT_TIMER0 + 1;
  ADS_EVT_INTERRUPT_TIMER2  =  ADS_EVT_INTERRUPT_TIMER0 + 2;
  ADS_EVT_INTERRUPT_TIMER3  =  ADS_EVT_INTERRUPT_TIMER0 + 3;
  ADS_EVT_INTERRUPT_TIMER4  =  ADS_EVT_INTERRUPT_TIMER0 + 4;
  ADS_EVT_INTERRUPT_TIMER5  =  ADS_EVT_INTERRUPT_TIMER0 + 5;
  ADS_EVT_INTERRUPT_TIMER6  =  ADS_EVT_INTERRUPT_TIMER0 + 6;
  ADS_EVT_INTERRUPT_TIMER7  =  ADS_EVT_INTERRUPT_TIMER0 + 7;

  ADS_EVT_TC_TIMER          =   $5200;

     ADS_EVT_INTERRUPT_TIMER0_NAME  =       'ADS_EVT_INTERRUPT_TIMER0';
     ADS_EVT_INTERRUPT_TIMER1_NAME  =       'ADS_EVT_INTERRUPT_TIMER1';
     ADS_EVT_INTERRUPT_TIMER2_NAME  =       'ADS_EVT_INTERRUPT_TIMER2';
     ADS_EVT_INTERRUPT_TIMER3_NAME  =       'ADS_EVT_INTERRUPT_TIMER3';
     ADS_EVT_INTERRUPT_TIMER4_NAME  =       'ADS_EVT_INTERRUPT_TIMER4';
     ADS_EVT_INTERRUPT_TIMER5_NAME  =       'ADS_EVT_INTERRUPT_TIMER5';
     ADS_EVT_INTERRUPT_TIMER6_NAME  =       'ADS_EVT_INTERRUPT_TIMER6';
     ADS_EVT_INTERRUPT_TIMER7_NAME  =       'ADS_EVT_INTERRUPT_TIMER7';
     ADS_EVT_TC_TIMER_NAME          =       'ADS_EVT_TC_TIMER';

  { sub group 0x5800 ~ 0x5fff for counter/Timer general ID                 }
implementation

end.










