{ *************************************************************************** }
{ Module Name: Paras.PAS                                                      }
{ Purpose: the declaration of functions, data structures, status codes,       }
{         constants, and messages                                             }
{ Version: 3.01                                                               }
{ Date: 01/08/2003                                                            }
{ Copyright (c) 1996-2003 Advantech Corp. Ltd.                                }
{ All rights reserved.                                                        }
{ *************************************************************************** }


{ Notes:                                                                      }
{ 1. From 0x0000 to 0x0FFF : General definition
{ 2. From 0x1000 to 0x1FFF : AI related                                       }
{ 3. From 0x2000 to 0x2FFF : AO related                                       }
{ 4. From 0x3000 to 0x3FFF : DI related                                       }
{ 5. From 0x4000 to 0x4FFF : DO related                                       }
{ 6. From 0x5000 to 0x5FFF : Counter/Timer related                            }

unit PARAS;
interface
const
{ ************************************************************************ }
{   Constant Definition                                                    }
{ ************************************************************************ }
   CFG_DeviceNumber    = $0000;
   CFG_BoardID         = $0001;
   CFG_SwitchID        = $0002;
   CFG_BaseAddress     = $0003;
   CFG_Interrupt       = $0004;
   CFG_BusNumber       = $0005;
   CFG_SlotNumber      = $0006;
   CFG_OnSystem        = $0007;


{   ==================================                                     }
{ 0x1000 ~ 0x1fff for AI Group                                             }
{   ----------------------------------                                     }

{ ----------------------------------                                       }
{ 0x2000 ~ 0x2fff for AO Group                                             }
{ ----------------------------------                                       }
{ sub group 0x2000 ~ 0x27ff for AO General ID                              }
     AO_RANGE_SETTING = $2000;

         AO_RANGE_SETTINGL_NAME = 'AO_RANGE_SETTING';

{ sub group 0x2800 ~ 0x2fff for AO other ID                                }


{ ----------------------------------                                       }
{ 0x3000 ~ 0x3fff for DI Group                                             }
{ ----------------------------------                                       }
{ sub group 0x3000 ~ 0x37ff for DI General ID                              }
  CFG_DioDirection       = $3000;     { each bit for one Auxiliary DIO channel for PCI-1755.}
  CFG_DioFdioDirection   = $3001;     { 32DI(0)/32DO(1)/ 16 DIO(2) / 8DIO(3) for PCI-1755}
  CFG_DioTerminator      = $3002;     { Bit 0: DI terminator (DI_TERM),ON (0) or OFF(1) for PCI-1755                                                      // Bit 1: DO terminator (DI_TERM) ON (0) or OFF(1) for PCI-1755}
  CFG_DiOperationMode    = $3003;     { normal mode (0), 8255 handshaking(1), Burst handshaking(2) for PCI-1755}


{ sub group 0x3800 ~ 0x3fff for DI General ID                             }
  CFG_DiStartMethod     = $3800;     { Software(1), External trigger(2), Pattern match(3)}
  CFG_DiStopMethod      = $3801;     { Software(1), External trigger(2), Pattern match(3)}
  CFG_DiPacerSource     = $3802;     { 30MHz(1), 15Mhz(2), 10MHz(3), Counter 0 OUT (4), External (5).}
  CFG_DiControlSignals  = $3803;     { Bit 0: External DI start signal control (STRRF), 0 rising edge, 1 falling edge. }
                                    { Bit 1: External DI stop signal control (STPRF), 0 rising edge, 1 falling edge.}
                                    { Bit 2: DI request signal control (REQRF), 0 rising edge, 1 falling edge.}
                                    { Bit 3: DI acknowledge signal control (ACKRF), 0 rising edge, 1 falling edge.}
                                    { Bit 4: DI sampling clock signal control (CLKRF), 0 rising edge, 1 falling edge}
  CFG_DiPatternMatchValue = $3804;

{ ----------------------------------                                       }
{ 0x4000 ~ 0x4fff for DO Group                                             }
{ ----------------------------------                                       }
{ sub group 0x4000 ~ 0x47ff for DO General ID                              }

{ sub group 0x4800 ~ 0x4fff for DO others ID                               }
  CFG_DoOperationMode   = $4800;     { Normal (0), Handahaking (1), Burst handshaking(2)}
  CFG_DoStartMethod     = $4801;     { Software(1), External signal(2)}
  CFG_DoStopMethod      = $4802;     { Software(1), External signal(2)}
  CFG_DoPacerSource     = $4803;     { 30, 15, 10 Mhz, Counter1, External signal (value range 1-5)}
  CFG_DoControlSignals  = $4804;     { Control signals }
                                         {   Bit 0: External DO start signal control (STRRF), 0 rising edge, 1 falling edge. }
                                         {   Bit 1: External DO stop signal control (STPRF), 0 rising edge, 1 falling edge.}
                                         {   Bit 2: DO request signal control (REQRF), 0 rising edge, 1 falling edge.}
                                         {   Bit 3: DO acknowledge signal control (ACKRF), 0 rising edge, 1 falling edge.}
                                         {   Bit 4: DO sampling clock signal control (CLKRF), 0 rising edge, 1 falling edge}
  CFG_DoPresetValue     = $4805;     { DO status before transfering.}

{ ---------------------------------------                                  }
{ 0x5000 ~ 0x5fff for Counter/Timer                                        }
{ ---------------------------------------                                  }
{ sub group 0x5000 ~ 0x57ff for Counter/Timer general ID                   }

  CFG_CounterCountValue = $5000;   { 82C54 Counter 0-2 value}

{ sub group 0x5800 ~ 0x5fff for counter/Timer general ID                   }

{ Other ID }
  CFG_IrqDiChangeStatusChannel  = $6000;       { Change status channel number.}
  CFG_IrqDiTriggerSignals       = $6001;       { Each bit for each Auxiliary DIO channel}

implementation

end.
